#include <math.h>
/**********************************************************************/
      int lufpdc(double a[],int ll[],int n,int *pnsd,int nn) {
/*    int lufpdc(a,ll,n,pnsd,nn)
      double a[]; int ll[],n,*pnsd,nn; {                              */
/*    ============================================================    */
      double add,aij,akj,akk,eps=1e-7;
      int    i,j,k,ik,jk,nsd;
      int    *p;                     /* a[i+p[j]] = a(i,j) */
      --a; --ll; ll-=2;              /* dimension ll(2,n)  */
      p=(int *)malloc(n*sizeof(int));
      --p; for(i=1; i<=n; i++) p[i]=i*nn-nn;
/*    ============================================================    */
/*    find [l] & [u] for given [a], such that  [r][a][c] = [l][u].    */
/*    where [c] is col_exchanges matrix indicated by ll(1,k)=jk       */
/*    where [r] is row_exchanges matrix indicated by ll(2,k)=ik       */
/*    ll(1,k)=jk : col_exchange between k_th_col & jk_th_col          */
/*    ll(2,k)=ik : row_exchange between k_th_row & ik_th_row          */
/*o   nsd = number of singular degrees of the given matrix            */
/*    ============================================================    */
      nsd=0;
      for(k=1; k<=n; k++) {
/*    +----------------------------------------------------------+    */
/*    | find ik, jk, akk=|a(ik,jk)|=max(|a(i,j)|, i=k,n & j=k,n) |    */
/*    | a(i,j)=a(i,j)-\sum_{k=1)^{k-1} l(i,k)*u(k,j)             |    */
/*    +----------------------------------------------------------+    */
        akk=0.0;
        for(j=k; j<=n; j++) {
          for(i=k; i<=n; i++) {
            if(akk >= fabs(a[i+p[j]])) continue;
            akk=fabs(a[i+p[j]]);
            ik=i;
            jk=j;
          }
        }
        if(k == 1) add=akk;
        if(akk <= add*eps) goto L90;
/*      +--------------------------------------------------------+    */
/*      | set ll(1,k)=jk  &  exchange k_th_col & jk_th_col       |    */
/*      | set ll(2,k)=ik  &  exchange k_th_row & ik_th_row       |    */
/*      | to save time no exchange to l(i,1:(k-1)), u(1:(k-1),j) |    */
/*      +--------------------------------------------------------+    */
        ll[1+k*2]=jk;
        if(k != jk) {
          j=jk;
          for(i=k; i<=n; i++) {
            akj=a[i+p[j]];
            a[i+p[j]]=a[i+p[k]];
            a[i+p[k]]=akj;
          }
        }
        ll[2+k*2]=ik;
        if(k != ik) {
          i=ik;
          for(j=k; j<=n; j++) {
            akj=a[i+p[j]];
            a[i+p[j]]=a[k+p[j]];
            a[k+p[j]]=akj;
          }
        }
/*      +--------------------------------------------------------+    */
/*      | a(i,j)=a(i,j) - l(i,k)*u(k,j)  for i=k+1,n & j=k+1,n   |    */
/*      | a(i,k)=l(i,k),  a(k,j)=l(k,k)*u(k,j) --> u(k,j)=akj    |    */
/*      +--------------------------------------------------------+    */
        if(k == n) goto L99;
        for(j=k+1; j<=n; j++) {
          akj=a[k+p[j]]/a[k+p[k]];
          for(i=k+1; i<=n; i++) {
            a[i+p[j]]=a[i+p[j]]-a[i+p[k]]*akj;
          }
          a[k+p[j]]=akj;
        }
      }
/*    +----------------------------------------------------------+    */
/*    | return(0) on singular matrix : l(k,k)=0                  |    */
/*    +----------------------------------------------------------+    */
  L90:nsd=n-k+1;
      if(add == 0.0) add=1.0;
      for(k=n-nsd+1; k<=n; k++) {
        ll[1+k*2]=k;
        ll[2+k*2]=k;
        a[k+p[k]]=fabs(a[k+p[k]]);
        if(a[k+p[k]] < add*eps*eps) a[k+p[k]]=add*eps*eps;
      }
  L99:free(++p); *pnsd=nsd; return(0);
}
/**********************************************************************/
      int lufpsb(double a[],double b[],int ll[],int n,int nn) {
/*    int lufpsb(a,b,ll,n,nn)
      double a[],b[]; int ll[],n,nn; {                                */
/*    ============================================================    */
      double bk;
      int    i,j,k,ik,jk;
      int    *p;                     /* a[i+p[j]] = a(i,j) */
      --a; --b; --ll; ll-=2;         /* dimension ll(2,n)  */
      p=(int *)malloc(n*sizeof(int));
      --p; for(i=1; i<=n; i++) p[i]=i*nn-nn;
/*    ============================================================    */
/*    find {x} from [a]{x}={b} for given {b} & [r][a][c]=[l][u].      */
/*    [c] is given by ll(1,n),   [r] is given by ll(2,n),             */
/*    [l] & [u] are given & put in a(nn,n),    [c]'=inverse of [c]    */
/*    [l][u][c]'{x}=[r]{b} ===> [l]{y}=[r]{b}, [u][c]'{x}={y}         */
/*    ============================================================    */
/*    +----------------------------------------------------------+    */
/*    | find {y} from [r]{b} by forward subst. of [l]{y}=[r]{b}  |    */
/*    | b(k) <==> b(ll(2,k));         y(k)=bk=b(ll(2,k))/l(k,k)  |    */
/*    | subst. y(k) to eq.i=(k+1):n : b(i)=b(i)-l(i,k)*y(k)      |    */
/*    | a(i,k)=l(i,k), a(k,k)=l(k,k), b(i)=b(i) --> l(i,i)*y(i)  |    */
/*    +----------------------------------------------------------+    */
      for(k=1; k<=n; k++) {
        ik=ll[2+k*2];
        bk=b[ik]/a[k+p[k]];
        b[ik]=b[k];
        for(i=k+1; i<=n; i++) {
          b[i]=b[i]-a[i+p[k]]*bk;
        }
        b[k]=bk;
      }
/*    +----------------------------------------------------------+    */
/*    | find {x} from {y} by backward subst. of [u][c]'{x}={y}   |    */
/*    | x(k) = bk = y(k) - \sum_{j=k+1}^{n} u(k,j)*x(j)          |    */
/*    | a(k,j)=u(k,j), b(j)=x(j);        x(k) <==> x(ll(1,k))    |    */
/*    +----------------------------------------------------------+    */
      for(k=n-1; k>=1; k--) {
        jk=ll[1+k*2];
        bk=b[k];
        b[k]=b[jk];
        for(j=k+1; j<=n; j++) {
          bk=bk-a[k+p[j]]*b[j];
        }
        b[jk]=bk;
      }
      free(++p); return(0);
}
/**********************************************************************/
      int lufpbs(double a[],double b[],int ll[],int n,int nn) {
/*    int lufpbs(a,b,ll,n,nn)
      double a[],b[]; int ll[],n,nn; {                                */
/*    ============================================================    */
      double bk;
      int    i,j,k,jk;
      int    *p;                     /* a[i+p[j]] = a(i,j) */
      --a; --b; --ll; ll-=2;         /* dimension ll(2,n)  */
      p=(int *)malloc(n*sizeof(int));
      --p; for(i=1; i<=n; i++) p[i]=i*nn-nn;
/*    ============================================================    */
/*    +----------------------------------------------------------+    */
/*    | find {x} from {y} by backward subst. of [u][c]'{x}={y}   |    */
/*    | x(k) = bk = y(k) - \sum_{j=k+1}^{n} u(k,j)*x(j)          |    */
/*    | a(k,j)=u(k,j), b(j)=x(j);        x(k) <==> x(ll(1,k))    |    */
/*    +----------------------------------------------------------+    */
      for(k=n-1; k>=1; k--) {
        jk=ll[1+k*2];
        bk=b[k];
        b[k]=b[jk];
        for(j=k+1; j<=n; j++) {
          bk=bk-a[k+p[j]]*b[j];
        }
        b[jk]=bk;
      }
      free(++p); return(0);
}
/**********************************************************************/
      int lufpsg(double a[],double b[],int ll[],int n,int nsd,int nn) {
/*    int lufpsg(a,b,ll,n,nsd,nn)
      double a[],b[]; int ll[],n,nsd,nn; {                            */
/*    ============================================================    */
      double bi;
      int    i,j;
      int    *p;                     /* b[i+p[j]] = b(i,j) */
      --a; --b;
      p=(int *)malloc(n*sizeof(int));
      --p; for(i=1; i<=n; i++) p[i]=i*nn-nn;
/*    ============================================================    */
/*    find the solution vectors [b] of the singular matrix [a]        */
/*    ------------------------------------------------------------    */
/*o   b(n,nsd) = solution vectors of singular matrix a(n,n)           */
/*    ============================================================    */
      for(j=1; j<=nsd; j++) {
        for(i=1; i<=n; i++) {
          b[i+p[j]]=0.0;
        }
        b[n-nsd+j+p[j]]=1.0;
        lufpbs(&a[1],&b[1+p[j]],ll,n,nn);
      }
      free(++p); return(0);
}
/**********************************************************************/
