#include <math.h>
/**********************************************************************/
      int lufpdc(double a[],int ll[],int n,int *pnsd,int nn) {
/*    int lufpdc(a,ll,n,pnsd,nn)
      double a[]; int ll[],n,*pnsd,nn; {                              */
/*    ============================================================    */
      double add,aij,akj,akk,eps=1e-7;
      int    i,j,k,ik,jk,nsd;
      int    *p;                     /* a[i+p[j]] = a(i,j) */
      --a; --ll; ll-=2;              /* dimension ll(2,n)  */
      p=(int *)malloc(n*sizeof(int));
      --p; for(i=1; i<=n; i++) p[i]=i*nn-nn;
/*    ============================================================    */
/*    find [l] & [u] for given [a], such that  [r][a][c] = [l][u].    */
/*    where [c] is col_exchanges matrix indicated by ll(1,k)=jk       */
/*    where [r] is row_exchanges matrix indicated by ll(2,k)=ik       */
/*    ll(1,k)=jk : col_exchange between k_th_col & jk_th_col          */
/*    ll(2,k)=ik : row_exchange between k_th_row & ik_th_row          */
/*o   nsd = number of singular degrees of the given matrix            */
/*    ============================================================    */
      nsd=0;
      for(k=1; k<=n; k++) {
/*    +----------------------------------------------------------+    */
/*    | find ik, jk, akk=|a(ik,jk)|=max(|a(i,j)|, i=k,n & j=k,n) |    */
/*    | a(i,j)=a(i,j)-\sum_{k=1)^{k-1} l(i,k)*u(k,j)             |    */
/*    +----------------------------------------------------------+    */
        akk=0.0;
        for(j=k; j<=n; j++) {
          for(i=k; i<=n; i++) {
            if(akk >= fabs(a[i+p[j]])) continue;
            akk=fabs(a[i+p[j]]);
            ik=i;
            jk=j;
          }
        }
        if(k == 1) add=akk;
        if(akk <= add*eps) goto L90;
/*      +--------------------------------------------------------+    */
/*      | set ll(1,k)=jk  & exchange k_th_col & jk_th_col        |    */
/*      | set ll(2,k)=ik  & exchange k_th_row & ik_th_row        |    */
/*      +--------------------------------------------------------+    */
        ll[1+k*2]=jk;
        if(k != jk) {
          j=jk;
          for(i=1; i<=n; i++) {
            akj=a[i+p[j]];
            a[i+p[j]]=a[i+p[k]];
            a[i+p[k]]=akj;
          }
        }
        ll[2+k*2]=ik;
        if(k != ik) {
          i=ik;
          for(j=1; j<=n; j++) {
            akj=a[i+p[j]];
            a[i+p[j]]=a[k+p[j]];
            a[k+p[j]]=akj;
          }
        }
/*      +--------------------------------------------------------+    */
/*      | a(i,j)=a(i,j) - l(i,k)*u(k,j)  for i=k+1,n & j=k+1,n   |    */
/*      | a(i,k)=l(i,k),  a(k,j)=l(k,k)*u(k,j) --> u(k,j)=akj    |    */
/*      +--------------------------------------------------------+    */
        if(k == n) goto L99;
        for(j=k+1; j<=n; j++) {
          akj=a[k+p[j]]/a[k+p[k]];
          for(i=k+1; i<=n; i++) {
            a[i+p[j]]=a[i+p[j]]-a[i+p[k]]*akj;
          }
          a[k+p[j]]=akj;
        }
      }
/*    +----------------------------------------------------------+    */
/*    | return(0) on singular matrix : l(k,k)=0                  |    */
/*    +----------------------------------------------------------+    */
  L90:nsd=n-k+1;
      if(add == 0.0) add=1.0;
      for(k=n-nsd+1; k<=n; k++) {
        ll[1+k*2]=k;
        ll[2+k*2]=k;
        a[k+p[k]]=fabs(a[k+p[k]]);
        if(a[k+p[k]] < add*eps*eps) a[k+p[k]]=add*eps*eps;
      }
  L99:free(++p); *pnsd=nsd; return(0);
}
/**********************************************************************/
      int lufpsb(double a[],double b[],int ll[],int n,int nn) {
/*    int lufpsb(a,b,ll,n,nn)
      double a[],b[]; int ll[],n,nn; {                                */
/*    ============================================================    */
      double bi;
      int    i,k,m;
      int    *p;                     /* a[i+p[j]] = a(i,j) */
      --a; --b; --ll; ll-=2;         /* dimension ll(2,n)  */
      p=(int *)malloc(n*sizeof(int));
      --p; for(i=1; i<=n; i++) p[i]=i*nn-nn;
/*    ============================================================    */
/*    find {x} from [a]{x}={b} for given {b} & [r][a][c]=[l][u].      */
/*    [c] is given by ll(1,n),   [r] is given by ll(2,n),             */
/*    [l] & [u] are given & put in a(nn,n),  [c]'=inverse of [c]      */
/*    [l][u][c]'{x}=[r]{b} => [l]{z}=[r]{b}, [u]{y}={z}, {x}=[c]{y}   */
/*    ============================================================    */
/*    +----------------------------------------------------------+    */
/*    | find {z} from [r]{b} by forward subst. of [l]{z}=[r]{b}  |    */
/*    | b(i) <==> b(ll(2,i))                                     |    */
/*    | bi = b(i) - \sum_{k=1}^{i-1} l(i,k)*z(k)                 |    */
/*    | a(i,k)=l(i,k), b(k)=z(k), b(i)=b(i) --> z(i)=bi/l(i,i)   |    */
/*    +----------------------------------------------------------+    */
      for(i=1; i<=n; i++) {
        m=ll[2+i*2];
        bi=b[m];
        b[m]=b[i];
        for(k=1; k<=i-1; k++) {
          bi=bi-a[i+p[k]]*b[k];
        }
        b[i]=bi/a[i+p[i]];
      }
/*    +----------------------------------------------------------+    */
/*    | find {y} from {z} by backward substitution of [u]{y}={z} |    */
/*    | y(i) = bi = z(i) - \sum_{k=i+1}^{n} u(i,k)*y(k)          |    */
/*    | a(i,k)=u(i,k), b(k)=y(k), b(i)=z(i) --> y(i)=bi          |    */
/*    +----------------------------------------------------------+    */
      for(i=n-1; i>=1; i--) {
        bi=b[i];
        for(k=i+1; k<=n; k++) {
          bi=bi-a[i+p[k]]*b[k];
        }
        b[i]=bi;
      }
/*    +----------------------------------------------------------+    */
/*    | find {x} from {y} by row_exchanges : {x}=[c]{y}          |    */
/*    +----------------------------------------------------------+    */
      for(i=n-1; i>=1; i--) {
        m=ll[1+i*2];
        bi=b[i];
        b[i]=b[m];
        b[m]=bi;
      }
      free(++p); return(0);
}
/**********************************************************************/
      int lufpbs(double a[],double b[],int ll[],int n,int nn) {
/*    int lufpbs(a,b,ll,n,nn)
      double a[],b[]; int ll[],n,nn; {                                */
/*    ============================================================    */
      double bi;
      int    i,k,m;
      int    *p;                     /* a[i+p[j]] = a(i,j) */
      --a; --b; --ll; ll-=2;         /* dimension ll(2,n)  */
      p=(int *)malloc(n*sizeof(int));
      --p; for(i=1; i<=n; i++) p[i]=i*nn-nn;
/*    ============================================================    */
/*    +----------------------------------------------------------+    */
/*    | find {y} from {z} by backward substitution of [u]{y}={z} |    */
/*    | y(i) = bi = z(i) - \sum_{k=i+1}^{n} u(i,k)*y(k)          |    */
/*    | a(i,k)=u(i,k), b(k)=y(k), b(i)=z(i) --> y(i)=bi          |    */
/*    +----------------------------------------------------------+    */
      for(i=n-1; i>=1; i--) {
        bi=b[i];
        for(k=i+1; k<=n; k++) {
          bi=bi-a[i+p[k]]*b[k];
        }
        b[i]=bi;
      }
/*    +----------------------------------------------------------+    */
/*    | find {x} from {y} by row_exchanges : {x}=[c]{y}          |    */
/*    +----------------------------------------------------------+    */
      for(i=n-1; i>=1; i--) {
        m=ll[1+i*2];
        bi=b[i];
        b[i]=b[m];
        b[m]=bi;
      }
      free(++p); return(0);
}
/**********************************************************************/
      int lufpsg(double a[],double b[],int ll[],int n,int nsd,int nn) {
/*    int lufpsg(a,b,ll,n,nsd,nn)
      double a[],b[]; int ll[],n,nsd,nn; {                            */
/*    ============================================================    */
      double bi;
      int    i,j;
      int    *p;                     /* b[i+p[j]] = b(i,j) */
      --a; --b;
      p=(int *)malloc(n*sizeof(int));
      --p; for(i=1; i<=n; i++) p[i]=i*nn-nn;
/*    ============================================================    */
/*    find the solution vectors [b] of the singular matrix [a]        */
/*    ------------------------------------------------------------    */
/*o   b(n,nsd) = solution vectors of singular matrix a(n,n)           */
/*    ============================================================    */
      for(j=1; j<=nsd; j++) {
        for(i=1; i<=n; i++) {
          b[i+p[j]]=0.0;
        }
        b[n-nsd+j+p[j]]=1.0;
        lufpbs(&a[1],&b[1+p[j]],ll,n,nn);
      }
      free(++p); return(0);
}
/**********************************************************************/
