/**********************************************************************/
      double xzero(double x,double f,double dx,int istep,double b[]) {
/*    double xzero(x,f,dx,istep,b)
      double x,f,dx,b[]; int istep; {                                 */
/*    ============================================================    */
      double x0,x1,x2,f0,f1,f2,xp,xn,fp,fn,st,u1,u2,xx;
      int    iget;
/*    ============================================================    */
/*i   x     = argument of function                                    */
/*i   f     = function value of x, f(x)                               */
/*o   xzero = a better guess for f(xzero)=0                           */
/*i   dx    = in case no better formula to guess xzero,               */
/*            this function will return :                             */
/*            either xzero = x + dx, if dx  !=  0                     */
/*            or     xzero = x + f,  if dx  ==  0                     */
/*                   this will simulate xzero = g(x)                  */
/*                   as in the direct substitution method             */
/*                   if set f(x) = g(x) - x                           */
/*i   istep = 1 : for the first call of a new function                */
/*          > 1 : otherwise                                           */
/*                                                                    */
/*    for istep = 1 :                                                 */
/*    xzero = x + dx  or  xzero = x + f                               */
/*                                                                    */
/*    for istep > 1 :                                                 */
/*    xzero by secant method or                                       */
/*          by false position method (if applicable)                  */
/*          if xzero by secant method goes outside (xp,xn)            */
/*    ============================================================    */
      if (istep == 1) {
        x1=0.0; x2=0.0; f1=0.0; f2=0.0;
        xp=0.0; xn=0.0; fp=-1.0; fn=1.0; st=0;
      } else {
        x1=b[1]; x2=b[2]; f1=b[4]; f2=b[5];
        xp=b[6]; xn=b[7]; fp=b[8]; fn=b[9]; st=b[10];
      }
      st=st+1;
/*    +----------------------------------------------------------+    */
/*    | save the most newly 3 points                             |    */
/*    +----------------------------------------------------------+    */
      x0=x1; x1=x2; x2=x; f0=f1; f1=f2; f2=f;
/*    +----------------------------------------------------------+    */
/*    | find new points (xp,xn) that bracket the root            |    */
/*    +----------------------------------------------------------+    */
      if (f < 0.0) {xn=x; fn=f;}
      else         {xp=x; fp=f;}
      iget=0;
      xx=0.0;
/*    +----------------------------------------------------------+    */
/*    | for slow convergence due to multiple roots               |    */
/*    | then use secant method on u(x)=f(x)/f'(x)=0              |    */
/*    | after 13 iterations (13 in statement below, >=3)         |    */
/*    +----------------------------------------------------------+    */
      if (st >= 13) {
        if ((f2-f1) != 0.0 && (f1-f0) != 0.0) {
          u1=f1*(x1-x0)/(f1-f0);
          u2=f2*(x2-x1)/(f2-f1);
          if ((u2-u1) != 0.0) {
            xx=x2-u2*(x2-x1)/(u2-u1);
            iget=4;
          }
        }
      }
/*    +----------------------------------------------------------+    */
/*    | if we have at least 2 functions and no xx get            |    */
/*    | then use secant method on f(x)=0                         |    */
/*    +----------------------------------------------------------+    */
      if (st >= 2 && iget == 0) {
        if ((f2-f1) != 0.0) {
          xx=x2-f2*(x2-x1)/(f2-f1);
          iget=3;
        }
      }
/*    +----------------------------------------------------------+    */
/*    | if root is bracketed by (xp,xn)                          |    */
/*    | and if no xx get or xx goes outside (xp,xn)              |    */
/*    | then use false position method by (xp,xn)                |    */
/*    +----------------------------------------------------------+    */
      if (fp >= 0.0 && fn < 0.0) {
        if (iget == 0 || (xx-xp)*(xx-xn) > 0.0) {
          xx=xp-fp*(xp-xn)/(fp-fn);
          iget=2;
        }
      }
/*    +----------------------------------------------------------+    */
/*    | if secant method & false position method cannot be used: |    */
/*    | if dx != 0 : arbitrary set xx=x+dx                       |    */
/*    |    else    : set xx=x+f(x)                               |    */
/*    |              to simulate direct substitution method      |    */
/*    |              if set f(x)=g(x)-x in calling program       |    */
/*    +----------------------------------------------------------+    */
      if (iget == 0) {
        if (dx != 0.0) xx=x+dx;
        else           xx=x+f;
        iget=1;
      }
/*    +----------------------------------------------------------+    */
/*    | for debug                                                |    */
/*    +----------------------------------------------------------+    */
      x0=iget;
      f0=xx;
/*    ------------------------------------------------------------    */
      b[0]=x0; b[1]=x1; b[2]=x2; b[3]=f0; b[4]=f1; b[5]=f2;
      b[6]=xp; b[7]=xn; b[8]=fp; b[9]=fn; b[10]=st;
      return(xx);
}
/**********************************************************************/
