/**********************************************************************/
      int cbdecp(double a[],int m,int n,int mm) {
/*    int cbdecp(a,m,n,mm)
      double a[]; int m,n,mm; {                                       */
/*    ============================================================    */
      double aij,ajj,akj;
      int    i,j,k,ia,ja,ka,jm;
      --a;
/*    ============================================================    */
/*    dimension a(mm,n+(n-1)/mm) or a((n-1)*mm+n)                     */
/*    ------------------------------------------------------------    */
/*    find [d] & [u] for given [a],        | a11 a12 a13  0   0  |    */
/*    such that  [a] = [u]'[d] [u],        | a12 a22 a23 a24  0  |    */
/*    where [u]'= transpose of [u].  [a] = | a13 a23 a33 a34 a35 |    */
/*                                         |  0  a24 a34 a44 a45 |    */
/*    for example: n=5, m=2, mm=2:         |  0   0  a35 a45 a55 |    */
/*                                                                    */
/*          | d11  0   0   0   0  |        |  1  u12 u13  0   0  |    */
/*          |  0  d22  0   0   0  |        |  0   1  u23 u24  0  |    */
/*    [d] = |  0   0  d33  0   0  |, [u] = |  0   0   1  u34 u35 |    */
/*          |  0   0   0  d44  0  |        |  0   0   0   1  u45 |    */
/*          |  0   0   0   0  d55 |        |  0   0   0   0   1  |    */
/*                                                                    */
/*    input  data in a(2,7) =  |\a11 a12 a13\a33 a34 a35\a55 |        */
/*                             |  ? \a22 a23 a24\a44 a45  ? \|        */
/*    or in 1-d array:                                                */
/*    a(13) = (a11 ? a12 a22 a13 a23 a33 a24 a34 a44 a35 a45 a55);    */
/*                                                                    */
/*    output data in a(2,7) =  |\d11 u12 u13\d33 u34 u35\d55 |        */
/*                             |  ? \d22 u23 u24\d44 u45  ? \|        */
/*    or in 1-d array:                                                */
/*    a(13) = (d11 ? u12 d22 u13 u23 d33 u24 u34 d44 u35 u45 d55).    */
/*    ============================================================    */
/*    +----------------------------------------------------------+    */
/*    | d(i,i)u(i,j)=a(i,j)-\sum_{k=jm}^{i-1} d(k,k)u(k,i)u(k,j) |    */
/*    | a(k+ia)=u(k,i),     a(k+ja)=d(k,k)*u(k,j)                |    */
/*    | a(i+ja)=aij=a(i,j) --> d(i,i)*u(i,j)                     |    */
/*    +----------------------------------------------------------+    */
      ja=0;
      for(j=1; j<=n; j++) {
        jm=j-m; if(jm < 1) jm=1;
        ia=jm*mm;
        for(i=jm+1; i<=j-1; i++) {
          aij=a[i+ja];
          for(k=jm; k<=i-1; k++) {
            aij=aij-a[k+ia]*a[k+ja];
          }
          a[i+ja]=aij;
          ia=ia+mm;
        }
/*    +----------------------------------------------------------+    */
/*    | d(j,j) =a(j,j) - \sum_{k=jm}^{j-1} d(k,k)*u(k,j)*u(k,j)  |    */
/*    | a(k+ja)=d(k,k)*u(k,j) --> u(k,j),  akj=u(k,j)            |    */
/*    | a(j+ja)=ajj=a(j,j) --> d(j,j),     a(k+ka)=d(k,k)        |    */
/*    +----------------------------------------------------------+    */
        ajj=a[j+ja];
        ka=(jm-1)*mm;
        for(k=jm; k<=j-1; k++) {
          akj=a[k+ja]/a[k+ka];
          ajj=ajj-akj*a[k+ja];
          a[k+ja]=akj;
          ka=ka+mm;
        }
        a[j+ja]=ajj;
        if(a[j+ja] == 0.0) goto L50;
        ja=ja+mm;
      }
      return(0);
/*    +----------------------------------------------------------+    */
/*    | return(j) with error : d(j,j)=0                          |    */
/*    +----------------------------------------------------------+    */
  L50:printf(" a[%d,%d] = 0\n",j,j);
      return(j);
}
/**********************************************************************/
      int cbsolx(double a[],double b[],int m,int n,int mm) {
/*    int cbsolx(a,b,m,n,mm)
      double a[],b[]; int m,n,mm; {                                   */
/*    ============================================================    */
      double bi;
      int    i,k,ia,im;
      --a; --b;
/*    ============================================================    */
/*    dimension a(mm,n+(n-1)/mm),b(n) or a((n-1)*mm+n),b(n)           */
/*    ------------------------------------------------------------    */
/*    find {x} from [a]{x}={b} for given {b} & [a]=[u]'[d][u].        */
/*    [d] & [u] are given & put in a(*) (get by subroutine cbdecp)    */
/*    [u]'[d][u]{x}={b} ===> [u]'{z}={b},  [d]{y}={z},  [u]{x}={y}    */
/*    ============================================================    */
/*    +----------------------------------------------------------+    */
/*    | find {z} from {b} by forward substitution of [u]'{z}={b} |    */
/*    | z(i) = bi = b(i) - \sum_{k=im}^{i-1} u'(i,k)*z(k)        |    */
/*    | a(k+ia)=u(k,i)=u'(i,k), b(k)=z(k), b(i)=b(i) --> z(i)    |    */
/*    +----------------------------------------------------------+    */
      ia=0;
      for(i=1; i<=n; i++) {
        im=i-m; if(im < 1) im=1;
        bi=b[i];
        for(k=im; k<=i-1; k++) {
          bi=bi-a[k+ia]*b[k];
        }
        b[i]=bi;
        ia=ia+mm;
      }
/*    +----------------------------------------------------------+    */
/*    | find {y} from {z} from the equation [d]{y}={z}           |    */
/*    | y(i) = z(i) / d(i,i)                                     |    */
/*    | b(i) = z(i) --> y(i),     a(i+ia)=d(i,i)                 |    */
/*    +----------------------------------------------------------+    */
      ia=0;
      for(i=1; i<=n; i++) {
        b[i]=b[i]/a[i+ia];
        ia=ia+mm;
      }
/*    +----------------------------------------------------------+    */
/*    | find {x} from {y} by backward substitution of [u]{x}={y} |    */
/*    | for i=n:2  compute  b(k)=b(k)-u(k,i)*x(i) for k=im:(i-1) |    */
/*    | a(k+ia)=u(k,i),     b(i)=bi=x(i),     b(k)=y(k) --> x(k) |    */
/*    +----------------------------------------------------------+    */
      for(i=n; i>=2; i--) {
        ia=ia-mm;
        im=i-m; if(im < 1) im=1;
        bi=b[i];
        for(k=im; k<=i-1; k++) {
          b[k]=b[k]-a[k+ia]*bi;
        }
      }
      return(0);
}
/**********************************************************************/
