#include <math.h>
/**********************************************************************/
      int cbdecp(double a[],int m,int n,int mm) {
/*    int cbdecp(a,m,n,mm)
      double a[]; int m,n,mm; {                                       */
/*    ============================================================    */
      double aij,ajj,akj;
      int    i,j,k,ia,ja,jm;
      --a;
/*    ============================================================    */
/*    dimension a(mm,n+(n-1)/mm) or a((n-1)*mm+n)                     */
/*    ------------------------------------------------------------    */
/*    find [u] for given [a],              | a11 a12 a13  0   0  |    */
/*    such that  [a] = [u]'[u],            | a12 a22 a23 a24  0  |    */
/*    where [u]'= transpose of [u].  [a] = | a13 a23 a33 a34 a35 |    */
/*                                         |  0  a24 a34 a44 a45 |    */
/*    for example: n=5, m=2, mm=2:         |  0   0  a35 a45 a55 |    */
/*                                                                    */
/*          | u11  0   0   0   0  |        | u11 u12 u13  0   0  |    */
/*          | u12 u22  0   0   0  |        |  0  u22 u23 u24  0  |    */
/*    [u]'= | u13 u23 u33  0   0  |, [u] = |  0   0  u33 u34 u35 |    */
/*          |  0  u24 u34 u44  0  |        |  0   0   0  u44 u45 |    */
/*          |  0   0  u35 u45 u55 |        |  0   0   0   0  u55 |    */
/*                                                                    */
/*    input  data in a(2,7) =  |\a11 a12 a13\a33 a34 a35\a55 |        */
/*                             |  ? \a22 a23 a24\a44 a45  ? \|        */
/*    or in 1-d array:                                                */
/*    a(13) = (a11 ? a12 a22 a13 a23 a33 a24 a34 a44 a35 a45 a55);    */
/*                                                                    */
/*    output data in a(2,7) =  |\u11 u12 u13\u33 u34 u35\u55 |        */
/*                             |  ? \u22 u23 u24\u44 u45  ? \|        */
/*    or in 1-d array:                                                */
/*    a(13) = (u11 ? u12 u22 u13 u23 u33 u24 u34 u44 u35 u45 u55).    */
/*    ============================================================    */
/*    +----------------------------------------------------------+    */
/*    | aij = a(i,j) - \sum_{k=jm}^{i-1} u(k,i)*u(k,j)           |    */
/*    | i<j : u(i,j)=aij/u(i,i);        i=j : u(i,j)=sqrt(aij)   |    */
/*    | a(k+ia)=u(k,i), a(k+ja)=u(k,j), a(i+ja)=a(i,j) --> u(i,j)|    */
/*    +----------------------------------------------------------+    */
      ja=0;
      for(j=1; j<=n; j++) {
        jm=j-m; if(jm < 1) jm=1;
        ia=(jm-1)*mm;
        for(i=jm; i<=j; i++) {
          aij=a[i+ja];
          for(k=jm; k<=i-1; k++) {
            aij=aij-a[k+ia]*a[k+ja];
          }
          if(i != j) {
            a[i+ja]=aij/a[i+ia];
          } else if(aij > 0.0) {
            a[i+ja]=sqrt(aij);
          } else {
            printf(" a[%d,%d] <= 0\n",j,j);
            return(j);
          }
        ia=ia+mm;
        }
      ja=ja+mm;
      }
      return(0);
}
/**********************************************************************/
      int cbsolx(double a[],double b[],int m,int n,int mm) {
/*    int cbsolx(a,b,m,n,mm)
      double a[],b[]; int m,n,mm; {                                   */
/*    ============================================================    */
      double bi;
      int    i,k,ia,im;
      --a; --b;
/*    ============================================================    */
/*    dimension a(mm,n+(n-1)/mm),b(n) or a((n-1)*mm+n),b(n)           */
/*    ------------------------------------------------------------    */
/*    find {x} from [a]{x}={b} for given {b} & [a]=[u]'[u].           */
/*    [u] is given & put in a(*) (get by subroutine cbdecp)           */
/*    [u]'[u]{x}={b} ===> [u]'{y}={b},  [u]{x}={y}                    */
/*    ============================================================    */
/*    +----------------------------------------------------------+    */
/*    | find {y} from {b} by forward substitution of [u]'{y}={b} |    */
/*    | u(i,i)*y(i) = bi = b(i) - \sum_{k=im}^{i-1} u'(i,k)*y(k) |    */
/*    | a(k+ia)=u(k,i)=u'(i,k), b(k)=y(k), b(i)=b(i) --> y(i)    |    */
/*    +----------------------------------------------------------+    */
      ia=0;
      for(i=1; i<=n; i++) {
        im=i-m; if(im < 1) im=1;
        bi=b[i];
        for(k=im; k<=i-1; k++) {
          bi=bi-a[k+ia]*b[k];
        }
        b[i]=bi/a[i+ia];
        ia=ia+mm;
      }
/*    +----------------------------------------------------------+    */
/*    | find {x} from {y} by backward substitution of [u]{x}={y} |    */
/*    | for i=n:1  compute  b(k)=b(k)-u(k,i)*x(i) for k=im:(i-1) |    */
/*    | a(k+ia)=u(k,i), b(k)=y(k) --> u(k,k)*x(k) ==> bi=x(i)    |    */
/*    +----------------------------------------------------------+    */
      for(i=n; i>=1; i--) {
        ia=ia-mm;
        im=i-m; if(im < 1) im=1;
        bi=b[i]/a[i+ia];
        for(k=im; k<=i-1; k++) {
          b[k]=b[k]-a[k+ia]*bi;
        }
        b[i]=bi;
      }
      return(0);
}
/**********************************************************************/
