#include <stdio.h>
#include <ctype.h>
int freeii(int    *, FILE *);
int freer8(double *, FILE *);
/**********************************************************************/
/*    program for decomposition of an unsymmetric banded matrix       */
/*    and solve the equation by substitution                          */
/*    ============================================================    */
      main(int argc,char **argv) { /* char **envp */
/*    ============================================================    */
      FILE   *fopen(),*fp_r,*fp_w;
      double *agen,*bg;
      double *apcd,*bp;
      double *afcd,*bf;
      int    mag,map,maf,nag,nap,naf;
      int    n,ma,mb,i,j,k,jma,jmb;
/*    ============================================================    */
      if(argc<3) {printf("Usage : cudman infile outfile"); exit(-1);}

      if((fp_r=fopen(argv[1],"r"))==NULL) exit(-1);
      if((fp_w=fopen(argv[2],"w"))==NULL) exit(-1);

      freeii(&n,fp_r); freeii(&ma,fp_r); freeii(&mb,fp_r);

      maf=ma+mb; if(maf>n) maf=n; /* for full condensed form    */
      map=maf+1; if(map>n) map=n; /* for partial condensed form */
      mag=n;                      /* for general form           */
      nag=n;
      nap=n+(n-1)/map;
      naf=n+(n-1)/maf;
      agen=(double *)calloc(mag*nag,sizeof(double));
      apcd=(double *)calloc(map*nap,sizeof(double));
      afcd=(double *)calloc(maf*naf,sizeof(double));
      bg  =(double *)calloc(n,      sizeof(double));
      bp  =(double *)calloc(n,      sizeof(double));
      bf  =(double *)calloc(n,      sizeof(double));
      reada(agen,n,n,mag,fp_r);
      reada(bg,1,n,1,fp_r);
/*    ------------------------------------------------------------    */
/*    cleara(apcd,map,nap,map);    */
      copya(agen,apcd,n,ma,mb,mag,map);
      copyb(bg,bp,n);
/*    cleara(afcd,maf,naf,maf);    */
      copya(agen,afcd,n,ma,mb,mag,maf);
      copyb(bg,bf,n);
/*    ------------------------------------------------------------    */
      fprintf(fp_w,"\n Given matrix in general form\n");
      printa(agen,n,n,mag,fp_w);
      fprintf(fp_w,"\n In partial condensed form\n");
      printa(apcd,map,nap,map,fp_w);
      fprintf(fp_w,"\n In full condensed form\n");
      printa(afcd,maf,naf,maf,fp_w);
      fprintf(fp_w,"\n Constant vector\n");
      printb(bg,n,fp_w);
/*    ------------------------------------------------------------    */
      cudecp(agen,n,ma,mb,mag);
      cusolx(agen,bg,n,ma,mb,mag);
/*    ------------------------------------------------------------    */
      cudecp(apcd,n,ma,mb,map);
      cusolx(apcd,bp,n,ma,mb,map);
/*    ------------------------------------------------------------    */
      cudecp(afcd,n,ma,mb,maf);
      cusolx(afcd,bf,n,ma,mb,maf);
/*    ------------------------------------------------------------    */
      fprintf(fp_w,"\n Decomposed matrix in general form\n");
      printa(agen,mag,nag,mag,fp_w);
      fprintf(fp_w,"\n Result vector\n");
      printb(bg,n,fp_w);
      fprintf(fp_w,"\n In partial condensed form\n");
      printa(apcd,map,nap,map,fp_w);
      fprintf(fp_w,"\n Result vector\n");
      printb(bp,n,fp_w);
      fprintf(fp_w,"\n In full condensed form\n");
      printa(afcd,maf,naf,maf,fp_w);
      fprintf(fp_w,"\n Result vector\n");
      printb(bf,n,fp_w);
      exit(0);
}
/**********************************************************************/
      int printa(double a[],int n,int m,int nn,FILE *fp_w) {
/*    ============================================================    */
      int i,j,jl,jh,mm=10;
      --a; a-=nn;     /* dimension a(nn,m) */
/*    ============================================================    */
      for(jl=1; jl<=m; jl+=mm) {
        jh=jl+mm-1; if(jh > m) jh=m;
        fprintf(fp_w,"\n     ");
        for(j=jl; j<=jh; j++) fprintf(fp_w,"%6d",j);
        fprintf(fp_w,"\n");
        for(i=1; i<=n; i++) {
          fprintf(fp_w,"%5d",i);
          for(j=jl; j<=jh; j++) fprintf(fp_w,"%6.2lf",a[i+j*nn]);
          fprintf(fp_w,"\n");
        }
      }
      return(0);
}
/**********************************************************************/
      int reada(double a[],int n,int m,int nn,FILE *fp_r) {
/*    ============================================================    */
      int i,j;
      --a; a-=nn;     /* dimension a(nn,m) */
/*    ============================================================    */
      for(i=1; i<=n; i++) {
        for(j=1; j<=m; j++) freer8(&a[i+j*nn],fp_r);
      }
      return(0);
}
/**********************************************************************/
      int cleara(double a[],int n,int m,int nn) {
/*    ============================================================    */
      int i,j;
      --a; a-=nn;     /* dimension a(nn,m) */
/*    ============================================================    */
      for(i=1; i<=n; i++) {
        for(j=1; j<=m; j++) a[i+j*nn]=0.0;
      }
      return(0);
}
/**********************************************************************/
      int printb(double b[],int n,FILE *fp_w) {
/*    ============================================================    */
      int j,mm=10;
      --b;            /* dimension b(n) */
/*    ============================================================    */
      for(j=1; j<=n; j++) {
        if(j%mm == 1) fprintf(fp_w,"\n     ");
        fprintf(fp_w,"%6.2lf",b[j]);
      }
      fprintf(fp_w,"\n");
      return(0);
}
/**********************************************************************/
      int copya(double a[],double b[],int n,int ma,int mb
               ,int mma,int mmb) {
/*    ============================================================    */
      int i,j,jma,jmb;
      --a; a-=mma;     /* dimension a(mma,n) */
      --b; b-=mmb;     /* dimension b(mmb,n) */
/*    ============================================================    */
      for(j=1; j<=n; j++) {
        jmb=j-mb; if(jmb < 1) jmb=1;
        jma=j+ma; if(jma > n) jma=n;
        for(i=jmb; i<=jma; i++) b[i+j*mmb]=a[i+j*mma];
      }
      return(0);
}
/**********************************************************************/
      int copyb(double a[],double b[],int n) {
/*    ============================================================    */
      int j;
      --a; --b;        /* dimension a(n),b(n) */
/*    ============================================================    */
      for(j=1; j<=n; j++) b[j]=a[j];
      return(0);
}
/**********************************************************************/
