#include <math.h>
/**********************************************************************/
      int vbdecp(double a[],int ll[],int n) {
/*    int vbdecp(a,ll,n)
      double a[]; int ll[],n; {                                       */
/*    ============================================================    */
      double aij;
      int    i,j,k,ia,ja,ib,jb,im,jm;
      --a; --ll;
/*    ============================================================    */
/*    dimension a(n+ll(n)),ll(n)                                      */
/*    ------------------------------------------------------------    */
/*    find [u] for given [a],              | a11  0  a13  0   0  |    */
/*    such that  [a] = [u]'[u],            |  0  a22 a23  0  a25 |    */
/*    where [u]'= transpose of [u].  [a] = | a13 a23 a33 a34 a35 |    */
/*    for example:                         |  0   0  a34 a44 a45 |    */
/*    n=5  and  ll(5)=(0 0 2 3 6):         |  0  a25 a35 a45 a55 |    */
/*                                                                    */
/*          | u11  0   0   0   0  |        | u11  0  u13  0   0  |    */
/*          |  0  u22  0   0   0  |        |  0  u22 u23  0  u25 |    */
/*    [u]'= | u13 u23 u33  0   0  |, [u] = |  0   0  u33 u34 u35 |    */
/*          |  0   0  u34 u44  0  |        |  0   0   0  u44 u45 |    */
/*          |  0  u25 u35 u45 u55 |        |  0   0   0   0  u55 |    */
/*                                                                    */
/*    given ll(5) = (  0   0           2       3               6),    */
/*      i + ll(j) :  1+0 2+0 1+2 2+2 3+2 3+3 4+3 2+6 3+6 4+6 5+6      */
/*    location    :    1   2   3   4   5   6   7   8   9  10  11      */
/*    input a(11) = (a11 a22 a13 a23 a33 a34 a44 a25 a35 a45 a55);    */
/*    outpu a(11) = (u11 u22 u13 u23 u33 u34 u44 u25 u35 u45 d55).    */
/*    ============================================================    */
/*    +----------------------------------------------------------+    */
/*    | aij = a(i,j) - \sum_{k=im}^{i-1} u(k,i)*u(k,j)           |    */
/*    | i<j : u(i,j)=aij/u(i,i);        i=j : u(i,j)=sqrt(aij)   |    */
/*    | a(k+ia)=u(k,i), a(k+ja)=u(k,j), a(i+ja)=a(i,j) --> u(i,j)|    */
/*    +----------------------------------------------------------+    */
      jb=0;
      for(j=1; j<=n; j++) {
        ja=ll[j]-ll[1];
        jm=j-ja+jb;
        ib=0;
        if(jm > 1) ib=ll[jm-1]-ll[1];
        for(i=jm; i<=j; i++) {
          ia=ll[i]-ll[1];
          im=i-ia+ib; if(im < jm) im=jm;
          aij=a[i+ja];
          for(k=im; k<=i-1; k++) {
            aij-=a[k+ia]*a[k+ja];
          }
          if(i != j) {
            a[i+ja]=aij/a[i+ia];
          } else if(aij > 0.0) {
            a[i+ja]=sqrt(aij);
          } else {
            printf(" a[%d,%d] <= 0\n",j,j);
            return(j);
          }
          ib=ia;
        }
        jb=ja;
      }
      return(0);
}
/**********************************************************************/
      int vbsolx(double a[],double b[],int ll[],int n) {
/*    int vbsolx(a,b,ll,n)
      double a[],b[]; int ll[],n; {                                   */
/*    ============================================================    */
      double bi;
      int    i,k,ia,ib,im;
      --a; --b; --ll;
/*    ============================================================    */
/*    dimension a(n+ll(n)),b(n),ll(n)                                 */
/*    ------------------------------------------------------------    */
/*    find {x} from [a]{x}={b} for given {b} & [a]=[u]'[u].           */
/*    [u] is given & put in a(*) (get by subroutine vbdecp)           */
/*    [u]'[u]{x}={b} ===> [u]'{y}={b},  [u]{x}={y}                    */
/*    ============================================================    */
/*    +----------------------------------------------------------+    */
/*    | find {y} from {b} by forward substitution of [u]'{y}={b} |    */
/*    | u(i,i)*y(i) = bi = b(i) - \sum_{k=im}^{i-1} u'(i,k)*y(k) |    */
/*    | a(k+ia)=u(k,i)=u'(i,k), b(k)=y(k), b(i)=b(i) --> y(i)    |    */
/*    +----------------------------------------------------------+    */
      ib=0;
      for(i=1; i<=n; i++) {
        ia=ll[i]-ll[1];
        im=i-ia+ib;
        bi=b[i];
        for(k=im; k<=i-1; k++) {
          bi-=a[k+ia]*b[k];
        }
        b[i]=bi/a[i+ia];
        ib=ia;
      }
/*    +----------------------------------------------------------+    */
/*    | find {x} from {y} by backward substitution of [u]{x}={y} |    */
/*    | for i=n:2  compute  b(k)=b(k)-u(k,i)*x(i) for k=im:(i-1) |    */
/*    | a(k+ia)=u(k,i), b(k)=y(k) --> u(k,k)*x(k) ==> bi=x(i)    |    */
/*    +----------------------------------------------------------+    */
      for(i=n; i>=2; i--) {
        ia=ll[i]-ll[1];
        im=i-ll[i]+ll[i-1];
        bi=b[i]/a[i+ia];
        for(k=im; k<=i-1; k++) {
          b[k]-=a[k+ia]*bi;
        }
        b[i]=bi;
      }
      b[1]/=a[1];
      return(0);
}
/**********************************************************************/
