#include <math.h>
/**********************************************************************/
      int jacobi(double a[],double v[],int n,double x[],int ih[]
                ,int jevc,int *pm,double epsi,int nv) {
/*    int jacobi(a,v,n,x,ih,jevc,pm,epsi,nv)
      double a[],v[],x[],epsi; int ih[],n,*pm,nv; {                   */
/*    ============================================================    */
      double aii,aij,ajj,amin,tang,sine,cosn,coss,tmp;
      int    i,j,ia,ja,ib,jb,ii,jj,ip,jp,ju,jv,jx,jy;
      --a; --v; --x; --ih;
/*    ============================================================    */
/*    dimension a(n*(n-1)/2),v(nv,n),x(n),ih(n)  ! a(na,n)            */
/*    ------------------------------------------------------------    */
/*    [a] {x} = d {x} --> [a] [v] = [v] [d]                           */
/*    ------------------------------------------------------------    */
/*    input : a,v,n,jevc,m,epsi,nv,na      output : a,v,x,m           */
/*    ------------------------------------------------------------    */
/*i   a(n,n) = [a] matrix required upper triangular part only         */
/*o          = [d] diagonal matrix                                    */
/*i   v(n,n) = matrix to be post-multiplied by eigenvector matrix     */
/*o          = eivec(jevc=1); none(jevc=0); input.v*eivec(jevc=-1)    */
/*i   n      = order of matrix [a]                                    */
/*o   x(n)   = eigenvalues (also in the diagonal of [a])              */
/*w   ih(n)  = working array for row i of max.a(i,j) in col j         */
/*i   jevc   = 1 : find eigenvectors and return in v(n,n)             */
/*i          = 0 : do not find eigenvectors and v(n,n) is dummy       */
/*i          =-1 : find eigenvectors and post-multiply to v(n,n)      */
/*o   m      = iteration count                                        */
/*i   epsi   = convergence if max.|a(i,j)| < min.a(k,k)*epsi          */
/*i   nv,na  = row dimensions of array v(nv,n) and a(na,n)            */
/*    ============================================================    */
/*    +----------------------------------------------------------+    */
/*    | initialize : set m=0, [v]=[i] (jevc=1), x(n) and ih(n)   |    */
/*    +----------------------------------------------------------+    */
/*    *pm=0;                                                          */
      if(jevc > 0) {
        for(j=1; j<=n; j++) {
          jv=(j-1)*nv;
          for(i=1; i<=n; i++) {
            v[i+jv]=0.0;
          }
          v[j+jv]=1.0;
        }
      }
      if(n <= 1) return(0);
/*    +----------------------------------------------------------+    */
/*    | x(jx) = max.|a(ih(jx),jx)| in col jx  (ih(jx) < jx)      |    */
/*    | amin  = min.a(k,k) (k=1:n)                               |    */
/*    +----------------------------------------------------------+    */
      amin=fabs(a[1]);
      for(jx=2; jx<=n; jx++) {
        ja=(jx-1)*jx/2;        /* ja=(jx-1)*na */
        if((tmp=fabs(a[jx+ja])) < amin) amin=tmp;
        x[jx]=-1.0;
        for(ii=1; ii<=jx-1; ii++) {
          if((tmp=fabs(a[ii+ja])) <= x[jx]) continue;
          x[jx]=tmp;
          ih[jx]=ii;
        }
      }
/*    +----------------------------------------------------------+    */
/*    | begin loops : return if converged                        |    */
/*    +----------------------------------------------------------+    */
/*    | find ip,jp for max.|a(ip,jp)| from x(n) and ih(n)        |    */
/*    +----------------------------------------------------------+    */
  L50:*pm=*pm+1;
/*    printt(a,n,"matrix a") */
      aij=-1.0;
      for(jx=2; jx<=n; jx++) {
        if(x[jx] <= aij) continue;
        aij=x[jx];
        ip=ih[jx];
        jp=jx;
      }
      if(aij <= amin*epsi) return(0);
/*    ------------------------------------------------------------    */
      ia=ip;
      ib=jp;
      ja=(ip-1)*ip/2;    /* ja=(ip-1)*na */
      jb=(jp-1)*jp/2;    /* jb=(jp-1)*na */
/*    +----------------------------------------------------------+    */
/*    | compute the rotation angle                               |    */
/*    +----------------------------------------------------------+    */
      aij=2.*a[ia+jb];
      aii=a[ia+ja];
      ajj=a[ib+jb];
      tang=aij/(fabs(ajj-aii)+sqrt((ajj-aii)*(ajj-aii)+aij*aij));
      aij=a[ia+jb]*tang;
      a[ia+jb]=0.0;
      if(aii >= ajj) {
        a[ia+ja]=aii+aij;
        a[ib+jb]=ajj-aij;
      } else {
        a[ia+ja]=ajj+aij;
        a[ib+jb]=aii-aij;
        tang=1.0/tang;
      }
      coss=1.0/(1.0+tang*tang);
      cosn=sqrt(coss);
      sine=tang*cosn;
/*    if((tmp=fabs(a[ib+jb])) < amin) amin=tmp;                       */
/*    +----------------------------------------------------------+    */
/*    |                 ip                    jp                 |    */
/*    |    / .        *                     *                  \ |    */
/*    |    |   .   a(ia+ja)              a(ib+jb)              | |    */
/*    |    |     .    *                     *                  | |    */
/*    | ip |       a(ip,ip) * a(ia+ja) * a(ip,jp) * a(ia+ja) * | |    */
/*    |    |                .               *                  | |    */
/*    |    |                     .       a(ib+jb)              | |    */
/*    |    |                           .    *                  | |    */
/*    | jp |                             a(jp,jp) * a(ib+jb) * | |    */
/*    |    |                                      .            | |    */
/*    |    |                                           .       | |    */
/*    |    \                                                 . / |    */
/*    +----------------------------------------------------------+    */
/*    | reset x(ip),ih(ip) and x(jp),ih(jp) for the changes      |    */
/*    +----------------------------------------------------------+    */
      jx=ip;
      jy=jp;
      x[jx]=0.0;
      x[jy]=0.0;
      for(jj=1; jj<=n; jj++) {
        if(jj < ip) {
/*        jx=ip                                                       */
/*        jy=jp                                                       */
          ia=jj;
          ib=jj;
/*        ja=(ip-1)*ip/2    ! ja=(ip-1)*na                            */
/*        jb=(jp-1)*jp/2    ! jb=(jp-1)*na                            */
        } else if(jj == ip) {
          ia=ip;
          continue;
        } else {
          jx=jj;
          ja=ja+jj-1;       /* ja=ja+na */
          if(jj < jp) {
/*          jx=jj                                                     */
/*          jy=jp                                                     */
/*          ia=ip                                                     */
            ib=jj;
/*          ja=(jj-1)*jj/2  ! ja=(jj-1)*na                            */
/*          jb=(jp-1)*jp/2  ! jb=(jp-1)*na                            */
          } else if(jj == jp) {
            ib=jp;
            continue;
          } else {
/*          jx=jj                                                     */
            jy=jx;
/*          ia=ip                                                     */
/*          ib=jp                                                     */
/*          ja=(jj-1)*jj/2  ! ja=(jj-1)*na                            */
            jb=ja;
          }
        }
        aij=a[ia+ja];
        a[ia+ja]= aij*cosn+a[ib+jb]*sine;
        a[ib+jb]=-aij*sine+a[ib+jb]*cosn;
/*      +--------------------------------------------------------+    */
/*      | reset x(jx),ih(jx) if the max.x(jx) has been changed   |    */
/*      +--------------------------------------------------------+    */
        if(ih[jx] == ip || ih[jy] == jp) {
          x[jx]=-1.0;
          for(ii=1; ii<=jj-1; ii++) {
            if(x[jx] >= (tmp=fabs(a[ii+ja]))) continue;
            x[jx]=tmp;
            ih[jx]=ii;
          }
          if(jj >= jp) continue;
        }
/*      +--------------------------------------------------------+    */
/*      | update x(jx),ih(jx) and x(jy),ih(jy)                   |    */
/*      +--------------------------------------------------------+    */
        if((tmp=fabs(a[ia+ja])) > x[jx]) {
          x[jx]=tmp;
          ih[jx]=ia;
        }
        if((tmp=fabs(a[ib+jb])) > x[jy]) {
          x[jy]=tmp;
          ih[jy]=ib;
        }
      }
/*    +----------------------------------------------------------+    */
/*    | v(n,n) post-multiply by the rotation matrix              |    */
/*    +----------------------------------------------------------+    */
      if(jevc != 0) {
        ju=(ip-1)*nv;
        jv=(jp-1)*nv;
        for(i=1; i<=n; i++) {
          aij=v[i+ju];
          v[i+ju]= aij*cosn+v[i+jv]*sine;
          v[i+jv]=-aij*sine+v[i+jv]*cosn;
        }
      }
      goto L50;
}
/**********************************************************************/
