#include <math.h>
/**********************************************************************/
      int azexbz(int na,int n,double a[],double b[],double d[]
                ,double e[],double f[],double z[],int *pierr) {
/*    int azexbz(na,n,a,b,d,e,f,z,pierr)
      double a[],b[],d[],e[],f[],z[]; int na,n,*pierr; {              */
/*    ============================================================    */
      int    i,j,ja;
      int    ortheg(),hqrids(),stcond(),qutgen(),quteve(),ztaz();
/*    --a; a-=na; --b; b-=na; --d; --e; --f; --z; z-=na;              */
/*    ============================================================    */
/*    find the eigenvalues and eigenvectors of [A]{Z} = e [B]{Z}      */
/*    ------------------------------------------------------------    */
/*i   a(n,n) = given real matrix [A]                                  */
/*o          = [Y] in upper triangular part (no need for user)        */
/*i   b(n,n) = given real matrix [B]                                  */
/*o          = [U] in upper triangular part (no need for user)        */
/*o   d(j)   = real part of j-th eigenvalue (if f(j) != 0)            */
/*o   e(j)   = imag part of j-th eigenvalue (if f(j) != 0)            */
/*o   f(j)   = absolute value of j-th eigenvalue of [B]{X}=e{X}       */
/*           = 0 & d(j)+e(j)i != 0 then j-th e-value is infinity      */
/*           = 0 & d(j)+e(j)i == 0 then j-th e-value is arbitary      */
/*o   z(n,j) = z(1:n,j) = eigenvector of real eigenvalue d(j)         */
/*             z(1:n,j)+z(1:n,j+1)i = eigenvector of d(j)+e(j)i       */
/*             z(1:n,j)-z(1:n,j+1)i = eigenvector of d(j)-e(j)i       */
/*o   ierr   = j: j-th eigenvalue no converge; = 0: normal return     */
/*    ------------------------+-----------------------------------    */
/*            B               |               A  Z                    */
/*    ------------------------+-----------------------------------    */
/*       [V]'[B][V] = [T]     |          [V]'[A][V] = [C]             */
/*    [u][V]'[B][V][U] = [S]  |       [u][V]'[A][V][U]                */
/*     [k][d][S][d] = [J]   /   [k][d][u][V]'[A][V][U][d] = [G]       */
/*        [L][J][R] = [J] /  [L][k][d][u][V]'[A][V][U][d][R]          */
/*               [Qut] = [Z]'[L][k][d][u][V]'[A][V][U][d][R][Z]       */
/*            [Qdg] = [y][Z]'[L][k][d][u][V]'[A][V][U][d][R][Z][Y]    */
/*    ------------------------------------------------------------    */
/*      Note : [y],[k],[d],[u] are inverses of [Y],[K],[D],[U]        */
/*           : [V],[Z] are orthogonal matrices                        */
/*           : [U],[Y] are upper triangular matrices                  */
/*           : [D],[d] are diagonal matrices                          */
/*           : [K],[k] are quasi-diagonal orthogonal matrices         */
/*           : [L],[R] are static condensation matrices               */
/*           : [T],[Qut] are quasi-upper triangular matrices          */
/*           : [S],[Qdg] are quasi-diagonal matrices                  */
/*           : [J] is quasi-identity matrix : [V]'[J][V]  !=  [J]     */
/*           : [J] = [y][Z]'[J][Z][Y] only for restrict [Z] & [Y]     */
/*    ============================================================    */
      for(j=0; j<n; j++) { ja=j*na;
        for(i=0; i<n; i++) {
          z[i+ja]=0.0;
        }
        z[j+ja]=1.0;
      }
      ortheg(na,n,1,n,b,d,z);             /*      [ ]'[B][ ]=[H]      */
      hqrids(na,n,1,n,b,d,e,z,pierr);     /*      [V]'[B][V]=[T]      */
      if(*pierr != 0) return(0);          /*                          */
      ztaz  (a,f,z,n,na);                 /*      [V]'[A][V]=[C]      */
      quteve(na,n,1,n,b,d,e,z);           /*       [u][T][U]=[S]      */
      qutgen(a,b,d,e,f,z,n,na);           /* [k][d][u][C][U][d]=[G]   */
      stcond(a,d,e,f,z,n,na);             /*       [L][G][R]          */
      ortheg(na,n,1,n,a,d,z);             /*   [ ]'[L][G][R][ ]=[Hes] */
      hqrids(na,n,1,n,a,d,e,z,pierr);     /*   [Z]'[L][G][R][Z]=[Tri] */
      quteve(na,n,1,n,a,d,e,z);           /* [V][U][d][R][Z][Y]=[Eve] */
      return(0);/* [y][Z]'[L][k][d][u][V]'[A][V][U][d][R][Z][Y]=[Eva] */
}
/**********************************************************************/
      int qutgen(double a[],double u[],double wr[],double wi[]
                ,double f[],double z[],int n,int na) {
/*    int qutgen(a,u,wr,wi,f,z,n,na)
      double a[],u[],wr[],wi[],f[],z[]; int n,na; {                   */
/*    ============================================================    */
      double c,s,dj,aij,akj,wrj,wij;
      int    i,j,k,ia,ja,ka;
      --a; a-=na; --u; u-=na; --wr; --wi; --f; --z; z-=na;
/*    ============================================================    */
/*    compute  [G] = [K]^{-1}[D]^{-1}[U]^{-1}[C][U][D]^{-1}           */
/*    ------------------------------------------------------------    */
/*i   a(n,n) = [C] = [V]'[A][V]                                       */
/*o          = [G] = [k][d][u][V]'[A][V][U][d]                        */
/*i   u(n,n) = [U] : Trans. matrix for [U]^{-1}[V]'[B][V][U] = [S]    */
/*i   wr(n)  = Real part of eigenvalues                               */
/*i   wi(n)  = Imag part of eigenvalues                               */
/*o   f(n)   = sqrt(wr(n)**2+wi(n)**2)                                */
/*i   z(n,n) = [V][U] : Trans. matrix for [u][V]'[B][V][U] = [S]      */
/*o          = [V][U][d] : [d] = [D]^{-1} for [S] = [D][K][J][D]      */
/*    ============================================================    */
/*    +----------------------------------------------------------+    */
/*    | compute [a] = [a][u] : [u] is an upper triangular matrix |    */
/*    +----------------------------------------------------------+    */
      for(j=n; j>=1; j--) { ja=j*na;
        for(i=1; i<=n; i++) {
          aij=0.0;
          for(k=1; k<=j; k++) {
            aij=aij+a[i+k*na]*u[k+ja];
          }
          a[i+ja]=aij;
        }
      }
/*    +----------------------------------------------------------+    */
/*    | compute [a] = [u]^{-1}[a] : by backward substitutions    |    */
/*    +----------------------------------------------------------+    */
      for(j=1; j<=n; j++) { ja=j*na;
        for(k=n; k>=1; k--) { ka=k*na;
          akj=a[k+ja]/u[k+ka];
          for(i=1; i<=k-1; i++) {
            a[i+ja]=a[i+ja]-u[i+ka]*akj;
          }
          a[k+ja]=akj;
        }
      }
/*    +----------------------------------------------------------+    */
/*    | compute [a] = [d]^{-1}[a][d]^{-1} ; [z] = [z][d]^{-1}    |    */
/*    +----------------------------------------------------------+    */
      for(j=1; j<=n; j++) { ja=j*na;
        wrj=fabs(wr[j]); wij=fabs(wi[j]);
        if(wrj == 0.0 && wij == 0.0) continue;
        if(wrj > wij) f[j]=wrj*sqrt(1.0+(wij/wrj)*(wij/wrj));
        else          f[j]=wij*sqrt(1.0+(wrj/wij)*(wrj/wij));
/*      f[j]=sqrt(wr[j]*wr[j]+wi[j]*wi[j])); */
/*      if(f[j] == 0.0) continue;            */
        dj=1.0/sqrt(f[j]);
        for(i=1; i<=n; i++) { ia=i*na;
          z[i+ja]=z[i+ja]*dj;
          a[i+ja]=a[i+ja]*dj;
          a[j+ia]=a[j+ia]*dj;
        }
      }
/*    +----------------------------------------------------------+    */
/*    | compute [a] = [k]^{-1}[a]                                |    */
/*    +----------------------------------------------------------+    */
      for(k=1; k<=n; k++) {
        if(wi[k] != 0.0) {         /* complex eigenvalue of [b] */
          if(wi[k] < 0.0) continue;
          c=wr[k]/f[k];
          s=wi[k]/f[k];
          for(j=1; j<=n; j++) { ja=j*na;
            akj=a[k+ja];
            a[k  +ja]=c*akj-s*a[k+1+ja];
            a[k+1+ja]=s*akj+c*a[k+1+ja];
          }
        } else if(wr[k] < 0.0) {   /* negative real eigenvalue of [b] */
          for(j=1; j<=n; j++) { ja=j*na;
            a[k+ja]=-a[k+ja];
          }
        }
      }
      return(0);
}
/**********************************************************************/
      int stcond(double a[],double d[],double e[],double f[],double z[]
                ,int n,int na) {
/*    int stcond(a,d,e,f,z,n,na)
      double a[],d[],e[],f[],z[]; int n,na; {                         */
/*    ============================================================    */
      int    i,j,k,ia,ja,ka;
      double s,akj;
      --a; a-=na; --d; --e; --f; --z; z-=na;
/*    ============================================================    */
/*    perform static condensations  [L][G][R] ==> [A]                 */
/*    ------------------------------------------------------------    */
/*i   a(n,n) = [G] = [k][d][u][V]'[A][V][U][d] : matrix to be cond.   */
/*o          = [L][G][R] = [L][k][d][u][V]'[A][V][U][d][R]            */
/*w   d,e(n) = Working array                                          */
/*i   f(n)   = Absolute value of eigenvalue                           */
/*i   z(n,n) = [V][U][d] : for [k][d][u][V]'[B][V][U][d] = [J]        */
/*o          = [V][U][d][R]                                           */
/*    ============================================================    */
/*    +----------------------------------------------------------+    */
/*    | compute  [A] = [L][G][R]                                 |    */
/*    |   ------[ L ]------     ------------[ R ]-------------   |    */
/*    |  | 1   -g13/g33    |   |    1                         |  |    */
/*    |  |   1 -g23/g33    |   |            1                 |  |    */
/*    |  |         1       |   |-g31/g33 -g32/g33  1 -g34/g33 |  |    */
/*    |  |     -g43/g33  1 |   |                        1     |  |    */
/*    +----------------------------------------------------------+    */
      for(k=1; k<=n; k++) { ka=k*na;
        if(f[k] != 0.0) continue;
        if(a[k+ka] == 0.0) {
          s=0.0;
          for(i=1; i<=n; i++) { ia=i*na;
            s=s+fabs(a[i+ka]*a[k+ia]);
            a[i+ka]=0.0;
            a[k+ia]=0.0;
          }
          a[k+ka]=1.0;
          if(s != 0.0) printf("\n Matrix is not wellposed !");
          continue;
        }
        for(i=1; i<=n; i++) {
          e[i]=z[i+ka];
          d[i]=a[i+ka];
          a[i+ka]=0.0;
        }
        a[k+ka]=d[k];
/*      +--------------------------------------------------------+    */
/*      | compute  [a] = [l][g][r]  and  [z] = [z][r]            |    */
/*      +--------------------------------------------------------+    */
        for(j=1; j<=n; j++) { ja=j*na;
          if(j == k) continue;
          akj=a[k+ja]/a[k+ka];
          for(i=1; i<=n; i++) {
            z[i+ja]=z[i+ja]-e[i]*akj;
            a[i+ja]=a[i+ja]-d[i]*akj;
          }
          a[k+ja]=0.0;
        }
      }
      return(0);
}
/**********************************************************************/
      int ztaz(double a[],double w[],double z[],int n,int na) {
/*    int ztaz(a,w,z,n,na)
      double a[],w[],z[]; int n,na; {                                 */
/*    ============================================================    */
      double  s;
      int     i,j,k,ia,ja,ka;
      --a; a-=na; --w; --z; z-=na;
/*    ============================================================    */
/*    compute  [A] = [Z]'[A][Z]                                       */
/*    ------------------------------------------------------------    */
/*i   a(n,n) = [A]                                                    */
/*o          = [Z]'[A][Z]                                             */
/*w   w(n)   = Working array                                          */
/*i   z(n,n) = Orthogonal matrix                                      */
/*    ============================================================    */
      for(i=1; i<=n; i++) {
        for(k=1; k<=n; k++) { ka=k*na;
          w[k]=a[i+ka];
        }
        for(j=1; j<=n; j++) { ja=j*na;
          s=0.0;
          for(k=1; k<=n; k++) {
            s=s+w[k]*z[k+ja];
          }
          a[i+ja]=s;
        }
      }
      for(j=1; j<=n; j++) { ja=j*na;
        for(k=1; k<=n; k++) {
          w[k]=a[k+ja];
        }
        for(i=1; i<=n; i++) { ia=i*na;
          s=0.0;
          for(k=1; k<=n; k++) {
            s=s+z[k+ia]*w[k];
          }
          a[i+ja]=s;
        }
      }
      return(0);
}
/**********************************************************************/
