#include <math.h>
/**********************************************************************/
      int ortheg(int na,int n,int ilw,int igh
                ,double a[],double d[],double z[]) {
/*    int ortheg(na,n,ilw,igh,a,d,z)
      int na,n,ilw,igh; double a[],d[],z[]; {                         */
/*    ============================================================    */
      double p,q,s,t,tk;
      int    i,j,k;
      int    *o;                     /* a[i+o[j]] = a(i,j) */
      --a; --d; --z;
      o=(int *)malloc(n*sizeof(int));
      --o; for(k=1; k<=n; k++) o[k]=k*na-na;
/*    ============================================================    */
/*    simularity transform real matrix to upper-hessenberg matrix     */
/*    by householder QR method                                        */
/*    ------------------------------------------------------------    */
/*i   ilw,igh = Part of matrix considered is A(ilw:igh,ilw:igh)       */
/*i   a(n,n)  = [B] = Real matrix                                     */
/*o           = [H] = Upper Hessenberg matrix = [P]'[B][P]            */
/*o           + Information about the orthogonal transformations      */
/*o   d(n)    = Further informations about the transformations        */
/*i   z(n,n)  = [I] or [X] = Matrix to be post-multiplied by [P]      */
/*o           = [P] or [X][P]                                         */
/*    ============================================================    */
      for(k=ilw; k<=igh-2; k++) {
        tk=0.0;
        for(i=k+1; i<=igh; i++) {
          tk=tk+fabs(a[i+o[k]]);
        }
        if(tk == 0.0) continue;
/*      +--------------------------------------------------------+    */
/*      | scale u-vector to avoid overflow {u} = {w}sqrt(2) = {d}|    */
/*      +--------------------------------------------------------+    */
        s=0.0;
        for(i=k+1; i<=igh; i++) {
          d[i]=a[i+o[k]]/tk;
          s=s+d[i]*d[i];
        }
        s=sqrt(s); if(d[k+1] < 0.0) s=-s;
        d[k+1]=d[k+1]+s;
        t=sqrt(s*d[k+1]);
        for(i=k+1; i<=igh; i++) {
          d[i]=d[i]/t;
        }
/*      +--------------------------------------------------------+    */
/*      | compute  [a] = [[i] - 2{w}<w>][a] = [a] - {u}<u>[a]    |    */
/*      +--------------------------------------------------------+    */
        for(j=k+1; j<=n; j++) {
          p=0.0;
          for(i=k+1; i<=igh; i++) {
            p=p+d[i]*a[i+o[j]];
          }
          for(i=k+1; i<=igh; i++) {
            a[i+o[j]]=a[i+o[j]]-d[i]*p;
          }
        }
/*      +--------------------------------------------------------+    */
/*      | compute  [a] = [a][[i] - 2{w}<w>] = [a] - [a]{u}<u>    |    */
/*      +--------------------------------------------------------+    */
        for(i=1; i<=igh; i++) {
          p=0.0;
          for(j=k+1; j<=igh; j++) {
            p=p+a[i+o[j]]*d[j];
          }
          for(j=k+1; j<=igh; j++) {
            a[i+o[j]]=a[i+o[j]]-p*d[j];
          }
        }
        a[k+1+o[k]]=-s*tk;
/*      +--------------------------------------------------------+    */
/*      | compute  [z] = [z][[i] - 2{w}<w>] = [z] - [z]{u}<u>    |    */
/*      +--------------------------------------------------------+    */
        for(i=1; i<=igh; i++) {
          q=0.0;
          for(j=k+1; j<=igh; j++) {
            q=q+z[i+o[j]]*d[j];
          }
          for(j=k+1; j<=igh; j++) {
            z[i+o[j]]=z[i+o[j]]-q*d[j];
          }
        }
        d[k+1]=d[k+1]*tk*t;
      }
      free(++o);
      return(0);
}
/**********************************************************************/
      int hqrids(int na,int n,int ilw,int igh,double h[]
                ,double wr[],double wi[],double z[],int *pierr) {
/*    int hqrids(na,n,ilw,igh,h,wr,wi,z,pierr)
      int na,n,ilw,igh,*pierr; double h[],wr[],wi[],z[]; {            */
/*    ============================================================    */
      double c,p,q,r,s,t,v,w,dd,sm,qp,sp,ps,qs,rs,rp,hsm;
      int    i,j,k,l,iter,i3,j3;
      int    *o;                     /* h[i+o[j]] = h(i,j) */
      --h; --wr; --wi; --z;
      o=(int *)malloc(n*sizeof(int));
      --o; for(k=1; k<=n; k++) o[k]=k*na-na;
/*    ============================================================    */
/*    Upper-Hessenberg ==> Quasi-Upper-Triangular Transformed by      */
/*    QR algorithm with Implicit Double Shifts                        */
/*    Also update the transformation matrix Z(N,N)                    */
/*    ------------------------------------------------------------    */
/*i   h(n,n) = Upper Hessenberg matrix       : [H] = [P]'[B][P]       */
/*o          = Quasi upper triangular matrix : [T] = [V]'[B][V]       */
/*o   wr(n)  = Real part of eigenvalues                               */
/*o   wi(n)  = Imag part of eigenvalues                               */
/*i   z(n,n) = Transformation matrix : [P] for [P]'[B][P]=[H]         */
/*o          = Transformation matrix : [V] for [V]'[B][V]=[T]         */
/*o   ierr   = j: j-th eigenvalue no converge; = 0: normal return     */
/*    ============================================================    */
      *pierr=0;
/*    +----------------------------------------------------------+    */
/*    | get eigenvalues outside (ilw:igh)                        |    */
/*    +----------------------------------------------------------+    */
      for(j=1; j<=n; j++) {
        if(j < ilw || j > igh) {
          wr[j]=h[j+o[j]];
          wi[j]=0.0;
        }
      }
/*    +----------------------------------------------------------+    */
/*    | find h(k-1,k)=0 for isolating h(k:l,k:l)                 |    */
/*    +----------------------------------------------------------+    */
/*    | d  h  h  h  h  h  h  h | 1                               |    */
/*    | h  d  h  h  h  h  h  h |    This program transforms      |    */
/*    |    0  d  h  h  h  h  h | k                               |    */
/*    |       h  d  h  h  h  h |    Upper-Hessenber-matrix       |    */
/*    |          h  d  h  h  h |                                 |    */
/*    |             h  d  h  h | l  ----< to >-----------------  |    */
/*    |                0  d  h |                                 |    */
/*    |                   x  d | n  Quasi-upper-triangular-matrix|    */
/*    +----------------------------------------------------------+    */
/*    do 280 l=igh,ilw,-1/-2                                          */
      l=igh;
 L100:for(iter=1; iter<=30; iter++) {
        for(k=l; k>=ilw+1; k--) {
          dd=fabs(h[k-1+o[k-1]])+fabs(h[k+o[k]]);
          if(dd+fabs(h[k+o[k-1]]) == dd) goto L130;
        }
        k=ilw;
 L130:  if(k == l) {
          wr[k]=h[k+o[k]];                      /* one real root */
          wi[k]=0.0;
          l=l-1;
          goto L280;
        }
/*      +--------------------------------------------------------+    */
/*      || h(l-1,l-1)-x h(l-1,l) | = x*x-2*sm*x+sp = (x-sm)**2-v |    */
/*      ||  h(l,l-1)    h(l,l)-x |   2*sm = x1+x2  ;  sp = x1*x2 |    */
/*      |                        v =-(x1-sm)*(x2-sm) = sm*sm-sp  |    */
/*      +--------------------------------------------------------+    */
        w =(h[l-1+o[l-1]]-h[l+o[l]])*0.5;               /* w =(d-g)/2 */
        sm=w+h[l+o[l]];                                 /* sm=(d+g)/2 */
        sp=h[l-1+o[l-1]]*h[l+o[l]]-h[l+o[l-1]]*h[l-1+o[l]];
        v =w*w+h[l+o[l-1]]*h[l-1+o[l]];                 /* sp=d*g-e*f */
        if(k == l-1) {                          /* v=w*w+e*f=sm*sm-sp */
          if(v < 0.0) {
            wr[k]=sm;                           /* two complex roots */
            wr[l]=sm;
            wi[k]=sqrt(-v);
            wi[l]=-wi[k];
            l=l-2;
            goto L280;
          } else {
            if(w > 0.0) wr[k]=sm+sqrt(v);       /* two real roots */
            else        wr[k]=sm-sqrt(v);
            wr[l]=sp/wr[k];
            wi[k]=0.0;
            wi[l]=0.0;
          }
/*        +--------------------------------------------------------+  */
/*        | perform jacobi transformation for h(l,k) = e ==> 0     |  */
/*        |    | c  s || d  f || c -s | ==> | dd  ff |             |  */
/*        |    |-s  c || e  g || s  c |     |  0  gg |             |  */
/*        |       e*c*c - (d-g)*c*s - f*s*s = 0                    |  */
/*        |    ==>  c/s = ((d-g)+sqrt((d-g)**2+4*e*f)/(2*e)        |  */
/*        +--------------------------------------------------------+  */
          if(w > 0.0) c=w+sqrt(v);    /* transform for two real roots */
          else        c=w-sqrt(v);
          s=h[l+o[k]];
          t=fabs(c)+fabs(s);
          c=c/t;
          s=s/t;
          t=sqrt(c*c+s*s);
          c=c/t;
          s=s/t;
          for(j=k; j<=n; j++) {
            t=h[k+o[j]];
            h[k+o[j]]= c*t+s*h[l+o[j]];
            h[l+o[j]]=-s*t+c*h[l+o[j]];
          }
          for(j=1; j<=l; j++) {
            t=h[j+o[k]];
            h[j+o[k]]= t*c+h[j+o[l]]*s;
            h[j+o[l]]=-t*s+h[j+o[l]]*c;
          }
          for(j=ilw; j<=igh; j++) {
            t=z[j+o[k]];
            z[j+o[k]]= t*c+z[j+o[l]]*s;
            z[j+o[l]]=-t*s+z[j+o[l]]*c;
          }
          l=l-2;
          goto L280;
        }
/*      +--------------------------------------------------------+    */
/*      | double shift qr will add 2 more sub-diagonals          |    */
/*      +--------------------------------------------------------+    */
        for(j=k; j<=l-2; j++) { j3=j+3; if(l < j3) j3=l;
          for(i=j+2; i<=j3; i++) {
            h[i+o[j]]=0.0;
          }
        }
/*      +--------------------------------------------------------+    */
/*      | {p,q,r}*h(k+1,k) = 1st col of [h]**2-2*sm*[h]+sp*[i]   |    */
/*      +--------------------------------------------------------+    */
        hsm=h[k+o[k]]-sm;
        p=(hsm*hsm-v)/h[k+1+o[k]]+h[k+o[k+1]];  /* v=sm*sm-sp */
        q=hsm+(h[k+1+o[k+1]]-sm);               /* sm=(x1+x2)/2 */
        r=h[k+2+o[k+1]];                        /* sp=x1*x2 */
/*      ----------------------------------------------------------    */
        for(i=k; i<=l-1; i++) { i3=i+3; if(l < i3) i3=l;
        t=fabs(p)+fabs(q)+fabs(r);
        if(t != 0.0) {
          p=p/t;
          q=q/t;
          r=r/t;
/*        +------------------------------------------------------+    */
/*        | store new sub-diagonal element in h(i,i-1)           |    */
/*        +------------------------------------------------------+    */
          s=sqrt(p*p+q*q+r*r); if(p < 0) s=-s;
          if(i > k) h[i+o[i-1]]=-s*t;
/*        +------------------------------------------------------+    */
/*        | comput  {u}={p+s,q,r}/s  ;  <v>=<p+s,q,r>/(p+s)      |    */
/*        +------------------------------------------------------+    */
          p=p+s;
          ps=p/s;
          qs=q/s;
          rs=r/s;
          qp=q/p;
          rp=r/p;
          if(i < l-1) {
/*          +----------------------------------------------------+    */
/*          | compute [h] = [[i] - 2{w}<w>][h] = [h] - {u}<v>[a] |    */
/*          +----------------------------------------------------+    */
            for(j=i; j<=n; j++) {
              t=h[i+o[j]]+qp*h[i+1+o[j]]+rp*h[i+2+o[j]];
              h[i  +o[j]]=h[i  +o[j]]-ps*t;
              h[i+1+o[j]]=h[i+1+o[j]]-qs*t;
              h[i+2+o[j]]=h[i+2+o[j]]-rs*t;
            }
/*          +----------------------------------------------------+    */
/*          | compute [h] = [h][[i] - 2{w}<w>] = [h] - [h]{u}<v> |    */
/*          +----------------------------------------------------+    */
            for(j=1; j<=i3; j++) {
              t=h[j+o[i]]*ps+h[j+o[i+1]]*qs+h[j+o[i+2]]*rs;
              h[j+o[i  ]]=h[j+o[i  ]]-t;
              h[j+o[i+1]]=h[j+o[i+1]]-t*qp;
              h[j+o[i+2]]=h[j+o[i+2]]-t*rp;
            }
/*          +----------------------------------------------------+    */
/*          | compute [z] = [z][[i] - 2{w}<w>] = [z] - [z]{u}<v> |    */
/*          +----------------------------------------------------+    */
            for(j=ilw; j<=igh; j++) {
              t=z[j+o[i]]*ps+z[j+o[i+1]]*qs+z[j+o[i+2]]*rs;
              z[j+o[i  ]]=z[j+o[i  ]]-t;
              z[j+o[i+1]]=z[j+o[i+1]]-t*qp;
              z[j+o[i+2]]=z[j+o[i+2]]-t*rp;
            }
          } else {
            for(j=i; j<=n; j++) {
              t=h[i+o[j]]+qp*h[i+1+o[j]];
              h[i  +o[j]]=h[i  +o[j]]-ps*t;
              h[i+1+o[j]]=h[i+1+o[j]]-qs*t;
            }
            for(j=1; j<=i3; j++) {
              t=h[j+o[i]]*ps+h[j+o[i+1]]*qs;
              h[j+o[i  ]]=h[j+o[i  ]]-t;
              h[j+o[i+1]]=h[j+o[i+1]]-t*qp;
            }
            for(j=ilw; j<=igh; j++) {
              t=z[j+o[i]]*ps+z[j+o[i+1]]*qs;
              z[j+o[i  ]]=z[j+o[i  ]]-t;
              z[j+o[i+1]]=z[j+o[i+1]]-t*qp;
            }
          }
        }
/*      +--------------------------------------------------------+    */
/*      | get {p,q,r} for next transformation                    |    */
/*      +--------------------------------------------------------+    */
/*      if(i == l-1) continue;                                        */
        p=h[i+1+o[i]];
        q=h[i+2+o[i]];
        r=0.0;
        if(i+3 <= l) r=h[i+3+o[i]];
        } /* for i  */
      } /* for iter */
      printf("\n Not converge in 30 iterations !");
      *pierr=l;
      free(++o);
      return(0);
 L280:if(l >= ilw) goto L100;
      free(++o);
      return(0);
}
/**********************************************************************/
      int quteve(int na,int n,int ilw,int igh,double h[]
                ,double wr[],double wi[],double z[]) {
/*    int quteve(na,n,ilw,igh,h,wr,wi,z)
      int na,n,ilw,igh; double h[],wr[],wi[],z[]; {                   */
/*    ============================================================    */
      double q,r,s,w,hnorm,epsmah=1.0e-15;
      double r1,s1,w1,qi,xi,qr,xr,wrxr;
      int    i,j,k,l,i1,j1,jigh;
      int    *o;                     /* h[i+o[j]] = h(i,j) */
      int    cdiv();
      --h; --wr; --wi; --z;
      o=(int *)malloc(n*sizeof(int));
      --o; for(k=1; k<=n; k++) o[k]=k*na-na;
/*    ============================================================    */
/*    Find Eigenvectors of Quasi-Upper-Triangular matrix by           */
/*    Solving the singular matrix system equations                    */
/*    ------------------------------------------------------------    */
/*i   h(n,n) = Quasi upper triangular matrix : [T] = [V]'[B][V]       */
/*o          = Transformation matrix : [U] for [U]^{-1}[T][U]=[S]     */
/*i   wr(n)  = Real part of eigenvalues                               */
/*i   wi(n)  = Imag part of eigenvalues                               */
/*i   z(n,n) = [V] : Transformation matrix for     [V]'[B][V]=[T]     */
/*o          = [Z] = [V][U] : Tran. matrix for [Z]^{-1}[B][Z]=[S]     */
/*    ============================================================    */
/*    +----------------------------------------------------------+    */
/*    | compute norm hnorm                                       |    */
/*    +----------------------------------------------------------+    */
      hnorm=0.0;
      for(j=1; j<=n; j++) { j1=j+1; if(n < j1) j1=n;
        for(i=1; i<=j1; i++) {
          hnorm=hnorm+fabs(h[i+o[j]]);
        }
      }
/*    +----------------------------------------------------------+    */
/*    | get  wr(n),wi(n) : eigen values                          |    */
/*    +----------------------------------------------------------+    */
      for(l=n; l>=1; l--) {
        xr=wr[l];
        xi=wi[l];
        if(xi == 0.0) {
/*        +------------------------------------------------------+    */
/*        | eigenvector of real eigenvalue                       |    */
/*        +------------------------------------------------------+    */
          h[l+o[l]]=1.0;
          for(i=l-1; i>=1; i--) {
            w=h[i+o[i]]-xr;
            r=0.0;
            i1=i+1;
            if(wi[i] > 0.0) i1=i+2;
            for(j=i1; j<=l; j++) {
              r=r-h[i+o[j]]*h[j+o[l]];
            }
            if(wi[i] == 0.0) {
              if(w == 0.0) w=hnorm/epsmah;
              h[i+o[l]]=r/w;
            } else if(wi[i] < 0.0) {
              w1=w;
              r1=r;
            } else {
/*            +--------------------------------------------------+    */
/*            | |    w     h(i,i+1) || h(i  ,l) | = | r  |       |    */
/*            | | h(i+1,i)    w1    || h(i+1,l) |   | r1 |       |    */
/*            +--------------------------------------------------+    */
              wrxr=wr[i]-xr;
              q=wrxr*wrxr+wi[i]*wi[i];    /* q=w*w1-h[i,i+1]*h[i+1,i] */
              h[i  +o[l]]=(w1*r-h[i+o[i+1]]*r1)/q;
              h[i+1+o[l]]=(w*r1-h[i+1+o[i]]*r)/q;
            }
          }
        } else if(xi < 0.0) {
/*      +--------------------------------------------------------+    */
/*      | eigenvectors of complex eigenvalues                    |    */
/*      +--------------------------------------------------------+    */
/*      | | hkk-xr+xi*i     hkl      || ukk+ukl*i | = | 0 |      |    */
/*      | |    hlk       hll-xr+xi*i || ulk+ull*i |   | 0 |      |    */
/*      |--------------------------------------------------------|    */
/*      | | 1  0 || ukk+ukl*i | = | -((hll-xr)+xi*i)*i/hlk |     |    */
/*      | | 0  1 || ulk+ull*i |   |              0+1*i     |     |    */
/*      |--------------------------------------------------------|    */
/*      | | 1  0 || ukk+ukl*i | = | -hkl*i/((hkk-xr)+xi*i) |     |    */
/*      | | 0  1 || ulk+ull*i |   |  0+1*i                 |     |    */
/*      +--------------------------------------------------------+    */
          k=l-1;
          if(fabs(h[l+o[k]]) > fabs(h[k+o[l]])) {
            h[k+o[k]]=xi/h[l+o[k]];
            h[k+o[l]]=-(h[l+o[l]]-xr)/h[l+o[k]];
          } else {
            cdiv(0.0,-h[k+o[l]],h[k+o[k]]-xr,xi,&h[k+o[k]],&h[k+o[l]]);
          }
          h[l+o[k]]=0.0;
          h[l+o[l]]=1.0;
/*        --------------------------------------------------------    */
          for(i=l-2; i>=1; i--) {
            w=h[i+o[i]]-xr;
            s=0.0;
            r=0.0;
            i1=i+1;
            if(wi[i] > 0.0) i1=i+2;
            for(j=i1; j<=l; j++) {
              s=s-h[i+o[j]]*h[j+o[k]];
              r=r-h[i+o[j]]*h[j+o[l]];
            }
            if(wi[i] == 0.0) {
              cdiv(s,r,w,xi,&h[i+o[k]],&h[i+o[l]]);
            } else if(wi[i] < 0.0) {
              w1=w;
              s1=s;
              r1=r;
            } else {
/*            +------------------------------------------------------+*/
/*            | w+xi*i  h(i,i+1) || h(i  ,k)+h(i  ,l)i | = | s +r *i |*/
/*            | h(i+1,i) w1+xi*i || h(i+1,k)+h(i+1,l)i |   | s1+r1*i |*/
/*            +------------------------------------------------------+*/
              wrxr=wr[i]-xr;      /* w*w1-h(i,i+1)*h(i+1,i)-xi*xi */
              qr=wrxr*wrxr+wi[i]*wi[i]-xi*xi;
              qi=wrxr*xi*2.0;                        /* (w+w1)*xi */
              cdiv(w1*s-h[i+o[i+1]]*s1-xi*r,w1*r-h[i+o[i+1]]*r1+xi*s
                                      ,qr,qi,&h[i+o[k]],&h[i+o[l]]);
              cdiv(w*s1-h[i+1+o[i]]*s-xi*r1,w*r1-h[i+1+o[i]]*r+xi*s1
                                  ,qr,qi,&h[i+1+o[k]],&h[i+1+o[l]]);
            }
          }
        }
      }
/*    +----------------------------------------------------------+    */
/*    | compute [z] = [v][u] = v(ilw:igh,ilw:igh)*u(ilw:igh,i:n) |    */
/*    +----------------------------------------------------------+    */
      for(i=n; i>=1; i--) {
        if(i >= ilw && i <= igh) continue;
        for(j=i; j<=n; j++) {
          z[i+o[j]]=h[i+o[j]];
        }
      }
      for(j=n; j>=1; j--) { jigh=j; if(igh < jigh) jigh=igh;
        for(i=ilw; i<=igh; i++) {
          s=0.0;
          for(k=ilw; k<=jigh; k++) {
            s=s+z[i+o[k]]*h[k+o[j]];
          }
          z[i+o[j]]=s;
        }
      }
      free(++o);
      return(0);
}
/**********************************************************************/
