#include <math.h>
/**********************************************************************/
      int tridis(int na,int n
                ,double a[],double d[],double e[],double z[]) {
/*    int tridis(na,n,a,d,e,z)
      double a[],d[],e[],z[]; int na,n; {                             */
/*    ============================================================    */
      double s,t,ej,amx,uau,waw,tmp;
      int    i,j,k,ja,ka;
      --a; a-=na; --d; --e; --z; z-=na;
/*    ============================================================    */
/*    simularity transform symmetric matrix to tridiagonal matrix     */
/*    by householder method :    [p]'[a][p] ==> [t]                   */
/*    ------------------------------------------------------------    */
/*i   a(n,n) = symmetric matrix given upper triangular part           */
/*o   d(k)   = t(k,k)   = diagonal elements of tridiagonal matrix     */
/*o   e(k)   = t(k-1,k) = t(k,k-1) = sub-diagonal elements of [t]     */
/*o   z(n,n) = transformation matrix : [p] for [p]'[a][p] = [t]       */
/*    ============================================================    */
      for(j=1; j<=n; j++) { ja=j*na;
        for(i=1; i<=j; i++) {
          z[i+ja]=a[i+ja]; /* change to a[j,i] if          */
        }                  /*  given lower triangular part */
      }
/*    ------------------------------------------------------------    */
      for(k=n; k>=1; k--) { ka=k*na;
        amx=0.0;
        for(i=1; i<=k-1; i++) {
          if((tmp=fabs(z[i+ka])) > amx) amx=tmp;
        }
/*      +--------------------------------------------------------+    */
/*      | in case of a(1:k-1,k)=0, set d(k) & e(k) (also for k=1)|    */
/*      +--------------------------------------------------------+    */
        e[k]=0.0;
        d[k]=z[k+ka];
        z[k+ka]=1.0;
        if(amx == 0.0) continue;
/*      +--------------------------------------------------------+    */
/*      | scale u-vector to avoid overflow, {u} ={w}sqrt(2)= {d} |    */
/*      +--------------------------------------------------------+    */
        s=0.0;
        for(i=1; i<=k-1; i++) {
          d[i]=z[i+ka]/amx;
          s=s+d[i]*d[i];
        }
        s=sqrt(s); if(d[k-1] < 0) s=-s;
        e[k]=-s*amx;
        d[k-1]=d[k-1]+s;
        t=sqrt(s*d[k-1]);
        for(i=1; i<=k-1; i++) {
          d[i]=d[i]/t;
        }
/*      +--------------------------------------------------------+    */
/*      | compute {p} = [a]{u} = [a]{w}sqrt(2) = {e}             |    */
/*      +--------------------------------------------------------+    */
        for(j=1; j<=k-1; j++) { ja=j*na;
          ej=z[j+ja]*d[j];
          for(i=1; i<=j-1; i++) {
            e[i]=e[i]+z[i+ja]*d[j];
            ej  =ej  +z[i+ja]*d[i];
          }
          e[j]=ej;
        }
/*      +--------------------------------------------------------+    */
/*      | compute {q} = {p}-k{u} ; k = <u>{p}/2 = <w>[a]{w} = waw|    */
/*      +--------------------------------------------------------+    */
        uau=0.0;
        for(i=1; i<=k-1; i++) {
          uau=uau+e[i]*d[i];
        }
        waw=uau*0.5;
        for(i=1; i<=k-1; i++) {
          e[i]=e[i]-waw*d[i];
        }
/*      +--------------------------------------------------------+    */
/*      | compute [a] = [a] - {u}<q> - {q}<u> ; set z(*,k) = {u} |    */
/*      +--------------------------------------------------------+    */
        for(j=1; j<=k-1; j++) { ja=j*na;
          for(i=1; i<=j; i++) {
            z[i+ja]=z[i+ja]-d[i]*e[j]-e[i]*d[j];
          }
          z[j+ka]=d[j];
        }
      }
/*    +----------------------------------------------------------+    */
/*    | compute transformation matrix [z] = [zn]...[z3][z2][i]   |    */
/*    +----------------------------------------------------------+    */
      for(k=2; k<=n; k++) { ka=k*na;
/*      +--------------------------------------------------------+    */
/*      | compute  s(1:k-1) = <u>[z]  from  z(*,k) = {u}         |    */
/*      +--------------------------------------------------------+    */
        for(j=1; j<=k-1; j++) { ja=j*na;
          s=0.0;
          for(i=1; i<=k-1; i++) {
            s=s+z[i+ka]*z[i+ja];
          }
/*        +------------------------------------------------------+    */
/*        | compute  [z] = [[i] - 2{w}<w>][z] = [z] - {u}<u>[z]  |    */
/*        +------------------------------------------------------+    */
          for(i=1; i<=k-1; i++) {
            z[i+ja]=z[i+ja]-z[i+ka]*s;
          }
          z[k+ja]=0.0;
        }
        for(i=1; i<=k-1; i++) {
          z[i+ka]=0.0;
        }
      }
      return(0);
}
/**********************************************************************/
