#include <stdio.h>
#include <ctype.h>
#include <math.h>
int freeii(int    *, FILE *);
int freer8(double *, FILE *);
/**********************************************************************/
      int newton(double x[],double y[],int n
                ,double c[],double p[],double f[]) {
/*    int newton(x,y,n,c,p,f) double x[],y[],c[],p[],f[]; int n; {    */
/*    ============================================================    */
      double pm;
      int    i,m;
      --x; --y; --c; --p; --f;
/*    ============================================================    */
/*    Newton's interpolation formula                                  */
/*                                                                    */
/*    f(x(i)) = y(i), for i=1,2,...,n                                 */
/*                                                                    */
/*    f(x) = c(1)*x**(n-1) + c(2)*x**(n-2) + ... + c(n-1)*x + c(n)    */
/*                                                                    */
/*    f(x) = y(1)+(x-x(1))f[x1,x2]+(x-x(1))(x-x(2))f[x1,x2,x3]+...    */
/*    or:                                                             */
/*    f(x) = y(4)+(x-x(4))f[x4,x3]+(x-x(4))(x-x(3))f[x4,x3,x2]+...    */
/*    ============================================================    */
/*    +----------------------------------------------------------+    */
/*    |           f(4)-f(3)              f[x4,3]-f[x3,2]         |    */
/*    | f[x4,3] = ---------; f[x4,3,2] = ---------------         |    */
/*    |            x4 - x3                   x4 - x2             |    */
/*    |                                                          |    */
/*    |                    f[x4,3,2]-f[x3,2,1]                   |    */
/*    |      p[x4,3,2,1] = -------------------                   |    */
/*    |                          x4 - x1                         |    */
/*    |                                                          |    */
/*    | x1 f(x1)                                                 |    */
/*    | x2 f(x2)    f[x2,1]                                      |    */
/*    | x3 f(x3)(3) f[x3,2](2) f[x3,2,1](1)                      |    */
/*    | xm f(x4)(4) f[x4,3](3) f[x4,3,2](2) f[x4,3,2,1](1)       |    */
/*    | x3     f(3)   \   f(i)   \     f(1)   \                  |    */
/*    | xm     f(m)   ->  f(i+1) ->    f(i)   ->      f(1)       |    */
/*    +----------------------------------------------------------+    */
      pm=1.0;
      for(m=1; m<=n; m++) {
        f[m]=y[m];
        for(i=m-1; i>=1; i--) {
          f[i]=(f[i+1]-f[i])/(x[m]-x[i]);
        }
/*      printf("\n %13.4le",x[m]);                                    */
/*      for(i=m; i>=1; i--) printf("%13.4le",f[i]);                   */
/*    +----------------------------------------------------------+    */
/*    | f = c(1)*x**(n-1) + c(2)*x**(n-2) + ... + c(n-1)*x + c(n)|    */
/*    |   = y(1)+(x-x(1))f[x1,x2]+(x-x(1))(x-x(2))f[x1,x2,x3]+...|    */
/*    |                                                          |    */
/*    | p = p(1)*x**(m-1) + p(2)*x**(m-2) + ... + p(m-1)*x + p(m)|    */
/*    |   = (x-x(1))(x-x(2))...(x-x(m-1))                        |    */
/*    +----------------------------------------------------------+    */
        p[m]=pm;
        for(i=m-1; i>=1; i--) {
          c[i+1]=c[i]+p[i+1]*f[1];
          p[i+1]=p[i+1]-p[i]*x[m];
        }
        c[1]=f[1];
        pm=-pm*x[m];
      }
      return(0);
}
/**********************************************************************/
      double polfun(double c[],int n,double x) {
/*    double polfun(c,n,x) double c[],x; int n; {                     */
/*    ============================================================    */
      double fun;
      int    i;
      --c;
/*    ============================================================    */
/*    f(x) = c(1)*x**(n-1) + c(2)*x**(n-2) + ... + c(n-1)*x + c(n)    */
/*    ============================================================    */
      fun=0.0;
      for(i=1; i<=n; i++) {
        fun=fun*x+c[i];
      }
      return(fun);
}
/**********************************************************************/
/*    Test program for Newton's interpolation formula                 */
/*    ============================================================    */
      main(int argc,char **argv) { /* char **envp */
/*    ============================================================    */
      FILE   *fopen(),*fp_r,*fp_w;
      double x[20],y[20],c[20],p[20],f[20],yy;
      int    i,n;
      double polfun(); int newton();
/*    ============================================================    */
      if(argc<3) {printf("Usage : newton infile outfile"); exit(-1);}
      if((fp_r=fopen(argv[1],"r"))==NULL) exit(-1);
      if((fp_w=fopen(argv[2],"w"))==NULL) exit(-1);
/*    ------------------------------------------------------------    */
  L10:freeii(&n,fp_r); if(n == 0 || n > 20) exit(0);
      for(i=0; i<n; i++) {freer8(&x[i],fp_r); freer8(&y[i],fp_r);}
      fprintf(fp_w,"\n         c            p");
      fprintf(fp_w,"            x            y            f");
      newton(x,y,n,c,p,f);
      for(i=0; i<n; i++) {
        fprintf(fp_w,"\n%2d%13.4le%13.4le%13.4le%13.4le%13.4le"
                                  ,i,c[i],p[i],x[i],y[i],f[i]);
      }
      for(i=0; i<n; i++) {
        yy=polfun(c,n,x[i]);
        fprintf(fp_w,"\n%2d%13.4le%13.4le",i,x[i],yy);
      }
      goto L10;
}
/**********************************************************************/
