#include <math.h>
/**********************************************************************/
      int fftr(double a[],double b[],int n,int inv) {
/*    int fftr(a,b,n,inv) double a[],b[]; int n,inv; {                */
/*    ============================================================    */
      double g0,g1,h0,h1,p0,p1,u0,u1,w0,w1,t0,ang,an0r;
      int    i,j; int ffta();
/*    ============================================================    */
/*    fft of real data for any even n                                 */
/*    ------------------------------------------------------------    */
/*    f(0:1,0:n-1) ===> f(0:1,0:n-1) : f(1,i)=0, f(1,0)=f(1,n/2)=0    */
/*                 f(0,n-i)=f(0,i), f(1,n-i)=-f(1,i),  for i=1,n/2    */
/*    ============================================================    */
/*    for inv=+-1 : dimension a(0:2*n-1)                              */
/*    input  : a(2*i)=f(0,i), for i=0,n-1  (others unused)            */
/*    output : a(2*i)=f(0,i), a(2*i+1)=f(1,i), for i=0,n-1            */
/*    ------------------------------------------------------------    */
/*    for inv=+-2 : dimension a(0:n+1)                                */
/*    input  : a(i) = f(0,i), for i=0,n-1                             */
/*    output : a(2*i)=f(0,i), a(2*i+1)=f(1,i), for i=0,n/2            */
/*    ------------------------------------------------------------    */
/*    for inv=+-3 : dimension a(0:n-1)                                */
/*    input  : a(i) = f(0,i), for i=0,n-1                             */
/*    output : a(2*i)=f(0,i), a(2*i+1)=f(1,i), for i=1,n/2-1          */
/*             a(0) = f(0,0), a(1)  =  f(0,n/2)                       */
/*    ============================================================    */
/*    working : b(0:n-1) required for n  !=  2**m only                */
/*    ============================================================    */
      if(abs(inv) != 1) goto L40;
      for(i=0; i<n; i++) {
        a[i]=a[2*i];
      }
/*    ------------------------------------------------------------    */
  L40:ffta(a,b,n/2,inv/abs(inv));
/*    ------------------------------------------------------------    */
      an0r=a[0]-a[1];
      a[0]=a[0]+a[1];
      a[1]=0.0;
/*    ------------------------------------------------------------    */
      ang=-3.1415926535897932*2/n; if(inv < 0) ang=-ang;
      w0=cos(ang);
      w1=sin(ang);
      u0=w0;
      u1=w1;
      j=n-2;
      for(i=2; i<=n/2; i=i+2) {
        h0=(a[i  ]+a[j  ])*0.5;
        g1=(a[j  ]-a[i  ])*0.5;
        g0=(a[i+1]+a[j+1])*0.5;
        h1=(a[i+1]-a[j+1])*0.5;
        p0=g0*u1+g1*u0;
        p1=g1*u1-g0*u0;
        a[i  ]=h0-p1;
        a[j  ]=h0+p1;
        a[i+1]=p0+h1;
        a[j+1]=p0-h1;
        t0=u0;
        u0=t0*w0-u1*w1;
        u1=t0*w1+u1*w0;
        j=j-2;
      }
/*    ------------------------------------------------------------    */
      switch(abs(inv)) {
        case 1:
          j=2*n-2;
          for(i=2; i<n; i=i+2) {
            a[j  ]= a[i  ];
            a[j+1]=-a[i+1];
            j=j-2;
          }
        case 2:
          a[n  ]=an0r;
          a[n+1]=0.0;
          return(0);
        case 3:
          a[1]=an0r;
          return(0);
      }
}
/**********************************************************************/
      int fftc(double a[],double b[],int n,int inv) {
/*    int fftc(a,b,n,inv) double a[],b[]; int n,inv; {                */
/*    ============================================================    */
      double g0,g1,h0,h1,p0,p1,u0,u1,w0,w1,t0,ang,an0r;
      int    i,j; int ffta();
/*    ============================================================    */
/*    fft of conjugate data for any even n                            */
/*    ------------------------------------------------------------    */
/*    f(0:1,0:n-1) <=== f(0:1,0:n-1) : f(1,i)=0, f(1,0)=f(1,n/2)=0    */
/*                 f(0,n-i)=f(0,i), f(1,n-i)=-f(1,i),  for i=1,n/2    */
/*    ============================================================    */
/*    for inv=+-1 : dimension a(0:2*n-1)                              */
/*    input  : a(2*i)=f(0,i), a(2*i+1)=f(1,i), for i=0,n/2            */
/*             note that  :   a(1)=0,  a(n+1)=0,  others unused       */
/*    output : a(2*i)=f(0,i), a(2*i+1)=f(1,i)=0, for i=0,n-1          */
/*    ------------------------------------------------------------    */
/*    for inv=+-2 : dimension a(0:n+1)                                */
/*    input  : a(2*i)=f(0,i), a(2*i+1)=f(1,i), for i=0,n/2            */
/*             note that  :   a(1)=0,  a(n+1)=0                       */
/*    output : a(i) = f(0,i), for i=0,n-1  (f(1,i)=0 not given)       */
/*    ------------------------------------------------------------    */
/*    for inv=+-3 : dimension a(0:n-1)                                */
/*    input  : a(2*i)=f(0,i), a(2*i+1)=f(1,i), for i=1,n/2-1          */
/*             a(0) = f(0,0), a(1)  =  f(0,n/2)                       */
/*    output : a(i) = f(0,i), for i=0,n-1  (f(1,i)=0 not given)       */
/*    ============================================================    */
/*    working : b(0:n-1) required for n  !=  2**m only                */
/*    ============================================================    */
      an0r=a[1];
      if(abs(inv) < 3) an0r=a[n];
      a[1]=a[0]-an0r;
      a[0]=a[0]+an0r;
/*    ------------------------------------------------------------    */
      ang=3.1415926535897932*2/n; if(inv < 0) ang=-ang;
      w0=cos(ang);
      w1=sin(ang);
      u0=w0;
      u1=w1;
      j=n-2;
      for(i=2; i<=n/2; i=i+2) {
        h0=a[i  ]+a[j  ];/*   reverse from:    h0=(a[i  ]+a[j  ])*0.5 */
        p1=a[j  ]-a[i  ];/*                    g1=(a[j  ]-a[i  ])*0.5 */
        p0=a[i+1]+a[j+1];/*  1 <           /   g0=(a[i+1]+a[j+1])*0.5 */
        h1=a[i+1]-a[j+1];/*      \       /     h1=(a[i+1]-a[j+1])*0.5 */
        g0=p0*u1-p1*u0;  /*        \   /       p0=g0*u1+g1*u0         */
        g1=p1*u1+p0*u0;  /*  2 <-----x------   p1=g1*u1-g0*u0         */
        a[i  ]=h0-g1;    /*        /   \       a[i  ]=h0-p1           */
        a[j  ]=h0+g1;    /*      /       \     a[j  ]=h0+p1           */
        a[i+1]=g0+h1;    /*  3 <           \   a[i+1]=p0+h1           */
        a[j+1]=g0-h1;    /*                    a[j+1]=p0-h1           */
        t0=u0;           /*                                           */
        u0=t0*w0-u1*w1;  /*   except *0.5 to be done outside fftc     */
        u1=t0*w1+u1*w0;
        j=j-2;
      }
/*    ------------------------------------------------------------    */
      ffta(a,b,n/2,inv/abs(inv));
/*    ------------------------------------------------------------    */
      if(abs(inv) != 1) return(0);
      for(i=n-1; i>=0; i--) {
        a[2*i+1]=0.0;
        a[2*i  ]=a[i];
      }
      return(0);
}
/**********************************************************************/
      int fftsc(double a[],double b[],int n,int ics) {
/*    int fftsc(a,b,n,inv) double a[],b[]; int n,ics; {               */
/*    ============================================================    */
      double r0,r1,u0,u1,w0,w1,t0,a1,ang;
      int    i; int fftr(),fftch();
/*    ============================================================    */
/*    half-sin or half-cos transform of real data for even n          */
/*    ------------------------------------------------------------    */
/*    input  : a(0:n-1)/a(0:n)/a(0:n-1) = ai() = real dat             */
/*    output : a(0:n-1)/a(0:n)/a(0:n-1) = asin()/acos()/ac()          */
/*    ------------------------------------------------------------    */
/*    ics=+-1: asin(k) = \sum_{j=0}^{n-1} ai(j)*sin(k*j*pi/n)         */
/*    ics=+-2: acos(k) = \sum_{j=1}^{n-1} ai(j)*cos(k*j*pi/n)         */
/*                               + (ai(0)+ai(n)*cos(k*pi))*0.5        */
/*    ics=+3 : ac(k)   = \sum_{j=0}^{n-1} ai(j)*cos(k*(j+.5)*pi/n)    */
/*    ics=-3 : ac(k)   = \sum_{j=1}^{n-1} ai(j)*cos((k+.5)*j*pi/n)    */
/*                                  + 0.5*ai(0)*cos((k+.5)*0*pi/n)    */
/*    ------------------------------------------------------------    */
/*    working : b(0:n-1) required for n  !=  2**m only                */
/*    ============================================================    */
      if(abs(ics) >= 3) {
         fftch(a,b,n,ics);
         return(0);
       }
/*    ------------------------------------------------------------    */
      ang=3.1415926535897932/n;
      w0=cos(ang);
      w1=sin(ang);
/*    ------------------------------------------------------------    */
      u0=w0;
      u1=w1;
      if(abs(ics) == 1) {
        a[0]=0.0;
      } else {
        a1  =(a[0]-a[n])*0.5;
        a[0]=(a[0]+a[n])*0.5;
      }
      for(i=1; i<=n/2; i++) {
        if(abs(ics) == 1) {
          r0=(a[i]-a[n-i])*0.5;
          r1=(a[i]+a[n-i])*u1;
          a[i]  =r1+r0;
          a[n-i]=r1-r0;
        } else {
          r0=(a[i]+a[n-i])*0.5;
          r1=(a[i]-a[n-i])*u1;
          a1=(a[i]-a[n-i])*u0+a1;
          a[i]  =r0-r1;
          a[n-i]=r0+r1;
        }
        t0=u0;
        u0=t0*w0-u1*w1;
        u1=t0*w1+u1*w0;
      }
/*    ------------------------------------------------------------    */
      fftr(a,b,n,-3);
      if(abs(ics) != 1) a[n]=a[1];
      a[1]=0.0;
/*    ------------------------------------------------------------    */
      if(abs(ics) == 1) {
        a1=-0.5*a[0];
        for(i=0; i<=n-2; i=i+2) {
          a1=a1+a[i];
          a[i]=a[i+1];
          a[i+1]=a1;
        }
      } else {
        for(i=1; i<=n-1; i=i+2) {
          a1=a1+a[i];
          a[i]=a1;
        }
      }
      return(0);
}
/**********************************************************************/
      int fftch(double a[],double b[],int n,int ics) {
/*    int fftch(a,b,n,inv) double a[],b[]; int n,ics; {               */
/*    ============================================================    */
      double r0,r1,u0,u1,w0,w1,t0,aj,dj,an,ang;
      int    i; int fftc(),fftr();
/*    ============================================================    */
/*    half-sin or half-cos transform of real data for even n          */
/*    ------------------------------------------------------------    */
/*    input  : a(0:n-1) = ai(0:n-1) = real data                       */
/*    output : a(0:n-1) = ac(0:n-1)                                   */
/*    ------------------------------------------------------------    */
/*    ics=+3 : ac(k)   = \sum_{j=0}^{n-1} ai(j)*cos(k*(j+.5)*pi/n)    */
/*    ics=-3 : ac(k)   = \sum_{j=1}^{n-1} ai(j)*cos((k+.5)*j*pi/n)    */
/*                                  + 0.5*ai(0)*cos((k+.5)*0*pi/n)    */
/*    ------------------------------------------------------------    */
/*    working : b(0:n-1) required for n  !=  2**m only                */
/*    ============================================================    */
      ang=3.1415926535897932/n;
      w0=cos(ang);
      w1=sin(ang);
/*    ------------------------------------------------------------    */
      if(ics > 0) {
        u0=cos(0.5*ang);
        u1=sin(0.5*ang);
        for(i=0; i<n/2; i++) {
          r0=(a[i]+a[n-1-i])*0.5;
          r1=(a[i]-a[n-1-i])*u1;
          a[i]    =r0-r1;
          a[n-1-i]=r0+r1;
          t0=u0;
          u0=t0*w0-u1*w1;
          u1=t0*w1+u1*w0;
        }
/*      ----------------------------------------------------------    */
        fftr(a,b,n,-3);
        an=a[1];
        a[1]=0.0;
/*      ----------------------------------------------------------    */
        aj=-0.5*an;
        u0=w0;
        u1=w1;
        for(i=n-2; i>=0; i=i-2) {
          dj    =a[i]*u0+a[i+1]*u1;
          a[i]  =a[i]*u1-a[i+1]*u0;
          a[i+1]=aj;
          aj=aj-dj;
          t0=u0;
          u0=t0*w0-u1*w1;
          u1=t0*w1+u1*w0;
        }
        return(0);
/*    ------------------------------------------------------------    */
      } else {
        an=a[n-1];
        u0=w0;
        u1=w1;
        for(i=n-2; i>=2; i=i-2) {
          dj    =a[i+1]-a[i-1];
          a[i+1]=dj*u1-a[i]*u0;
          a[i]  =dj*u0+a[i]*u1;
          t0=u0;
          u0=t0*w0-u1*w1;
          u1=t0*w1+u1*w0;
        }
/*      ----------------------------------------------------------    */
        a[1]=-2.0*an;
        fftc(a,b,n,3);
/*      ----------------------------------------------------------    */
        u0=cos(0.5*ang);
        u1=sin(0.5*ang);
        for(i=0; i<n/2; i++) {
          r0=(a[i]+a[n-1-i])*0.25;
          r1=(a[i]-a[n-1-i])*0.125/u1;
          a[i]    =r0-r1;
          a[n-1-i]=r0+r1;
          t0=u0;
          u0=t0*w0-u1*w1;
          u1=t0*w1+u1*w0;
        }
        return(0);
      }
}
/**********************************************************************/
      int fftcs(double a[],double b[],int n,int ics) {
/*    int fftcs(a,b,n,inv) double a[],b[]; int n,ics; {               */
/*    ============================================================    */
      int    i; int fftsc();
/*    ============================================================    */
/*    sin or cos transform of real data for even n                    */
/*    ------------------------------------------------------------    */
/*    input  : a(0:n-1) = ai(0:n-1) = real data                       */
/*    output : a(0:n-1) = asin(0:n-1) or acos(0:n-1)                  */
/*    ------------------------------------------------------------    */
/*    ics=+-1: asin(k) = \sum_{j=0}^{n-1} ai(j)*sin(k*j*2pi/n)        */
/*    ics=+-2: acos(k) = \sum_{j=0}^{n-1} ai(j)*cos(k*j*2pi/n)        */
/*    ------------------------------------------------------------    */
/*    working : b(0:n/2-1) required for n  !=  2**m only              */
/*    ============================================================    */
      if(abs(ics) == 1) {
        a[0]=0.0;
        for(i=1; i<n/2; i++) {
          a[i]=a[i]-a[n-i];
        }
      } else {
        a[0]=a[0]+a[0];
        for(i=1; i<n/2; i++) {
          a[i]=a[i]+a[n-i];
        }
        a[n/2]=a[n/2]+a[n/2];
      }
/*    ------------------------------------------------------------    */
      fftsc(a,b,n/2,ics);
/*    ------------------------------------------------------------    */
      if(abs(ics) == 1) {
        a[n/2]=0.0;
        for(i=1; i<n/2; i++) {
          a[n-i]=-a[i];
        }
      } else {
        for(i=1; i<n/2; i++) {
          a[n-i]=a[i];
        }
      }
      return(0);
}
/**********************************************************************/
