#include <math.h>
/**********************************************************************/
      double xmin(double x,double f,double dx
                 ,int istep,int *ivy,double b[]) {
/*    double xmin(x,f,dx,istep,ivy,b)
      double x,f,dx,b[]; int istep,*ivy; {                             */
/*    ============================================================    */
      double xx,ff,ddx,ddf,rate,pmax=1.0e30;
      double x1,x2,x3,f1,f2,f3,xl,xm,xr,fl,fm,fr,st,vy;
      int    i,iget;
/*    ============================================================    */
/*    find  xmin                                                      */
/*    ------------------------------------------------------------    */
/*    this program combines 3 methods to get the best guess of x :    */
/*    (1) from quadratic eq. of newest_3_points (if concave_upward)   */
/*        but the max_step_size is set to 3.75*previous_step_size     */
/*    (2) from quadratic eq. of valley_set_of_3_points (if any)       */
/*        when (1)_is_NA or x_from_(1) outside the bounds_of_valley   */
/*    (3) step on the downward_side twice the previous_step_size      */
/*        when both (1)&(2)_are_NA                                    */
/*    method(1) makes the program so fast as quadratic convergence    */
/*    method(2) makes the program so stable as global convergence     */
/*    method(3) makes the program so insensitive to init_step_size    */
/*    ------------------------------------------------------------    */
/*    xmin  = new guess of x for minimizing f(x)                      */
/*    x     = argument of function                                    */
/*    f     = f(x) = function value of x                              */
/*    dx    = return xmin=x+dx if no better formula to find xmin      */
/*    istep = number of call to function xmin                         */
/*    ivy   = 1/0 if valley set is found/not found                    */
/*    b(14) = buffer must not be changed between each calling istep   */
/*    ------------------------------------------------------------    */
/*    for istep = 1  :  (x1=x,f1=f) (st=1) (vy=0)                     */
/*        xmin  = x1 + dx   if dx != 0                                */
/*        xmin  = x1 + 0.1  if dx == 0                                */
/*    for istep = 2  :  (x2=x,f2=f) (st=2) (vy=0)                     */
/*        xmin  = x1 + (x1-x2)*2  if f1<f2                            */
/*        xmin  = x2 + (x2-x1)*2  if f2<f1                            */
/*    for istep > 2  :  (x3=x,f3=f) (st>2) (vy=1 if found valley)     */
/*        xmin  = y1  from quadratic eq. of newest point (x1,x2,x3)   */
/*        xmin  = y2  from quadratic eq. of valley set (xl,xm,xr)     */
/*                    if vy=1 and when y1 is na or outside (xl,xr)    */
/*    ============================================================    */
      if(istep == 1) {
        x1=pmax; x2=pmax; x3=pmax; f1=pmax; f2=pmax; f3=pmax;
        xl=pmax; xm=pmax; xr=pmax; fl=pmax; fm=pmax; fr=pmax;
        st=0; vy=0;
      } else {
        x1=b[0]; x2=b[1]; x3=b[2]; f1=b[3]; f2=b[4];  f3=b[5];
        xl=b[6]; xm=b[7]; xr=b[8]; fl=b[9]; fm=b[10]; fr=b[11];
        st=b[12]; vy=b[13];
      }
      st=st+1;
/*    +----------------------------------------------------------+    */
/*    | save  the newest 3 points to (x1,x2,x3)                  |    */
/*    +----------------------------------------------------------+    */
      x1=x2;
      x2=x3;
      x3=x;
      f1=f2;
      f2=f3;
      f3=f;
/*    +----------------------------------------------------------+    */
/*    | find  new points (xl,xm,xr) that bracket the root        |    */
/*    +----------------------------------------------------------+    */
      if(x > xm) {
        if(f > fm || (f == fm && fr > fl)) {
          xr=x;
          fr=f;
        } else {
          xl=xm;
          xm=x;
          fl=fm;
          fm=f;
        }
      } else if(x < xm) {
        if(f > fm || (f == fm && fl > fr)) {
          xl=x;
          fl=f;
        } else {
          xr=xm;
          fr=fm;
          xm=x;
          fm=f;
        }
      }
      if(fl < pmax && fr < pmax) vy=1;
      iget=0;
      xx=0.0;
/*    +----------------------------------------------------------+    */
/*    | find a new point from qradratic equation on (x1,x2,x3)   |    */
/*    | if it is concave upward  d2f/dx2 = 1/ddf > 0             |    */
/*    | and make sure that abs(xx-(x3+x1)/2) < 3*abs(x3-x1)      |    */
/*    |                                                          |    */
/*    | x1  x2      x3    <---3.75*(x3-x2)--->    xxmax          |    */
/*    |  *---*---+---*---+---x---+---x---+---x---+-*             |    */
/*    |        *---+---+---x---+---+---x---+---+---*             |    */
/*    |        0        <---3*(x3-x1)--->        -ddx            |    */
/*    +----------------------------------------------------------+    */
      if(st > 2) {
        ff=(f3-f2)*(x1-x2)-(f1-f2)*(x3-x2);
        if(ff != 0) {
          ddf=(x3-x1)*(x3-x2)*(x1-x2)/ff;
          if(ddf > 0) {
            ddx=0.5*(f3-f1)/(x3-x1)*ddf;
            rate=fabs(ddx/(x3-x1))/3.0;
            if(rate <= 1) rate=1.0;
            xx=0.5*(x3+x1)-ddx/rate;
            iget=4;
          }
        }
      }
/*    +----------------------------------------------------------+    */
/*    | if minimum is bracketed by (xl,xm,xr) and if no xx get   |    */
/*    | or xx goes outside (xl,xr) {    use quadratic equation   |    */
/*    +----------------------------------------------------------+    */
      if(vy != 0) {
        if(iget == 0 || (xx-xl)*(xx-xr) > 0) {
          ff=(fr-fm)*(xl-xm)-(fl-fm)*(xr-xm);
          xx=0.5*(xr+xl);
          if(ff != 0) xx=xx-0.5*(fr-fl)*(xr-xm)*(xl-xm)/ff;
          iget=3;
        }
      }
/*    +----------------------------------------------------------+    */
/*    | if none xx get so far, then:                             |    */
/*    | if known at least 2 points, go the downward side         |    */
/*    | if known only one point, set xx=x+dx or xx=x+0.1         |    */
/*    +----------------------------------------------------------+    */
      if(iget == 0) {
        if(st >= 2) {
          if(f3 > f2) {
            xx=x2;
            x2=x3;
            x3=xx;
            xx=f2;
            f2=f3;
            f3=xx;
          }
          xx=x3+(x3-x2)*2.0;
          iget=2;
        } else {
          if(dx != 0) {
            xx=x+dx;
          } else {
            xx=x+0.1;
          }
          iget=1;
        }
      }
/*    ------------------------------------------------------------    */
      b[0]=x1; b[1]=x2; b[2]=x3; b[3]=f1; b[4]=f2;  b[5]=f3;
      b[6]=xl; b[7]=xm; b[8]=xr; b[9]=fl; b[10]=fm; b[11]=fr;
      b[12]=st; b[13]=vy;
      *ivy=vy;
      return(xx);
}
/**********************************************************************/
