#include <math.h>
/**********************************************************************/
      int rungec(int n,int (*ff)(),double dt,double *pt
                ,double yi[],double yo[],double x[]
                ,int mmax,int *pmuse,double epsreq,double *pepsold) {
/*    int rungec(n,ff,dt,t,yi,yo,x,mmax,muse,epsreq,epsold)
      double dt,t,yi[],yo[],x[],epsreq,*pepsold;
      int    n,(*ff)(),nmax,*pmuse;                                   */
/*    ============================================================    */
      double t,t1,t2,ht,tmp,epsold,*x1,*x2,*x3,*x4,*x5,*x6,*x7;
      int    i,istep,nstep,muse; int runge();
      x1=x; x2=x1+n; x3=x2+n; x4=x3+n; x5=x4+n; x6=x5+n; x7=x6+n;
      t=*pt;
/*    ============================================================    */
/*    input t,yi(n) at t=ti;          output t,yo(n) at t=to=ti+dt    */
/*    also output fi(n)=f_i(yi,ti) in x(n,1)                          */
/*    yi(n) & yo(n) can be the same location : loc.yo(n)=loc.yi(n)    */
/*    ------------------------------------------------------------    */
/*i   n      = number of dependent variables                          */
/*s   ff     = subroutine to compute f(n) for given t & y(n)          */
/*i   dt     = time_step = final_time - initial_time                  */
/*o   t      = initial time ti & final time to=ti+dt                  */
/*i   yi(n)  = y_i(ti)                                                */
/*o   yo(n)  = y_i(to)                                                */
/*w   x(n,7) = buffers x(4*n) if mmax=0; buffers x(7*n) if mmax>0     */
/*i   mmax   = index for accuracy control                             */
/*           = 0: no control on accuracy (epsreq,epsold,muse:na)      */
/*           > 0: control accuracy with allowed max_steps = 2**mmax   */
/*o   muse   = actual used_steps = 2**muse                            */
/*i   epsreq = req. epsilon such that epsold < epsreq                 */
/*o   epsold = max. epsilon = max_{i=1}^n |1-y2(i)/y1(i)|/15          */
/*    to adjust dt : dt_new = (epsreq/epsold)**(1/4) * dt / 2**muse   */
/*    ============================================================    */
      ff(t,yi,x1,n);
/*    ------------------------------------------------------------    */
      if(mmax <= 0) {
        runge(n,ff,dt,&t,yi,yo,x1,x2,x3,x4);
/*      ff(t,yo,x2,n)                                                 */
        *pt=t; return(0);
      }
/*    +----------------------------------------------------------+    */
/*    | compute y2(n) = x(n,5) by large_step_size = 2*h          |    */
/*    +----------------------------------------------------------+    */
      t2=t;
      runge(n,ff,dt,&t2,yi,x5,x1,x2,x3,x4);
/*    +----------------------------------------------------------+    */
/*    | compute y1(n) = x(n,6) by small_step_size = h = dt/nstep |    */
/*    +----------------------------------------------------------+    */
      nstep=1;
      for(muse=1; muse<=mmax; muse++) {
        nstep=nstep*2;
        ht=dt/nstep;
/*      +--------------------------------------------------------+    */
/*      | restart nstep of ht = h = dt/nstep                     |    */
/*      +--------------------------------------------------------+    */
        t1=t;
        runge(n,ff,ht,&t1,yi,x6,x1,x2,x3,x4);
        for(istep=2; istep<=nstep; istep++) {
          ff(t1,x6,x7,n);
          runge(n,ff,ht,&t1,x6,x6,x7,x2,x3,x4);
        }
/*      +--------------------------------------------------------+    */
/*      | find the max. reletive error = |1 - y2(i))/y1(i)|/15     |    */
/*      +--------------------------------------------------------+    */
        epsold=0.0;
        for(i=0; i<n; i++) {
          if(x6[i] != 0) {
            if((tmp=fabs(1.0-x5[i]/x6[i])) > epsold) epsold=tmp;
          }
        }
        epsold=epsold/15.0;
/*      +--------------------------------------------------------+    */
/*      | if epsold >  epsreq : double the number of steps       |    */
/*      | if epsold <= epsreq : return yo(i)=y1(i)+(y1(i)-y2(i))/15   */
/*      +--------------------------------------------------------+    */
        if(epsold > epsreq && muse < mmax) {
          for(i=0; i<n; i++) {
            x5[i]=x6[i];
          }
        } else {
          t=t+dt;
          for(i=0; i<n; i++) {
            yo[i]=x6[i]+(x6[i]-x5[i])/15;
          }
/*        ff(t,yo,x2,n)                                               */
          *pt=t; *pmuse=muse; *pepsold=epsold; return(0);
        }
      }
      *pt=t; *pmuse=muse; *pepsold=epsold; return(0);
}
/**********************************************************************/
      int runge(int n,int (*ff)(),double dt,double *pt
               ,double yi[],double yo[]
               ,double fi[],double fo[],double fc[],double yc[]) {
/*    int runge(n,ff,dt,t,yi,yo,fi,fo,fc,yc)
      double dt,t,yi[],yo[],fi[],fo[],fc[],yc[]; int n,(*ff)(); {     */
/*    ============================================================    */
      double t,ht;
      int    i;
      t=*pt;
/*    ------------------------------------------------------------    */
/*    input  t,yi(n),fi(n) at t=ti                                    */
/*    output t,yo(n)       at t=to=ti+dt                              */
/*    yi(n) & yo(n) can be the same location : loc.yo(n)=loc.yi(n)    */
/*    ------------------------------------------------------------    */
/*i   n      = number of dependent variables                          */
/*s   ff     = subroutine to compute f(n) for given t & y(n)          */
/*i   dt     = time_step = final_time - initial_time                  */
/*o   t      = initial time ti & final time to=ti+dt                  */
/*i   yi(n)  = y_i(ti)                                                */
/*o   yo(n)  = y_i(to)                                                */
/*i   fi(n)  = f_i(yi,ti)                                             */
/*w   fo(n)  = working array for fa(n),fa(n)+fb(n)                    */
/*w   fc(n)  = working array for       fb(n),fc(n)                    */
/*w   yc(n)  = working array for ya(n),yb(n),yc(n)                    */
/*    ============================================================    */
      ht=dt*0.5;
      t=t+ht;
      for(i=0; i<n; i++) {
        yc[i]=yi[i]+fi[i]*ht;
      }
      ff(t,yc,fo,n);
/*    ------------------------------------------------------------    */
      for(i=0; i<n; i++) {
        yc[i]=yi[i]+fo[i]*ht;
      }
      ff(t,yc,fc,n);
/*    ------------------------------------------------------------    */
      t=t+ht;
      for(i=0; i<n; i++) {
        yc[i]=yi[i]+fc[i]*dt;
        fo[i]=fo[i]+fc[i];
      }
      ff(t,yc,fc,n);
/*    ------------------------------------------------------------    */
      ht=dt/6.0;
      for(i=0; i<n; i++) {
        yo[i]=yi[i]+(fi[i]+fo[i]+fo[i]+fc[i])*ht;
      }
/*    ff(t,yo,fo,n);                                                  */
      *pt=t; return(0);
}
/**********************************************************************/
