#include <math.h>
/**********************************************************************/
      int ffttrr(double w,double e,double yst,double y[]
                ,double dt,int n,double u[],double u0,double v0) {
/*    int ffttrr(w,e,yst,y,dt,n,u,u0,v0)
      double w,e,yst,y[],dt,u[],u0,v0; int n; {                       */
/*    ============================================================    */
/*    double y[0:n-1],u[0:n+1];                                       */
      double o,d,h1,h2,u1,hh,dw,uo,vo,uu,vv,edt,wdt,ewdt,wddt;
      int    i; int fftc(),fftr();
/*    ============================================================    */
/*    this subroutine computes transient response of the equation     */
/*    rel.a(i) + 2*e*w*v(i) + w*w*u(i) = w*w*yst*y(i)                 */
/*    with initial conditions :   u(0)=u0,   v(0)=v0                  */
/*    and n=2**m by fast fourier transform.  (v=du/dt, a=dv/dt)       */
/*    ------------------------------------------------------------    */
/*i   w    = natural frequence of the system                          */
/*i   e    = damping ratio of the system                              */
/*i   yst  = scale constant                                           */
/*         > 0 : for transient response                               */
/*         < 0 : for steady-state response                            */
/*i   y(i) = input force divided by w*w*yst at t=i*dt                 */
/*i   dt   = time interval                                            */
/*i   n    = number of data points                                    */
/*o   u(i) = output response at t=i*dt                                */
/*i   u0   = initial displacement at t=0                              */
/*i   v0   = initial velocity at t=0                                  */
/*    ------------------------------------------------------------    */
/*    array u(n+2) may be equivalence to y(n).                        */
/*    if n  !=  2**integer then array u() must has length 2*n+2       */
/*    ============================================================    */
      for(i=0; i<n; i++) {
        u[i]=y[i]*yst*w*w;
      }
/*    +----------------------------------------------------------+    */
/*    | perform fourier transform by fft(real)                   |    */
/*    +----------------------------------------------------------+    */
      fftr(u,&u[n+2],n,2);
/*    +----------------------------------------------------------+    */
/*    | multiple by transfer function h(i*dw) and divided by n   |    */
/*    +----------------------------------------------------------+    */
      dw=6.2831853071795864/(dt*n);
      for(i=0; i<=n/2; i++) {
        h1= w*w-(i*dw)*(i*dw);
        h2=-2.*e*w*dw*i;
        hh=(h1*h1+h2*h2)*n;
        u1=u[2*i];
        u[2*i  ]=(u1*h1-u[2*i+1]*h2)/hh;
        u[2*i+1]=(u1*h2+u[2*i+1]*h1)/hh;
      }
      u[n+1]=0.0;
/*    +----------------------------------------------------------+    */
/*    | for yst<0 : compute steady-state response by inverse fft |    */
/*    +----------------------------------------------------------+    */
      if(yst < 0) {
        fftc(u,&u[n+2],n,-2);
        return(0);
      }
/*    +----------------------------------------------------------+    */
/*    | for yst>0 : get initial value of steady-state response   |    */
/*    +----------------------------------------------------------+    */
      uo=0.0;
      vo=0.0;
      for(i=1; i<n/2; i++) {
        uo=uo+u[2*i];
        vo=vo-u[2*i+1]*dw*i;
      }
      uo=uo+uo+u[0]+u[n];
      vo=vo+vo;
/*    +----------------------------------------------------------+    */
/*    | then : compute the steady-state response by inverse fft  |    */
/*    +----------------------------------------------------------+    */
      fftc(u,&u[n+2],n,-2);
/*    +----------------------------------------------------------+    */
/*    | and : adjust i.c. to u0 and v0 for transient response    |    */
/*    +----------------------------------------------------------+    */
      uu=u0-uo;
      vv=(v0-vo)/w;
      d=sqrt(1.0-e*e);
      o=atan(e/d);
      edt=1.0/d;
      wddt=w*d*dt;
      ewdt=exp(-e*w*dt);
/*    ------------------------------------------------------------    */
      for(i=0; i<n; i++) {
        wdt=wddt*i;
        u[i]=u[i]+edt*(uu*cos(wdt-o)+vv*sin(wdt));
        edt=edt*ewdt;
      }
      return(0);
}
/**********************************************************************/
