#include <math.h>
/**********************************************************************/
      int bfgslg(int n,double xc[],double xp[],double gc[],double gp[]
                ,double epsmah,double eta,double angrad
                ,double d[],int l[],double h[]
                ,double s[],double y[],double t[],double vm) {
/*    int bfgslg(n,xc,xp,gc,gp,epsmah,eta,angrad,d,l,h,s,y,t,vm)
      double xc[],xp[],gc[],gp[],d[],h[],s[],y[],t[];
      double epsmah,eta,angrad,vm; int n,l[]; {                       */
/*    ============================================================    */
      double ss,tt,yy,ts,ys; int i,j;
/*    ============================================================    */
/*    update hp from hc                                               */
/*    ------------------------------------------------------------    */
/*i   n      = number of variables                                    */
/*i   xc(n)  = last point                                             */
/*i   xp(n)  = this point                                             */
/*i   gc(n)  = gradient at xc                                         */
/*i   gp(n)  = gradient at xp                                         */
/*i   epsmah = machine epsilon                                        */
/*    eta,angrad = dummy (unused in this subroutine)                  */
/*i   d(n)   = diagonal matrix for adaptive lagrange multiplier       */
/*i   l(n)   = location index of matrix h                             */
/*i   h(n,n) = hessian matrix hc                                      */
/*o          = hessian matrix hp                                      */
/*w   s(n)   = xp(n)-xc(n)  -> sqrt((1-vm)*tts)*(y(n)/yts-t(n)/tts)   */
/*w   y(n)   = gp(n)-gc(n)  -> y(n)/sqrt(yts)                         */
/*w   t(n)   = hc(n,n)*s(n) -> t(n)/sqrt(tts)                         */
/*i   vm     = bfgs/(dfp+bfgs) : proportion of update methods         */
/*             h = h_bfgs + (1-vm) * (h_dfp-h_bfgs)                   */
/*    hp     = hc + y(i)*y(j)/yts - t(i)*t(j)/tts                     */
/*             +(1-vm)*tts*(y(i)/yts-t(i)/tts)*(y(j)/yts-t(j)/tts)    */
/*    ============================================================    */
      ss=0.0;
      yy=0.0;
      ys=0.0;
      for(i=0; i<n; i++) {
        s[i]=xp[i]-xc[i];
        y[i]=gp[i]-gc[i] + d[i]*s[i];
        ss=ss+s[i]*s[i];
        yy=yy+y[i]*y[i];
        ys=ys+y[i]*s[i];
      }
      if(fabs(ys) <= epsmah*sqrt(ss*yy)) return(0);
/*    ------------------------------------------------------------    */
      tt=0.0;
      ts=0.0;
      for(i=0; i<n; i++) {
        t[i]=0.0 + d[i]*s[i];
        for(j=0; j<=i; j++) {
          t[i]=t[i]+h[j+l[i]]*s[j];
        }
        for(j=i+1; j<n; j++) {
          t[i]=t[i]+h[i+l[j]]*s[j];
        }
        tt=tt+t[i]*t[i];
        ts=ts+t[i]*s[i];
      }
      if(fabs(ts) <= epsmah*sqrt(tt*ss)) return(0);
/*    ------------------------------------------------------------    */
      ys=sqrt(ys);
      ts=sqrt(ts);
      for(i=0; i<n; i++) {
        y[i]=y[i]/ys;
        t[i]=t[i]/ts;
        s[i]=(y[i]*ts/ys-t[i])*sqrt(1-vm);
      }
/*    ------------------------------------------------------------    */
      for(j=0; j<n; j++) {
        for(i=0; i<=j; i++) {
          h[i+l[j]]=h[i+l[j]]+y[i]*y[j]-t[i]*t[j]+s[i]*s[j];
        }
      }
      return(0);
}
/**********************************************************************/
      int bfgs(int n,double xc[],double xp[],double gc[],double gp[]
              ,double epsmah,double eta,double angrad,int l[],double b[]
              ,double s[],double y[],double t[],double vm) {
/*    int bfgs(n,xc,xp,gc,gp,epsmah,eta,angrad,l,b,s,y,t,vm)
      double xc[],xp[],gc[],gp[],b[],s[],y[],t[];
      double epsmah,eta,angrad,vm; int n,l[]; {                       */
/*    ============================================================    */
      double ss,tt,yy,sy,ty; int i,j;
/*    ============================================================    */
/*    update bp from bc  (b is the inverse of hessian matrix h)       */
/*    ------------------------------------------------------------    */
/*i   n      = number of variables                                    */
/*i   xc(n)  = last point                                             */
/*i   xp(n)  = this point                                             */
/*i   gc(n)  = gradient at xc                                         */
/*i   gp(n)  = gradient at xp                                         */
/*i   epsmah = machine epsilon                                        */
/*    eta,angrad = dummy (unused in this subroutine)                  */
/*i   l(n)   = location index of matrix b                             */
/*i   b(n,n) = inverse of hessian matrix hc                           */
/*o          = inverse of hessian matrix hp                           */
/*w   y(n)   = gp(n)-gc(n)  -> sqrt(  vm * tty)*(s(n)/sty-t(n)/tty)   */
/*w   s(n)   = xp(n)-xc(n)  -> s(n)/sqrt(sty)                         */
/*w   t(n)   = bc(n,n)*y(n) -> t(n)/sqrt(tty)                         */
/*i   vm     = bfgs/(dfp+bfgs) : proportion of update methods         */
/*             b = b_dfp + vm * (b_bfgs-b_dfp)                        */
/*    bp     = bc + s(i)*s(j)/sty - t(i)*t(j)/tty                     */
/*                + vm*tty*(s(i)/sty-t(i)/tty)*(s(j)/sty-t(j)/tty)    */
/*    ============================================================    */
/*    for(i=0; i<5; i++) t[i]=0.0;                                    */
/*    bfgslg(n,gc,gp,xc,xp,epsmah,eta,angrad,t,l,b,y,s,t,1-vm);       */
/*    return(0);                                                      */
      ss=0.0;
      yy=0.0;
      sy=0.0;
      for(i=0; i<n; i++) {
        s[i]=xp[i]-xc[i];
        y[i]=gp[i]-gc[i];
        ss=ss+s[i]*s[i];
        yy=yy+y[i]*y[i];
        sy=sy+s[i]*y[i];
      }
      if(fabs(sy) <= epsmah*sqrt(ss*yy)) return(0);
/*    ------------------------------------------------------------    */
      tt=0.0;
      ty=0.0;
      for(i=0; i<n; i++) {
        t[i]=0.0;
        for(j=0; j<=i; j++) {
          t[i]=t[i]+b[j+l[i]]*y[j];
        }
        for(j=i+1; j<n; j++) {
          t[i]=t[i]+b[i+l[j]]*y[j];
        }
        tt=tt+t[i]*t[i];
        ty=ty+t[i]*y[i];
      }
      if(fabs(ty) <= epsmah*sqrt(tt*yy)) return(0);
/*    ------------------------------------------------------------    */
      sy=sqrt(sy);
      ty=sqrt(ty);
      for(i=0; i<n; i++) {
        s[i]=s[i]/sy;
        t[i]=t[i]/ty;
        y[i]=(s[i]*ty/sy-t[i])*sqrt(vm);
      }
/*    ------------------------------------------------------------    */
      for(j=0; j<n; j++) {
        for(i=0; i<=j; i++) {
          b[i+l[j]]=b[i+l[j]]+s[i]*s[j]-t[i]*t[j]+y[i]*y[j];
        }
      }
      return(0);
}
/**********************************************************************/
