#include <math.h>
/**********************************************************************/
      int fdgrad(int nx,int ns,int ne,int n,double x[],double pfac[]
         ,double ppen[],double c[],double *fp,double eta,double g[]) {
/*    int fdgrad(nx,ns,ne,n,x,pfac,ppen,c,fp,eta,g)
      double x[],pfac[],ppen[],c[],*fp,eta,g[]; int nx,ns,ne,n; {     */
/*    ============================================================    */
      double xp,fj,etah; int j,nc; int gfnlag(),gfnpn0(),gfnpn1();
/*    ============================================================    */
/*    compute gradient g(n) by difference & partial analysis          */
/*    ------------------------------------------------------------    */
/*i   nx       = number of original variables                         */
/*i   ns       = number of constraints gj(xp(nx)) <= 0                */
/*i   ne       = number of constraints hi(xp(nx))  = 0                */
/*i   n        = nx            : for penalty function                 */
/*i            = nx+ns+ns+ne   : for lagrange multiplier              */
/*i   x(n)     = xp(nx)[,s(ns),ls(ns),le(ne)]                         */
/*i   pfac(nx) = scalar factor of xp(nx)                              */
/*i   ppen(6)  = r(1),r(2),r(3): penalty parameters                   */
/*i              t,c1,c2       : hyperbolic penalty function coef.    */
/*w   c(2*nc)  = gj(xp(nx)),hi(xp(nx)),dysj/dgj,dyei/dhi : penalty    */
/*w   c(ns+ne) = gj(xp(nx)),hi(xp(nx))                   : lagrange   */
/*o   fp       = f(x(n))                                              */
/*i   eta      = machine epsilon                                      */
/*o   g(n)     = gradient at x(n)                                     */
/*s   gfnpn0,1 = sub. to compute diff. penalty obj. func. f(x(n))     */
/*s   gfnlag   = sub. to compute lagrange objective func. f(x(n))     */
/*    ============================================================    */
      nc=ns+ne;
      etah=sqrt(eta);
      if(n == nx) gfnpn0(nx,ns,ne,x,pfac,ppen,c,fp,&c[nc],&c[nc]);
      if(n != nx) gfnlag(nx,ns,ne,x,pfac,ppen,&g[nx],fp);
      for(j=0; j<nx; j++) {
        xp=x[j];
        if(fabs(xp) > 1.0) x[j]=xp+etah*fabs(xp);
        else               x[j]=xp+etah;
        if(n == nx) gfnpn1(nx,ns,ne,x,pfac,ppen,c,&fj,&c[nc],&c[nc]);
        if(n != nx) gfnlag(nx,ns,ne,x,pfac,ppen,c,&fj);
        g[j]=(fj-*fp)/(x[j]-xp);
        x[j]=xp;
      }
/*    +----------------------------------------------------------+    */
/*    | for lagrange multiplier method                           |    */
/*    +----------------------------------------------------------+    */
      for(j=nx+ns; j<n-ne; j++) {
        g[j-ns]=2*x[j-ns]*x[j];
        g[j]=g[j]+x[j-ns]*x[j-ns];
      }
/*    for(i=nx+ns+ns; i<n; i++) {                                     */
/*      g[i]=g[i];                                                    */
/*    }                                                               */
      return(0);
}
/**********************************************************************/
      int fdhess(int nx,int ns,int ne,int n,double x[],double pfac[]
                ,double ppen[],double c[],double f[],double dx[]
                ,double eta,int l[],double h[]) {
/*    int fdhess(nx,ns,ne,n,x,pfac,ppen,c,f,dx,eta,l,h)
      double x[],pfac[],ppen[],c[],f[],dx[],eta,h[];
      int    nx,ns,ne,n,l[]; {                                        */
/*    ============================================================    */
      double xi,xj,fij,gij,hij,fp,etah;
      int    i,j,k,nc,nd,no,ndd; int gfnpn0(),gfnpn1(),gfnlag();
/*    ============================================================    */
/*    compute hessian matrix h(n,n) by difference & partial analysis  */
/*    ------------------------------------------------------------    */
/*i   nx       = number of original variables                         */
/*i   ns       = number of constraints gj(xp(nx)) <= 0                */
/*i   ne       = number of constraints hi(xp(nx))  = 0                */
/*i   n        = nx            : for penalty function                 */
/*i            = nx+ns+ns+ne   : for lagrange multiplier              */
/*i   x(n)     = xp(nx)[,s(ns),ls(ns),le(ne)]                         */
/*i   pfac(nx) = scalar factor of xp(nx)                              */
/*i   ppen(6)  = r(1),r(2),r(3): penalty parameters                   */
/*i              t,c1,c2       : hyperbolic penalty function coef.    */
/*w   c(nc*(nx+4)) = c(nc),hh(nc,nx),o(nc),d(nc),dd(nc) : penalty     */
/*w   c(nc*2)      = c(nc),o(nc)                        : lagrange    */
/*w   f(nx)    = f(x(n)+dx(i)*ei)                                     */
/*w   dx(nx)   = step size                                            */
/*i   eta      = machine epsilon                                      */
/*i   l(n)     = location index of matrix h(n,n)                      */
/*o   h(n+l(n))= hessian matrix h(n,n) at x(n)                        */
/*s   gfnpn0,1 = sub. to compute diff. penalty obj. func. f(x(n))     */
/*s   gfnlag   = sub. to compute lagrange objective func. f(x(n))     */
/*    ============================================================    */
      nc=ns+ne;
      no=nc+nc*nx;
      nd=no+nc;
      ndd=no+nc+nc;
      etah=pow(eta,0.333333);
      for(i=0; i<n*(n+1)/2; i++) {
        h[i]=0.0;
      }
      if(n == nx) {
        gfnpn0(nx,ns,ne,x,pfac,ppen,&c[no],&fp,&c[nd],&c[ndd]);
      } else {
        gfnlag(nx,ns,ne,x,pfac,ppen,&c[nc],&fp);
      }
      for(i=0; i<nx; i++) {
        xi=x[i];
        if(fabs(xi) > 1.0) x[i]=xi+etah*fabs(xi);
        else               x[i]=xi+etah;
        dx[i]=x[i]-xi;
        if(n == nx) {
          gfnpn1(nx,ns,ne,x,pfac,ppen,c,&f[i],&c[nd],&c[ndd]);
          for(j=0; j<ns+ne; j++) {
            c[nc+nc*i+j]=(c[j]-c[no+j])/dx[i];
          }
        } else {
          gfnlag(nx,ns,ne,x,pfac,ppen,c,&f[i]);
          for(j=0; j<ns+ne; j++) {
            h[i+l[nx+ns+j]]=(c[j]-c[nc+j])/dx[i];
          }
        }
        x[i]=xi;
      }
/*    +----------------------------------------------------------+    */
/*    | use forward difference for off-diagonal h(i,j)           |    */
/*    +----------------------------------------------------------+    */
      for(j=0; j<nx; j++) {
        xj=x[j];
        x[j]=x[j]+dx[j];
        for(i=0; i<j; i++) {
          xi=x[i];
          x[i]=x[i]+dx[i];
          if(n == nx) gfnpn1(nx,ns,ne,x,pfac,ppen,c,&fij,&c[nd],&c[ndd]);
          if(n != nx) gfnlag(nx,ns,ne,x,pfac,ppen,c,&fij);
          h[i+l[j]]=((fij-f[j])-(f[i]-fp))/(dx[i]*dx[j]);
          x[i]=xi;
        }
/*      +--------------------------------------------------------+    */
/*      | use central difference for diagonal h(j,j)             |    */
/*      +--------------------------------------------------------+    */
        x[j]=xj-dx[j];
        if(n == nx) gfnpn1(nx,ns,ne,x,pfac,ppen,c,&fij,&c[nd],&c[ndd]);
        if(n != nx) gfnlag(nx,ns,ne,x,pfac,ppen,c,&fij);
        h[j+l[j]]=((f[j]-fp)-(fij-fp))/(dx[j]*dx[j]);
        x[j]=xj;
      }
/*    +----------------------------------------------------------+    */
/*    | for lagrange multiplier method                           |    */
/*    +----------------------------------------------------------+    */
      if(n != nx) {
        for(j=nx; j<n; j++) {
          h[j+l[j+ns]]=2*x[j];
          h[j+l[j]]=2*x[j+ns];
        }
/*    +----------------------------------------------------------+    */
/*    | for penalty function method                              |    */
/*    +----------------------------------------------------------+    */
      } else {
        for(j=0; j<nx; j++) {
          for(i=0; i<=j; i++) {
            gij=0.0;
            hij=0.0;
            for(k=0; k<ns; k++) {
              gij=gij+c[ndd+k]*c[nc+nc*i+k]*c[nc+nc*j+k];
            }
            for(k=ns; k<ns+ne; k++) {
              hij=hij+c[ndd+k]*c[nc+nc*i+k]*c[nc+nc*j+k];
            }
            h[i+l[j]]=h[i+l[j]]+ppen[1]*gij+ppen[0]*hij;
          }
        }
      }
      return(0);
}
/**********************************************************************/
      int gfnlag(int nx,int ns,int ne,double x[],double pfac[]
                ,double ppen[],double c[],double *w) {
/*    int gfnlag(nx,ns,ne,x,pfac,ppen,c,w)
      double x[],pfac[],ppen[],c[],*w; int nx,ns,ne; {                */
/*    ============================================================    */
      double f; int i,j; int fff();
/*    ============================================================    */
/*    compute general objective function by lagrange multiplier       */
/*    ------------------------------------------------------------    */
/*i   n        = nx+ns+ns+ne                                          */
/*i   x(n)     = xp(nx),s(ns),ls(ns),le(ne)                           */
/*    pfac(nx) = scalar factor of xp(nx)    (unused)                  */
/*    ppen(6)  = r(3),t,c1,c2               (unused)                  */
/*w   c(ns+ne) = gj(xp(nx)),hi(xp(nx))                                */
/*o   w        = f + ls(j)*(g(j) + s(j)**2)+ le(i)*h(i)               */
/*s   fff      = sub. to compute f(xp(nx)),gj(xp(nx)),hi(xp(nx))      */
/*    ============================================================    */
      fff(nx,ns,ne,x,&f,c);
      *w=0.0;
      for(j=0; j<ns; j++) {
        *w=*w+x[nx+ns+j]*(c[j]+x[nx+j]*x[nx+j]);
      }
      for(i=ns; i<ns+ne; i++) {
        *w=*w+x[nx+ns+i]*c[i];
      }
      *w=f+*w;
      return(0);
}
/**********************************************************************/
      int gfmlag(int nx,int ns,int ne,double x[],double pfac[]
                ,double ppen[],double c[],double *w) {
/*    int gfmlag(nx,ns,ne,x,pfac,ppen,c,w)
      double x[],pfac[],ppen[],c[],*w; int nx,ns,ne; {                */
/*    ============================================================    */
      double fl,dx; int i,n; int gfnlag();
/*    ============================================================    */
/*    compute adaptive modified lagrange function                     */
/*    ------------------------------------------------------------    */
/*i   n        = nx+ns+ns+ne                                          */
/*i   x(n)     = xp(nx),s(ns),ls(ns),le(ne)                           */
/*i   pfac(nx) = scalar factor of xp(nx)                              */
/*i   pfac(nx+1  :nx+n  ) = newton point xn(n)                        */
/*i   pfac(nx+n+1:nx+n+n) = modified diagonal d(n)                    */
/*    ppen(6)  = r(3),t,c1,c2               (unused)                  */
/*w   c(ns+ne) = gj(xp(nx)),hi(xp(nx))                                */
/*o   w        = fl + (x(i)-xn(i))*d(i)*(x(i)-xn(i)) / 2              */
/*    fl       = f(xp(nx)) + ls(j)*(g(j) + s(j)**2)+ le(i)*h(i)       */
/*s   gfnlag   = sub. to compute lagrange objective func. fl(x(n))    */
/*    ============================================================    */
      n=nx+ns+ns+ne;
      gfnlag(nx,ns,ne,x,pfac,ppen,c,&fl);
      *w=0.0;
      for(i=0; i<n; i++) {
        dx=x[i]-pfac[i+nx];
        *w=*w+dx*pfac[i+nx+n]*dx;
      }
      *w=fl+0.5*(*w);
      return(0);
}
/**********************************************************************/
      int gfnpen(int nx,int ns,int ne,double x[],double pfac[]
                ,double ppen[],double c[],double *w) {
/*    int gfnpen(nx,ns,ne,x,pfac,ppen,c,w)
      double x[],pfac[],ppen[],c[],*w; int nx,ns,ne; {                */
/*    ============================================================    */
      double f,pg,ph,t=0.0001,c1=1.0,c2=0.1;
      int    i,j;   double pf();  int fff();
/*    ============================================================    */
/*    compute general objective function by penalty function          */
/*    ------------------------------------------------------------    */
/*i   n        = nx                                                   */
/*i   x(nx)    = xp(nx)                                               */
/*i   pfac(nx) = scalar factor of xp(nx)    (unused)                  */
/*i   ppen(6)  = r(3),t,c1,c2                                         */
/*w   c(ns+ne) = gj(xp(nx)),hi(xp(nx))                                */
/*o   w        = r(3)*f + r(2)*ys(gj;t,c1,c2) + r(1)*ye(hi;t,c1,c2)   */
/*s   fff      = sub. to compute f(xp(nx)),gj(xp(nx)),hi(xp(nx))      */
/*    ============================================================    */
      fff(nx,ns,ne,x,&f,c);
      pg=0.0;
      ph=0.0;
      for(j=0; j<ns; j++) {
        pg=pg+pf(c[j],pfac[j],ppen[3],ppen[4],ppen[5],-1);
      }
      for(i=ns; i<ns+ne; i++) {
        ph=ph+pf(c[i],pfac[i],ppen[3],ppen[4],ppen[5], 0);
      }
      *w=ppen[2]*f+ppen[1]*pg+ppen[0]*ph;
      return(0);
}
/**********************************************************************/
      int gfnpn0(int nx,int ns,int ne,double x[],double pfac[]
         ,double ppen[],double c[],double *w,double d[],double dd[]) {
/*    int gfnpn0(nx,ns,ne,x,pfac,ppen,c,w,d,dd)
      double x[],pfac[],ppen[],c[],*w,d[],dd[]; int nx,ns,ne; {       */
/*    ============================================================    */
      double f,y,pg,ph; /* double t=0.0001,c1=1.0,c2=0.1; */
      int    i,j; int pd(),fff();
/*    ============================================================    */
/*    compute differential penalty function for the gradient          */
/*    ------------------------------------------------------------    */
/*i   n         = nx                                                  */
/*i   x(nx)     = xp(nx)                                              */
/*i   pfac(nx)  = scalar factor of xp(nx)                             */
/*i   ppen(6)   = r(3),t,c1,c2                                        */
/*w   c(ns+ne)  = gj(xp(nx)),hi(xp(nx))                               */
/*o   w         = r(3)*f + r(2)*dyj*gj+ r(1)*dyi*hi                   */
/*o   d(ns+ne)  = dysj/dgj   or dyei/dhi                              */
/*o   dd(ns+ne) = d2ysj/dgj2 or d2yei/dhi2                            */
/*s   fff       = sub. to compute f(xp(nx)),gj(xp(nx)),hi(xp(nx))     */
/*    ============================================================    */
      fff(nx,ns,ne,x,&f,c);
      pg=0.0;
      ph=0.0;
      for(j=0; j<ns; j++) {
        pd(c[j],pfac[j],ppen[3],ppen[4],ppen[5],-1,&y,&d[j],&dd[j]);
        pg=pg+d[j]*c[j];
      }
      for(i=ns; i<ns+ne; i++) {
        pd(c[i],pfac[i],ppen[3],ppen[4],ppen[5], 0,&y,&d[i],&dd[i]);
        ph=ph+d[i]*c[i];
      }
      *w=ppen[2]*f+ppen[1]*pg+ppen[0]*ph;
      return(0);
}
/**********************************************************************/
      int gfnpn1(int nx,int ns,int ne,double x[],double pfac[]
         ,double ppen[],double c[],double *w,double d[],double dd[]) {
/*    int gfnpn1(nx,ns,ne,x,pfac,ppen,c,w,d,dd)
      double x[],pfac[],ppen[],c[],*w,d[],dd[]; int nx,ns,ne; {       */
/*    ============================================================    */
      double f,pg,ph; /* double t=0.0001,c1=1.0,c2=0.1; */
      int    i,j; int fff();
/*    ============================================================    */
/*    compute differential penalty function for the gradient          */
/*    ------------------------------------------------------------    */
/*i   n         = nx                                                  */
/*i   x(nx)     = xp(nx)                                              */
/*i   pfac(nx)  = scalar factor of xp(nx)                             */
/*i   ppen(6)   = r(3),t,c1,c2                                        */
/*w   c(ns+ne)  = gj(xp(nx)),hi(xp(nx))                               */
/*o   w         = r(3)*f + r(2)*dyj*gj + r(1)*dyi*hi                  */
/*i   d(ns+ne)  = dysj/dgj   or dyei/dhi   : input  for grdpn1        */
/*o   d(ns+ne)  = dysj/dgj   or dyei/dhi   : output for grdpn0        */
/*i   dd(ns+ne) = d2ysj/dgj2 or d2yei/dhi2 : input  for grdpn1        */
/*o   dd(ns+ne) = d2ysj/dgj2 or d2yei/dhi2 : output for grdpn0        */
/*s   fff       = sub. to compute f(xp(nx)),gj(xp(nx)),hi(xp(nx))     */
/*    ============================================================    */
      fff(nx,ns,ne,x,&f,c);
      pg=0.0;
      ph=0.0;
      for(j=0; j<ns; j++) {
/*      pd(c[j],pfac[j],ppen[3],ppen[4],ppen[5],-1,&y,&d[j],&dd[j]);  */
        pg=pg+d[j]*c[j];                         /* d(1:ns) are given */
      }
      for(i=ns; i<ns+ne; i++) {
/*      pd(c[i],pfac[i],ppen[3],ppen[4],ppen[5], 0,&y,&d[i],&dd[i]);  */
        ph=ph+d[i]*c[i];                   /* d(ns+1:ns+ne) are given */
      }
      *w=ppen[2]*f+ppen[1]*pg+ppen[0]*ph;
      return(0);
}
/**********************************************************************/
