#include <math.h>
/**********************************************************************/
      int hsdecp(double a[],int ll[],int not,int *pnnt,int *pnsd
                ,int maxa,int maxn,double dd[]) {
/*    int hsdecp(a,ll,not,pnnt,pnsd,maxa,maxn,dd)
      double a[],dd[]; int ll[],not,*pnnt,*pnsd,maxa,maxn; {          */
/*    ============================================================    */
/*    dimension a(maxa),ll(maxn),dd(not)                              */
      double aij,ajj,akj,add,tmp,eps=1.0e-7;
      int    i,j,k,ia,ja,ka,jb,im,jm,nat,nbt,nnt,nsd;
      --a; --ll; --dd;
/*    ============================================================    */
/*    modified cholesky decomposition [a]=[u'][d][u]                  */
/*    ------------------------------------------------------------    */
/*i   a(not+ll(not)) = the given original matrix a(not,not)           */
/*o   a(nnt+ll(nnt)) = decomposed matrix d+u-i : u(nnt,nnt),d(nnt)    */
/*i   ll(1:not)      = original storage index of a(not,not)           */
/*o   ll(not+1:nnt)  = expanded storage index of u(nnt,nnt)           */
/*i   not     = the order of the original matrix a(not,not)           */
/*o   nnt     = the order of the expanded matrix a(nnt,nnt)           */
/*o   nsd     = number of singular degrees of the given matrix        */
/*i   maxa    = the maximum allowed dimension of a (maxa)             */
/*i   maxn    = the maximum allowed dimension of ll(maxn)             */
/*o   dd(not) = added diagonal elements                               */
/*    eps     = epsilon for accuracy control                          */
/*    ------------------------------------------------------------    */
/*    find [d] & [u] for given [a], such that  [a] = [u]'[d] [u]      */
/*    where [u]'= transpose of [u], [d] is a diagonal matrix.         */
/*    for example:                                                    */
/*    input : not=3, ll(3)=(0 1 3), [a] is a not*not matrix           */
/*    output: nnt=5, ll(5)=(0 1 3 6 9), [d][u] are nnt*nnt matrices   */
/*                                                                    */
/*          | a11 a12 a13 |           | a11+p1 a12   a13 -p1  0  |    */
/*    [a] = | a12 a22 a23 |           |  a12  a22+p2 a23  0  -p2 |    */
/*          | a13 a23 a33 |           |  a13   a23   a33  0   0  |    */
/*    if it is expanded to ---> [a] = |  -p1    0     0  +p1  0  |    */
/*    then decomposed matrices are:   |   0    -p2    0   0  +p2 |    */
/*                                                                    */
/*          | d11  0   0   0   0  |        |  1  u12 u13 u14  0  |    */
/*          |  0  d22  0   0   0  |        |  0   1  u23 u24 u25 |    */
/*    [d] = |  0   0  d33  0   0  |, [u] = |  0   0   1  u34 u35 |    */
/*          |  0   0   0  d44  0  |        |  0   0   0   1  u45 |    */
/*          |  0   0   0   0  d55 |        |  0   0   0   0   1  |    */
/*                                                                    */
/*o  ll(5):( 0       1           3 |             6               9)   */
/* i+ll(j):1+0 1+1 2+1 1+3 2+3 3+3 1+6 2+6 3+6 4+6 2+9 3+9 4+9 5+9    */
/*    loc : 1   2   3   4   5   6   7   8   9   10  11  12  13  14    */
/*i  a(6):(a11 a12 a22 a13 a23 a33)                                   */
/*o a(14):(d11 u12 d22 u13 u23 d33 u14 u24 u34 d44 u25 u35 u45 d55)   */
/*    ============================================================    */
/*    +----------------------------------------------------------+    */
/*    | initial set nsd=0 : this also allows matrix to expend    |    */
/*    | set to decompose the original given matrix a(not,not)    |    */
/*    +----------------------------------------------------------+    */
      nsd=0;
      nat=0;
      nbt=not;
      nnt=not;
/*    +----------------------------------------------------------+    */
/*    | decompose the sub-matrix a((nat+1):nbt,(nat+1):nbt)      |    */
/*    +----------------------------------------------------------+    */
/*    | d(i,i)u(i,j)=a(i,j)-\sum_{k=im}^{i-1} u(k,i)d(k,k)u(k,j) |    */
/*    | a(k+ia)=u(k,i),     a(k+ja)=d(k,k)*u(k,j)                |    */
/*    | a(i+ja)=aij=a(i,j) --> d(i,i)*u(i,j)                     |    */
/*    +----------------------------------------------------------+    */
      jb=0;
  L10:for(j=nat+1; j<=nbt; j++) {
        ja=ll[j]-ll[1];
        jm=j-ja+jb;
        jb=ja;
        for(i=jm+1; i<=j-1; i++) {
          ia=ll[i]-ll[1];
          im=i-ll[i]+ll[i-1]; if(jm > im) im=jm;
          aij=a[i+ja];
          for(k=im; k<=i-1; k++) {
            aij=aij-a[k+ia]*a[k+ja];
          }
          a[i+ja]=aij;
        }
/*      +--------------------------------------------------------+    */
/*      | d(j,j) =a(j,j) - \sum_{k=jm}^{j-1} u(k,j)*d(k,k)*u(k,j)|    */
/*      | a(k+ja)=d(k,k)*u(k,j) --> u(k,j),  akj=u(k,j)          |    */
/*      | a(j+ja)=ajj=a(j,j) --> d(j,j),     a(k+ka)=d(k,k)      |    */
/*      +--------------------------------------------------------+    */
        ajj=a[j+ja];
        add=fabs(ajj);
        for(k=jm; k<=j-1; k++) {
          ka=ll[k]-ll[1];
          akj=a[k+ja]/a[k+ka];
          ajj=ajj-akj*a[k+ja];
          a[k+ja]=akj;
          if((tmp=fabs(ajj)) > add) add=tmp;
        }
        a[j+ja]=ajj;
/*      +--------------------------------------------------------+    */
/*      | if the pivoting element a(j,j) > add*eps ?             |    */
/*      | yes:80 : ok and continue                               |    */
/*      | no     : add pivoting by add and expand one column     |    */
/*      +--------------------------------------------------------+    */
        if(j <= not) {
          dd[j]=0.0;
          if(a[j+ja] > add*eps) continue;    /* modified for lagrange */
        } else {
          if(fabs(a[j+ja]) > add*eps) continue;
        }
/*      +--------------------------------------------------------+    */
/*      | if add == 0 ?                                          |    */
/*      | yes: no reference value available, get from diagonal   |    */
/*      +--------------------------------------------------------+    */
        if(add == 0.0) {
          for(k=nat+1; k<=nbt; k++) {
            ka=ll[k]-ll[1];
            if((tmp=fabs(a[k+ka])) > add) add=tmp;
          }
        }
        if(add == 0.0) add=1.0;
        if(a[j+ja] <= 0.0) add=add-a[j+ja];  /* modified for lagrange */
        if(j <= not) dd[j]=add;
/*      +--------------------------------------------------------+    */
/*      | if j == nnt && nsd == 0                                |    */
/*      | then it is the only & the last zero_diagonal,          |    */
/*      | so set nsd=1 to prevent from any further expanding     |    */
/*      +--------------------------------------------------------+    */
        if(j == nnt && nsd == 0) nsd=1;
/*      +--------------------------------------------------------+    */
/*      | if nsd > 0 : no more expanding is allowed              |    */
/*      +--------------------------------------------------------+    */
        if(nsd > 0) {
          a[j+ja]=fabs(a[j+ja]);
          if((tmp=add*eps*eps) > a[j+ja]) a[j+ja]=tmp;
/*      +--------------------------------------------------------+    */
/*      | if nsd == 0 : expand the matrix by                     |    */
/*      | modify the diagonal : a(j,j)=a(j,j)+add                |    */
/*      | add column : a(j,nnt)=-add, a(nnt,nnt)=add, a(*,nnt)=0 |    */
/*      +--------------------------------------------------------+    */
        } else {
          a[j+ja]=a[j+ja]+add;
          nnt=nnt+1;
          if(nnt > maxn) {
            printf("\n hsdecp : nnt > maxn"); exit(1);
          }
          ll[nnt]=ll[nnt-1]+nnt-j;
          ja=ll[nnt]-ll[1];
          if(nnt+ja > maxa) {
            printf("\n hsdecp : nnt+ja > maxa"); exit(1);
          }
          a[j+ja]=-add;
          for(i=j+1; i<=nnt-1; i++) {
            a[i+ja]=0.0;
          }
          a[nnt+ja]=add;
        }
      }
/*    +----------------------------------------------------------+    */
/*    | if nnt > nbt  then decompose the newly expand matrix     |    */
/*    +----------------------------------------------------------+    */
      if(nnt > nbt) {
/*      +--------------------------------------------------------+    */
/*      | if newly expand numbers  ==  previously expand numbers |    */
/*      | then it indicates that :                               |    */
/*      | a((nbt+1):nnt,(nbt+1):nnt)=a((nat+1):nbt,(nat+1):nbt)  |    */
/*      | so set nsd=nnt-nbt to prevent from any further expand  |    */
/*      +--------------------------------------------------------+    */
        if(nnt-nbt == nbt-nat && nat != 0) nsd=nnt-nbt;
/*      +--------------------------------------------------------+    */
/*      | decompose the newly expand matrix (nbt+1):nnt          |    */
/*      +--------------------------------------------------------+    */
        nat=nbt;
        nbt=nnt;
        goto L10;
      }
      *pnnt=nnt; *pnsd=nsd; return(0);
}
/**********************************************************************/
      int hssolx(double a[],double p[],int ll[]
                ,int not,int nnt,double b[]) {
/*    int hssolx(a,p,ll,not,nnt,b)
      double a[],p[],b[]; int ll[],not,nnt; {                         */
/*    ============================================================    */
      double bi; int i,k,ia,im;
      --a; --p; --ll; --b;
/*    ============================================================    */
/*    dimension a(nnt+ll(nnt)),p(not),ll(nnt),b(nnt)                  */
/*    ------------------------------------------------------------    */
/*    find {x} from [a]{x}={p} for given {p} & [a]=>[u]'[d][u].       */
/*    [d] & [u] are given & put in a(*)  (expanded by sub. hsdecp)    */
/*    [u]'[d][u]{x}={b} ===> [u]'{z}={b},  [d]{y}={z},  [u]{x}={y}    */
/*    ------------------------------------------------------------    */
/*i   p(not) = the given constant vector                              */
/*o   p(not) = the solution vector                                    */
/*w   b(nnt) = the expanded vector (form by hssolx)                   */
/*    ============================================================    */
/*    +----------------------------------------------------------+    */
/*    | expand p(not) to b(nnt)                                  |    */
/*    +----------------------------------------------------------+    */
      for(i=1; i<=not; i++) {
        b[i]=p[i];
      }
      for(i=not+1; i<=nnt; i++) {
        b[i]=0.0;
      }
/*    +----------------------------------------------------------+    */
/*    | find {z} from {b} by forward substitution of [u]'{z}={b} |    */
/*    | z(i) = bi = b(i) - \sum_{k=im}^{i-1} u'(i,k)*z(k)        |    */
/*    | a(k+ia)=u(k,i)=u'(i,k), b(k)=z(k), b(i)=b(i) --> z(i)    |    */
/*    +----------------------------------------------------------+    */
      for(i=2; i<=nnt; i++) {
        ia=ll[i]-ll[1];
        im=i-ll[i]+ll[i-1];
        bi=b[i];
        for(k=im; k<=i-1; k++) {
          bi=bi-a[k+ia]*b[k];
        }
        b[i]=bi;
      }
/*    +----------------------------------------------------------+    */
/*    | find {y} from {z} from the equation [d]{y}={z}           |    */
/*    | y(i) = z(i) / d(i,i)                                     |    */
/*    | b(i) = z(i) --> y(i),     a(i+ia)=d(i,i)                 |    */
/*    +----------------------------------------------------------+    */
      for(i=1; i<=nnt; i++) {
        ia=ll[i]-ll[1];
        b[i]=b[i]/a[i+ia];
      }
/*    +----------------------------------------------------------+    */
/*    | find {x} from {y} by backward substitution of [u]{x}={y} |    */
/*    | for i=n:2  compute  b(k)=b(k)-u(k,i)*x(i) for k=im:(i-1) |    */
/*    | a(k+ia)=u(k,i),     b(i)=bi=x(i),     b(k)=y(k) --> x(k) |    */
/*    +----------------------------------------------------------+    */
      for(i=nnt; i>=2; i--) {
        ia=ll[i]-ll[1];
        im=i-ll[i]+ll[i-1];
        bi=b[i];
        for(k=im; k<=i-1; k++) {
          b[k]=b[k]-a[k+ia]*bi;
        }
      }
/*    +----------------------------------------------------------+    */
/*    | move solution vector b(not) to p(not)                    |    */
/*    +----------------------------------------------------------+    */
      for(i=1; i<=not; i++) {
        p[i]=b[i];
      }
      return(0);
}
/**********************************************************************/
      int hsbksb(double a[],double b[],int ll[],int not,int nnt) {
/*    int hsbksb(a,b,ll,not,nnt)
      double a[],b[]; int ll[],not,nnt; {                             */
/*    ============================================================    */
      double bi; int i,k,ia,im;
      --a; --b; --ll;
/*    ============================================================    */
/*    dimension a(nnt+ll(nnt)),b(nnt),ll(nnt)                         */
/*    ------------------------------------------------------------    */
/*    find {x} from {y} by backward substitution of [u]{x}={y}        */
/*    ------------------------------------------------------------    */
/*i   b(nnt) = the given vector {y}                                   */
/*o   b(nnt) = the solution vector {x}                                */
/*    ============================================================    */
/*    +----------------------------------------------------------+    */
/*    | find {x} from {y} by backward substitution of [u]{x}={y} |    */
/*    | for i=nnt:2 compute b(k)=b(k)-u(k,i)*x(i) for k=im:(i-1) |    */
/*    | a(k+ia)=u(k,i),     b(i)=bi=x(i),     b(k)=y(k) --> x(k) |    */
/*    +----------------------------------------------------------+    */
      for(i=nnt; i>=2; i--) {
        ia=ll[i]-ll[1];
        im=i-ll[i]+ll[i-1];
        bi=b[i];
        for(k=im; k<=i-1; k++) {
          b[k]=b[k]-a[k+ia]*bi;
        }
      }
      return(0);
}
/**********************************************************************/
      int hssngx(double a[],double s[],int ll[]
                ,int not,int nnt,int nsd,double b[]) {
/*    int hssngx(a,s,ll,not,nnt,nsd,b)
      double a[],s[],b[]; int ll[],not,nnt,nsd; {                     */
/*    ============================================================    */
      int i,j,js; int hsbksb();
      --a; --s; --ll; --b;
/*    ============================================================    */
/*    dimension a(nnt+ll(nnt)),s(not,nsd),ll(nnt),b(nnt)              */
/*    ------------------------------------------------------------    */
/*    find the solution vectors [s] of the singular matrix [a]        */
/*    ------------------------------------------------------------    */
/*i   a(nnt+ll(nnt)) = decomposed matrix d+u-i : u(nnt,nnt),d(nnt)    */
/*i   ll(nnt)        = expanded storage index of u(nnt,nnt)           */
/*o   s(not,nsd) = solution vectors of singular matrix a(not,not)     */
/*i   not     = the order of the original matrix a(not,not)           */
/*i   nnt     = the order of the expanded matrix a(nnt,nnt)           */
/*i   nsd     = number of singular degrees of the given matrix        */
/*w   b(nnt)  = the working array                                     */
/*    ============================================================    */
      for(j=nnt-nsd+1; j<=nnt; j++) {
        js=(j-nnt+nsd-1)*not;
        for(i=1; i<=nnt; i++) {
          b[i]=0.0;
        }
        b[j]=1.0;
        hsbksb(&a[1],&b[1],&ll[1],not,nnt);
        for(i=1; i<=not; i++) {
          s[i+js]=b[i];
        }
      }
      return(0);
}
/**********************************************************************/
