#include <math.h>
/**********************************************************************/
      int pd(double g,double p,double t,double c1,double c2
            ,int nctp,double *f,double *df,double *ddf) {
/*    int pd(g,p,t,c1,c2,nctp,f,df,ddf)
      double g,p,t,c1,c2,*f,*df,*ddf; int nctp; {                     */
/*    ============================================================    */
      double s,x,y,d1,d2,y2,dy,ddy;
/*    ============================================================    */
/*i   g     = value of constraint                                     */
/*i   p     = positive scale factor of g                              */
/*i   t     = shape parameter of the hyperbola                        */
/*i   c1,c2 = coefficients for modified penalty function              */
/*i   nctp  = type of constraint                                      */
/*          > 0 for constraint p*g>=0  :   s=-p ==> x = s*g <= 0      */
/*          < 0 for constraint p*g<=0  :   s=+p ==> x = s*g <= 0      */
/*          = 0 for constraint p*g =0  :   s=+p ==> x = s*g  = 0      */
/*o   f     = c1*y + c2*y*y      = penalty function for t>0           */
/*          = exp(c1*y+c2*y*y)-1 = penalty function for t<0           */
/*o   df    = (df/dy)*dy                     :  dy  =  dy/dg          */
/*o   ddf   = (df/dy)*ddy + (d2f/dy2)*dy*dy  :  ddy = d2y/dg2         */
/*    ============================================================    */
      s=p;
      if(nctp > 0) s=-s;
      x=s*g;
/*    +----------------------------------------------------------+    */
/*    | penalty function for x=0 constraint                      |    */
/*    | from  y**2 - x**2 = t**2                 \        /      |    */
/*    | y   = sqrt(x*x+t*t)                        \    /        |    */
/*    | dy  = s*x/sqrt(x*x+t*t) = s*x/y              \/          |    */
/*    | ddy = (s*t)**2/sqrt(x*x+t*t)**3                          |    */
/*    +----------------------------------------------------------+    */
      y  =sqrt(x*x+t*t);
      dy =s*x/y;
      ddy=(s*t/y)*(s*t/y)/y;
/*    +----------------------------------------------------------+    */
/*    | penalty function for x<=0 constraint                     |    */
/*    | from  (2y)**2 - 2*x*(2y) = t**2                     /    |    */
/*    | 2y  = sqrt(x*x+t*t)+x                             /      |    */
/*    | 2dy = s*(sqrt(x*x+t*t)+x)/sqrt(x*x+t*t)   ______/        |    */
/*    | 2ddy= (s*t)**2/sqrt(x*x+t*t)**3                          |    */
/*    +----------------------------------------------------------+    */
      if(nctp != 0) {
        y2=y+fabs(x);
        if(x < 0.0) y2=t*t/y2;
        ddy=0.5*ddy;
        dy =0.5*s*y2/y;
        y  =0.5*y2;
      }
/*    +----------------------------------------------------------+    */
/*    | modified penalty function                                |    */
/*    +----------------------------------------------------------+    */
      *f=c1*y+c2*y*y;
      d1=c1+2*c2*y;
      d2=2*c2;
      if(t < 0) {
        if(*f > 64.0) *f=64.0;
        *f=exp(*f);
        d2=(d1*d1+d2)*(*f);
        d1=d1*(*f);
        *f=(*f)-1;
      }
/*    +----------------------------------------------------------+    */
/*    | for gradient, only df is required, loc.df=loc.ddf is ok  |    */
/*    +----------------------------------------------------------+    */
      *ddf=d1*ddy+d2*dy*dy;
      *df =d1*dy;
      return(0);
}
/**********************************************************************/
      int pe(double g,double p,double t,double c1,double c2
            ,int nctp,double *f) {
/*    int pe(g,p,t,c1,c2,nctp,f) double g,p,t,c1,c2,*f; int nctp; {   */
/*    ============================================================    */
      double s,x,y,y2;
/*    ============================================================    */
/*i   g     = value of constraint                                     */
/*i   p     = positive scale factor of g                              */
/*i   t     = shape parameter of the hyperbola                        */
/*i   c1,c2 = coefficients for modified penalty function              */
/*i   nctp  = type of constraint                                      */
/*          > 0 for constraint p*g>=0  :   s=-p ==> x = s*g <= 0      */
/*          < 0 for constraint p*g<=0  :   s=+p ==> x = s*g <= 0      */
/*          = 0 for constraint p*g =0  :   s=+p ==> x = s*g  = 0      */
/*o   f     = c1*y + c2*y*y      = penalty function for t>0           */
/*          = exp(c1*y+c2*y*y)-1 = penalty function for t<0           */
/*o   df    = (df/dy)*dy                     :  dy  =  dy/dg          */
/*o   ddf   = (df/dy)*ddy + (d2f/dy2)*dy*dy  :  ddy = d2y/dg2         */
/*    ============================================================    */
      s=p;
      if(nctp > 0) s=-s;
      x=s*g;
/*    +----------------------------------------------------------+    */
/*    | penalty function for x=0 constraint                      |    */
/*    | from  y**2 - x**2 = t**2                 \        /      |    */
/*    | y   = sqrt(x*x+t*t)                        \    /        |    */
/*    | dy  = s*x/sqrt(x*x+t*t) = s*x/y              \/          |    */
/*    | ddy = (s*t)**2/sqrt(x*x+t*t)**3                          |    */
/*    +----------------------------------------------------------+    */
      y  =sqrt(x*x+t*t);
/*    dy =s*x/y;                                                      */
/*    ddy=(s*t/y)*(s*t/y)/y;                                          */
/*    +----------------------------------------------------------+    */
/*    | penalty function for x<=0 constraint                     |    */
/*    | from  (2y)**2 - 2*x*(2y) = t**2                     /    |    */
/*    | 2y  = sqrt(x*x+t*t)+x                             /      |    */
/*    | 2dy = s*(sqrt(x*x+t*t)+x)/sqrt(x*x+t*t)   ______/        |    */
/*    | 2ddy= (s*t)**2/sqrt(x*x+t*t)**3                          |    */
/*    +----------------------------------------------------------+    */
      if(nctp != 0) {
        y2=y+fabs(x);
        if(x < 0.0) y2=t*t/y2;
/*      ddy=0.5*ddy;                                                  */
/*      dy =0.5*s*y2/y;                                               */
        y  =0.5*y2;
      }
/*    +----------------------------------------------------------+    */
/*    | modified penalty function                                |    */
/*    +----------------------------------------------------------+    */
      *f=c1*y+c2*y*y;
/*    d1=c1+2*c2*y;                                                   */
/*    d2=2*c2;                                                        */
      if(t < 0) {
        if(*f > 64.0) *f=64.0;
        *f=exp(*f);
/*      d2=(d1*d1+d2)*(*f);                                           */
/*      d1=d1*(*f);                                                   */
        *f=(*f)-1;
      }
/*    +----------------------------------------------------------+    */
/*    | for gradient, only df is required, loc.df=loc.ddf is ok  |    */
/*    +----------------------------------------------------------+    */
/*    *ddf=d1*ddy+d2*dy*dy;                                           */
/*    *df =d1*dy;                                                     */
      return(0);
}
/**********************************************************************/
      double pf(double g,double p,double t,double c1,double c2
               ,int nctp) {
/*    double pf(g,p,t,c1,c2,nctp)
      double g,p,t,c1,c2; int nctp; {                                 */
/*    ============================================================    */
      double fun;
/*    ============================================================    */
      pe(g,p,t,c1,c2,nctp,&fun);
      return(fun);
}
/**********************************************************************/
