************************************************************************
      PROGRAM SLPDSM
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(50000)
      DATA NDIM/50001/
C**   ============================================================    **
C**   Sequential Linear Programming by Double Secant Method           **
C**   ============================================================    **
      READ(*,'(6I4,F8.0)') NX,NS,NE,IS,IC,MAXS,EPS
      IF(IS.GT.0) OPEN(IS,FILE='SLP.IS',STATUS='UNKNOWN'
     *                   ,FORM='UNFORMATTED',ACCESS='SEQUENTIAL')
      IF(IC.GT.0) OPEN(IC,FILE='SLP.IC',STATUS='UNKNOWN'
     *                   ,FORM='FORMATTED',ACCESS='SEQUENTIAL')
C**   ------------------------------------------------------------    **
      NGW =NS+NE+1
      NGWA=NS+NE+2      ! Constrained eqs.+ objective fun.+ big.M
      NXXB=NX+NX+1      ! Positive increment + Negative inc.+ RHS
      NXXS=NX+NX+NS     ! Positive inc.+ Negative inc.+ Slack var.
C**   ------------------------------------------------------------    **
      N1=1
      N2=N1+NX                  ! XX(NX)
      N3=N2+NX                  ! DX(NX)
      N4=N3+NX                  ! DXM(NX)
      N5=N4+NX                  ! XXU(NX)
      N6=N5+NX                  ! XXL(NX)
      N7=N6+NS                  ! GGU(NS)
      N8=N7+NGW                 ! GWO(NGW)
      M1=N8+NGW                 ! GWN(NGW)
      M2=M1+NGWA*NXXB           ! A(NIA,NJA) = A(NS+NE+2,NX+NX+NS)
      M3=M2+NGWA                ! LI(NIA)
      M4=M3+NXXB                ! LJ(NJA)
      M5=M4+NXXS                ! D(NJA)
      K1=M5+NXXS                ! X(NJA)
      K2=K1+NGWA*NXXB           ! B(NIA,NJA)
      IF(IS.GE.0) K2=K1         ! B is not reqd for IS.GE.0
      K3=K2+NGWA                ! MI(NIA)
      K4=K3+NXXB                ! MJ(NJA)
      IF(IS.EQ.0) K4=K1         ! MI,MJ are not reqd for IS.EQ.0
      IF(K4.GE.NDIM) STOP 1111
C**   +----------------------------------------------------------+    **
C**   | Input XX(NX),DXM(NX),XXU(NX),XXL(NX),GGU(NS)             |    **
C**   +----------------------------------------------------------+    **
      CALL INPXDX(A(N1),A(N3),A(N4),A(N5),A(N6),NX,NS)
C**   +----------------------------------------------------------+    **
C**   | Call SLP to perform Sequential Linear Programming        |    **
C**   +----------------------------------------------------------+    **
      CALL SLP(A(N1),A(N2),A(N3),A(N4),A(N5),A(N6),A(N7),A(N8)
     *        ,NX,NS,NE,NGW,EPS,MAXS
     *        ,A(M1),A(M2),A(M3),A(M4),A(M5),NGWA,NXXB,NXXS
     *        ,A(K1),A(K2),A(K3),IS,IC)
      STOP
      END
************************************************************************
      SUBROUTINE INPXDX(XX,DXM,XXU,XXL,GGU,NX,NS)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION XX(NX),DXM(NX),XXU(NX),XXL(NX),GGU(NS)
C**   ============================================================    **
C**   Read initial values XX(NX), and maximum move size DXM(NX)       **
C**   Read upper bounds  XXU(NX), and lower bounds XXL(NX)             **
C**   Read upper bounds of slack variables GGU(NS)                    **
C**   ============================================================    **
      READ(*,'(10F8.0)') (XX (J),J=1,NX)
      READ(*,'(10F8.0)') (DXM(J),J=1,NX)
      READ(*,'(10F8.0)') (XXU(J),J=1,NX)  ! If unbound, XXU(J)=XXL(J)=0
      READ(*,'(10F8.0)') (XXL(J),J=1,NX)
      READ(*,'(10F8.0)') (GGU(I),I=1,NS)  ! If unbound, GGU(I)=0
      RETURN
      END
************************************************************************
      SUBROUTINE SLP(XX,DX,DXM,XXU,XXL,GGU,GWO,GWN
     *              ,NX,NS,NE,MGW,EPS,MAXS
     *              ,A,LI,LJ,D,X,NIA,NJA,NJX,B,MI,MJ,IS,IC)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION XX(NX),DX(NX),DXM(NX),XXU(NX),XXL(NX),GGU(NS)
      DIMENSION GWO(MGW),GWN(MGW)
      DIMENSION A(NIA,NJA),LI(NIA),LJ(NJA),D(NJX),X(NJX)
      DIMENSION B(NIA,NJA),MI(NIA),MJ(NJA)
C**   ============================================================    **
C**   Sequential Linear Programming by Double Secant Method           **
C**   ------------------------------------------------------------    **
C**   Input   : NX,NS,NE,MGW,EPS,MAXS,NIA,NJA,NJX,IS,IC               **
C**           : XX(NX),DXM(NX),XXU(NX),XXL(NX),GGU(NS)                **
C**   Output  : XX(1:NX),GWO(1:NS+NE),GWO(NS+NE+1)                    **
C**   Working : DX(NX),GWN(MGW),A(NIA,NJA)                            **
C**           : A,LI,LJ,D,X : For linear programming (see LINEAR)     **
C**           : MI,MJ : For linear programming (required if IS<>0)    **
C**           : B     : For linear programming (required if IS< 0)    **
C**   ------------------------------------------------------------    **
C*I   NX        = Number of original variables                        **
C*I   NS        = Number of .LE. constraints                          **
C*I   NE        = Number of .EQ. constraints                          **
C*I   MGW      >= NGW  = NS+NE+1                                      **
C*I   EPS       = Relative error for convergence control              **
C*I   MAXS      = Maximum allowed numbers of linear programming       **
C*I   NIA      >= NGWA = NS+NE+2  : Dimension of Simplex tabulea      **
C*I   NJA      >= NXXB = NX+NX+1  : Dimension of Simplex tabulea      **
C*I   NJX      >= NXXS = NX+NX+NS : Var^+, Var^-, Slack-var           **
C*I   IS,IC     = See SUBROUTINE LINEAR                               **
C*I   XX(NX)    = Initial values of original variables                **
C*O             = Optimum values of original variables                **
C*W   DX(NX)    = Move limits                                         **
CoI   DXM(NX)   = Maximum Move size                                   **
CoI   XXU(NX)   = Upper bounds of original variables                  **
CoI   XXL(NX)   = Lower bounds of original variables                  **
CoI   GGU(NS)   = Upper bounds of slack variables for .LE. constr.    **
C*O   GWO(1:NG) = Constrained values at optimum (NG=NS+NE)            **
C*O   GWO(NG+1) = Objective function at optimum                       **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Set constants                                            |    **
C**   +----------------------------------------------------------+    **
      NGW =NS+NE+1
      NGWA=NS+NE+2
      NXXB=NX+NX+1
      NXXS=NX+NX+NS
C**   +----------------------------------------------------------+    **
C**   | Check initial setting of XX,DXM,XXU,XXL(NX) and GGU(NS)  |    **
C**   | Set initial DX(NX), WOLD, NSH = Number of LP performed   |    **
C**   +----------------------------------------------------------+    **
C**   CALL INPXDX(XX,DXM,XXU,XXL,GGU,NX,NS)
      DO 120 J=1,NX
      IF(DXM(J).LE.0.0) DXM(J)=DMAX1(0.10*DABS(XX(J)),0.10D0)
      DX(J)=DABS(DXM(J))
      IF(XXU(J).GT.XXL(J)) GO TO 120
      XXU(J)= 1.0D10                     ! Set to infinity for XXU=XXL=0
      XXL(J)=-1.0D10
  120 XX (J)=DMAX1(DMIN1(XXU(J),XX(J)),XXL(J))
      DO 130 I=1,NS
      IF(GGU(I).LE.0.0) GGU(I)=1.0D10    ! Set to infinity for GGU=0
  130 CONTINUE
      WOLD=0.0                           ! Old objective function
      NSH =0                             ! Iteration count
C**   +----------------------------------------------------------+    **
C**   | Call DSM for SLP by Double Secant Method                 |    **
C**   | To find X(NXXS) from given XX(NX),DX(NX)                 |    **
C**   +----------------------------------------------------------+    **
  200 CALL DSM(XX,DX,XXU,XXL,GGU,GWO,GWN,NX,NS,NE,MGW,ICASE,KKK
     *        ,A,LI,LJ,D,X,NIA,NJA,NJX,B,MI,MJ,IS,IC)
C**   +----------------------------------------------------------+    **
C**   | Update variables XX(NX), move limits DX(NX) & count NSH  |    **
C**   +----------------------------------------------------------+    **
      DXMAX=0.0                                    ! For converge. test
      DO 230 J=1,NX
      DXJ=X(2*J-1)-X(2*J)                          ! Movement of var. J
      DXNJ=DABS(DXJ)
      DXMJ=DABS(DXM(J))
      DXMAX=DMAX1(DXMAX,DXNJ/DXMJ)                 ! Maximum movement
      IF(DXJ*DXM(J).LT.0.AND.NSH.NE.0) DXNJ=DXNJ/2 ! Reduce move limit
      DX(J)=DMAX1(DMIN1((1+DXNJ/DX(J))*DXNJ,DXMJ)  ! if move dir. change
     *        ,0.25*DX(J),0.04*DXMJ,4.0*EPS*DXMJ)  ! Update DX(J)
      DXM(J)=DSIGN(DXMJ,DXJ)                       ! Save move direction
  230 XX(J)=XX(J)+DXJ                              ! Update XX(J)
      NSH=NSH+1                                    ! Iteration count
C**   +----------------------------------------------------------+    **
C**   | Output intermediate results                              |    **
C**   +----------------------------------------------------------+    **
      CALL OUTMED(XX,DX,X,D,NX,NXXS,NSH,ICASE,KKK)
C**   +----------------------------------------------------------+    **
C**   | Test convergence                                         |    **
C**   +----------------------------------------------------------+    **
      WDIF=WOLD-GWO(NGW)
      WOLD=GWO(NGW)                            ! Save objective function
      IF(NSH.GE.MAXS) GO TO 300                ! Maximum allowed number
      IF(ICASE.NE.0) GO TO 200                 ! Infeasible or unbounded
      IF(DABS(WDIF).LE.0.001*DABS(WOLD)) GO TO 300  ! Objective function
      IF(DXMAX.GT.EPS) GO TO 200               ! Max. movement too small
C**   +----------------------------------------------------------+    **
C**   | Output final results & return                            |    **
C**   +----------------------------------------------------------+    **
  300 CALL FFF(NX,NS,NE,XX,GWO(NGW),GWO)
      CALL OUTFNL(NSH,NX,NS,NE,XX,GWO(NGW),GWO)
      RETURN
      END
************************************************************************
      SUBROUTINE DSM(XX,DX,XXU,XXL,GGU,GWO,GWN,NX,NS,NE,MGW,ICASE,KKK
     *              ,A,LI,LJ,D,X,NIA,NJA,NJX,B,MI,MJ,IS,IC)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION XX(NX),DX(NX),XXU(NX),XXL(NX),GGU(NS)
      DIMENSION GWO(MGW),GWN(MGW)
      DIMENSION A(NIA,NJA),LI(NIA),LJ(NJA),D(NJX),X(NJX)
      DIMENSION B(NIA,NJA),MI(NIA),MJ(NJA)
      DATA SPOS,SNEG/1.0D0,-1.0D0/
C**   ============================================================    **
C**   Double Secant Method for Sequential Linear Programming          **
C**   Setup simplex tableau and call LINEAR for linear programming    **
C**   ------------------------------------------------------------    **
C**   Input   : NX,NS,NE,MGW,EPS,MAXS,NIA,NJA,NJX,IS,IC               **
C**           : XX(NX),DX(NX),XXU(NX),XXL(NX),GGU(NS)                 **
C**   Working : GWO(MGW),GWN(MGW)                                     **
C**           : A,LI,LJ,D,X,B,MI,MJ : For linear programming          **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Set constants                                            |    **
C**   +----------------------------------------------------------+    **
      NGW =NS+NE+1
      NXX =NX+NX
      NXXB=NXX+1
C**   +----------------------------------------------------------+    **
C**   | Compute GWO(1:NG)=G(I), GWO(NG+1)=F_obj at XX(NX)        |    **
C**   | Setup Simplex Tableau : A(1:NG+1,1:NX+NX)    (NG=NS+NE)  |    **
C**   +----------------------------------------------------------+    **
      CALL FFF(NX,NS,NE,XX,GWO(NGW),GWO)
      CALL GRDGW(XX,DX,GWO,GWN,A(1,1),SPOS,NX,NS,NE,MGW,NIA+NIA)
      CALL GRDGW(XX,DX,GWO,GWN,A(1,2),SNEG,NX,NS,NE,MGW,NIA+NIA)
C**   ------------------------------------------------------------    **
      DO 20 J=1,NXX,2            ! \ . Use \./ if this . /
      DO 20 I=1,NS+NE            !   \.  is feasible  ./      \./ secant
      ASUM=A(I,J)+A(I,J+1)       !     +    region   +
      IF(ASUM.GE.0.0) GO TO 20   !      .\         /.      ... nonlinear
      AVAG=0.5*(A(I,J)-A(I,J+1)) !        .\     /.            constrain
      A(I,J)  = AVAG             ! __________\./___________
      A(I,J+1)=-AVAG             ! Use _._ if this is feasible region
   20 CONTINUE                   ! _._ tangent
C**   +----------------------------------------------------------+    **
C**   | Set upper bound D(J) and variable index LJ(J)            |    **
C**   +----------------------------------------------------------+    **
      J=0
      DO 30 JJ=1,NX
      J=J+1
      LJ(J)=J
      D(J)=MIN(DX(JJ),XXU(JJ)-XX(JJ))
      J=J+1
      LJ(J)=J
   30 D(J)=MIN(DX(JJ),XX(JJ)-XXL(JJ))
C**   +----------------------------------------------------------+    **
C**   | Set upper bound D(NXX+I) of slack var. & basic var. index|    **
C**   | Set Simplex tableau : A(1:NG+1,NX+NX+1) = RHS of constra.|    **
C**   +----------------------------------------------------------+    **
      NXXS=NXX
      DO 40 I=1,NGW
      LI(I)=0
      IF(I.GT.NS) GO TO 40
      NXXS=NXXS+1
      LI(I)=NXXS
      D(NXXS)=GGU(I)
   40 A(I,NXXB)=-GWO(I)
C**   +----------------------------------------------------------+    **
C**   | Perform linear programming                               |    **
C**   +----------------------------------------------------------+    **
      CALL LINEAR
     *(A,LI,LJ,D,X,NGW,NXXB,NXXS,ICASE,KKK,NIA,NJA,B,MI,MJ,IS,IC)
      RETURN
      END
************************************************************************
      SUBROUTINE GRDGW(XX,DX,GWO,GWN,DGW,S,NX,NS,NE,MGW,NIA2)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION XX(NX),DX(NX),GWO(MGW),GWN(MGW),DGW(NIA2,NX)
C**   ============================================================    **
C**   Compute gradients by forward or backward difference             **
C**   ------------------------------------------------------------    **
C*I   XX(NX)    = Values of variables                                 **
C*I   DX(NX)    = Delta Values of variables > 0                       **
C*I   GWO(NG+1) = Constra. G(1:NG) & Obj.Fun. F_obj at XX(NX)         **
C*W   GWN(NG+1) = Constra. G(1:NG) & Obj.Fun. F_obj at XX(NX)+DX(J)   **
C*O   DGW(I,J)  = (G_I(XX(J)+DX(J))-G_I(XX(J)))/DX(J) if S=+1         **
C*O   DGW(I,J)  = (G_I(XX(J)-DX(J))-G_I(XX(J)))/DX(J) if S=-1         **
C*I   NX        = Number of original variables    (NG=NS+NE)          **
C*I   NS,NE     = Number of .LE.,.EQ. constraints (NGW=NG+1)          **
C**   ============================================================    **
      DO 40 J=1,NX
C**   +----------------------------------------------------------+    **
C**   | Compute constraint GWN(1:NS+NE)=G(I) and GWN(NGW)=F_obj  |    **
C**   +----------------------------------------------------------+    **
      XXJ=XX(J)
      XX(J)=XX(J)+DX(J)*S
      DXP=DABS(XX(J)-XXJ)
      CALL FFF(NX,NS,NE,XX,GWN(NS+NE+1),GWN)
      XX(J)=XXJ
C**   +----------------------------------------------------------+    **
C**   | Compute DGW(I,J)=dG(I)/dX(J) and DGW(NGW,J)=dF_obj/dX(J) |    **
C**   +----------------------------------------------------------+    **
      DO 40 I=1,NS+NE+1
   40 DGW(I,J)=(GWN(I)-GWO(I))/DXP
      RETURN
      END
************************************************************************
      SUBROUTINE OUTMED(XX,DX,X,D,NX,NXXS,NSH,ICASE,KKK)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION XX(NX),DX(NX),X(NXXS),D(NXXS)
C**   ============================================================    **
C**   Output intermediate results                                     **
C**   ============================================================    **
      WRITE(*,'(1X,29(''-'')/'' NSH,ICASE,KKK ='',3I5)') NSH,ICASE,KKK
      WRITE(*,'(''  D =''/(1X,1P8E10.3))') (D(J),J=1,NXXS)
      WRITE(*,'(''  X =''/(1X,1P8E10.3))') (X(J),J=1,NXXS)
      WRITE(*,'('' DX =''/(1X,1P8E10.3))') (DX(J),J=1,NX)
      WRITE(*,'('' XX =''/(1X,1P8E10.3))') (XX(J),J=1,NX)
      RETURN
      END
************************************************************************
      SUBROUTINE OUTFNL(NSH,NX,NS,NE,XX,WO,GWO)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION XX(NX),GWO(NS+NE)
C**   ============================================================    **
C**   Output final results                                            **
C**   ============================================================    **
      WRITE(*,'(1X,9(''='')/A,I4,A)') ' After',NSH,' Linear Programming'
      WRITE(*,'(A,1PE20.9)') ' The optimum objective function is',WO
      WRITE(*,'(/'' Values of variables ''//(1X,1P8E10.3))') XX
      WRITE(*,'(/'' Values of constraint''//(1X,1P8E10.3))') GWO
      RETURN
      END
************************************************************************
