#include <stdio.h>
#include <ctype.h>
#include <math.h>
int freeii(int    *, FILE *);
int freer8(double *, FILE *);
#define MaxM 5
#define MaxF 4
#define MaxG 9
#define MaxS 15
#define MaxT 1000
#define MaxV (MaxG * MaxF + 2)
double  Omg[MaxM],Ksi[MaxM],Phi[MaxS*MaxM],Inf[MaxS*MaxG];
double  Ag[MaxT*MaxG*MaxF],At[MaxT*MaxS],Ar[MaxT*MaxS],ArM[MaxT*MaxS];
double  Am[MaxT*MaxM],Norm[MaxS];
double  dT = 0.02;
int     NumM,NumF,NumG,NumS,NumT,NumV,Ti,Tf,Mi;
/**********************************************************************/
/*    MAIN PROGRAM  BridgeIdent3D                                     */
/*  ================================================================  */
      main() {
/*  ================================================================  */
      int  InputData(),Optimize(),OutputResult();
/*  ================================================================  */
/*    System Identification analysis for Multiple Supported Inputs    */
/*    of Bridge Structures by Modal Minimization Method ( Beck,J.L.,  */
/*    "Determining Models of Structures from Earthquake Records",     */
/*    CalTech, CA, 1978 )                                             */
/*  ================================================================  */
      InputData();
      Optimize();
      OutputResult();
      exit(0);
}
/**********************************************************************/
      int  InputData() {
/*  ================================================================  */
      int  UserInput(),ReadData(),Initialize();
/*  ================================================================  */
/*    UserInput   computational parameters, initial guess of modal    */
/*                parameters, pseudo influence matrix                 */
/*    ReadData    of recorded ground and structure responses          */
/*    Initialize  recorded and computed responses, normalization c    */
/*  ================================================================  */
      UserInput();
      ReadData();
      Initialize();
      return(0);
}
/**********************************************************************/
      int  UserInput() {
/*  ================================================================  */
/*    READ  NumM,NumF,NumG,NumS,NumT,Mi,Ti,Tf                         */
/*    READ  Omg,Ksi    INITIALIZE  Phi    READ  Inf                   */
/*    ------------------------------------------------------------    */
/*    NumM           = Number of Modes to be analyized                */
/*    NumF           = Number of support Forcing inputs   = 1,2,3     */
/*    NumG           = Number of instrumented Ground support DOFs     */
/*    NumS           = Number of instrumented Superstructure DOFs     */
/*    NumT           = Number of Time steps in input data file        */
/*    Mi             = Initial number of Modes of the analysis        */
/*    Ti,Tf          = Initial and Final Time step of the analysis    */
/*    ............................................................    */
/*    Omg(NumM)      = estimated modal frequencies                    */
/*    Ksi(NumM)      = estimated modal damping coefficients           */
/*    Phi(NumS,NumM) = estimated mode  shapes                  = 1    */
/*    Inf(NumS,NumG) = pseudo Influence matrix                        */
/*  ================================================================  */
      FILE    *fopen(),*fp_r;
      int     Mode,Grnd,Stru,SMode,SGrnd;
/*    double  Omg[MaxM],Ksi[MaxM],Phi[MaxS,MaxM],Inf[MaxS,MaxG];      */
/*    COMMON  NumM,NumF,NumG,NumS,NumT,Ti,Tf,Mi,Omg,Ksi,Phi,Inf;      */
/*  ================================================================  */
      printf("\n Bridge Identification by 'Modal Minimization Method'");
      if((fp_r=fopen("BI3D1.In","r"))==NULL) exit(-1);
/*    +----------------------------------------------------------+    */
/*    |  READ  NumM,NumF,NumG,NumS,NumT,Mi,Ti,Tf                 |    */
/*    +----------------------------------------------------------+    */
      freeii(&NumM,fp_r); freeii(&NumF,fp_r); freeii(&NumG,fp_r);
      freeii(&NumS,fp_r); freeii(&NumT,fp_r);
      freeii(&Mi  ,fp_r); freeii(&Ti  ,fp_r); freeii(&Tf  ,fp_r);
      if (NumM > MaxM) {printf("\n UserInput : NumM > MaxM"); exit(0);}
      if (NumF > MaxF) {printf("\n UserInput : NumF > MaxF"); exit(0);}
      if (NumG > MaxG) {printf("\n UserInput : NumG > MaxG"); exit(0);}
      if (NumS > MaxS) {printf("\n UserInput : NumS > MaxS"); exit(0);}
      if (NumT > MaxT) {printf("\n UserInput : NumT > MaxT"); exit(0);}
      if (Mi < 0 || Mi >= NumM)  Mi = NumM-1;
      if (Ti < 0)                Ti = 0;
      if (Tf >= NumT)            Tf = NumT-1;
      Tf = Ti + (Tf-Ti)/2*2;
/*    ............................................................    */
/*    if((fp_d=fopen("BI3D1.Deb","w"))==NULL) exit(-1);               */
      printf("\n Max M F G S T =%5d%5d%5d%5d%5d"
                                          ,MaxM,MaxF,MaxG,MaxS,MaxT);
      printf("\n Num M F G S T =%5d%5d%5d%5d%5d"
                                          ,NumM,NumF,NumG,NumS,NumT);
      printf("\n     Mi  Ti Tf =%5d%5d%5d",Mi,Ti,Tf);
/*    +----------------------------------------------------------+    */
/*    |  READ  Omg(NumM),Ksi(NumM)                               |    */
/*    +----------------------------------------------------------+    */
      for(Mode=0; Mode<NumM; Mode++) {
          freer8(&Omg[Mode],fp_r); freer8(&Ksi[Mode],fp_r);
          printf("\n Omg Ksi =%8.4lf%8.4lf",Omg[Mode],Ksi[Mode]);
      }
/*    +----------------------------------------------------------+    */
/*    |  INITIALIZE  Phi(NumS,NumM)                              |    */
/*    +----------------------------------------------------------+    */
      for(Mode=0; Mode<NumM; Mode++) { SMode = MaxS*Mode;
        for(Stru=0; Stru<NumS; Stru++) {
          Phi[Stru+SMode] = 1.0;
        }
      }
/*    +----------------------------------------------------------+    */
/*    |  READ  Inf(NumS,NumG)                                    |    */
/*    +----------------------------------------------------------+    */
      for(Grnd=0; Grnd<NumG; Grnd++) { SGrnd = MaxS*Grnd;
          for(Stru=0; Stru<NumS; Stru++) freer8(&Inf[Stru+SGrnd],fp_r);
          printf("\n Inf =");
          for(Stru=0; Stru<NumS; Stru++) {
              if(Stru % 10 == 0) printf("\n");
              printf("%8.4lf",Inf[Stru+SGrnd]);
          }
      }
      fclose(fp_r);
      return(0);
}
/**********************************************************************/
      int  ReadData() {
/*  ================================================================  */
/*    READ  Ag,At                                                     */
/*    ------------------------------------------------------------    */
/*    Ag(NumT,NumG,NumF) = recorded Ground support Acce,velo,displ    */
/*    At(NumT,NumS)      = recorded Total superstructure Accel.       */
/*  ================================================================  */
      FILE    *fopen(),*fp_r;
      int     Forc,Grnd,Stru,Time,GForc,TGrnd,TStru;
/*    double  Ag[MaxT,MaxG,MaxF],At[MaxT,MaxS];                       */
/*    COMMON  NumF,NumG,NumS,NumT,Ag,At;                              */
/*  ================================================================  */
      if((fp_r=fopen("BI3D1.Rec","r"))==NULL) exit(-1);
      free00(fp_r);
/*    +----------------------------------------------------------+    */
/*    |  READ  Ag(NumT,NumG,NumF)                                |    */
/*    +----------------------------------------------------------+    */
      for(Grnd=0; Grnd<NumG; Grnd++) { TGrnd = MaxT*Grnd;
        for(Forc=0; Forc<NumF; Forc++) { GForc = MaxT*MaxG*Forc;
          for(Time=0; Time<NumT; Time++) {
            freer8(&Ag[Time+TGrnd+GForc],fp_r);
          }
        }
      }
/*    +----------------------------------------------------------+    */
/*    |  READ  At(NumT,NumS)                                     |    */
/*    +----------------------------------------------------------+    */
      for(Stru=0; Stru<NumS; Stru++) { TStru = MaxT*Stru;
        for(Time=0; Time<NumT; Time++) {
          freer8(&At[Time+TStru],fp_r);
        }
      }
      fclose(fp_r);
      return(0);
}
/**********************************************************************/
      int  Initialize() {
/*  ================================================================  */
/*    COMPUTE  Ar    INITIALIZE  Am    COMPUTE  Norm                  */
/*    ------------------------------------------------------------    */
/*    Ar(T,NumS) = recorded Relative Acceleration      = At-Inf*Ag    */
/*    Am(T,NumM) = computed Modal relative Acceleration               */
/*    Norm(NumS) = Normalization constant of Ar      = 1/Int(Ar^2)    */
/*  ================================================================  */
      int     Mode,Grnd,Stru,Time,TMode,TGrnd,SGrnd,TStru;
/*    double  Ag[MaxT,MaxG,MaxF],At[MaxT,MaxS],Inf[MaxS,MaxG];        */
/*    double  Ar[MaxT,MaxS],Am[MaxT,MaxM],Norm[MaxS];                 */
      double  InfAg; double Intg();
/*    COMMON  NumM,NumG,NumS,Ti,Tf,Ag,At,Ar,Am,Inf,Norm,dT;           */
/*  ================================================================  */
/*    +----------------------------------------------------------+    */
/*    |  COMPUTE  Ar(T,NumS) = At(T,NumS)                        |    */
/*    |                      - Inf(NumS,NumG)*Ag(T,NumG,1)       |    */
/*    +----------------------------------------------------------+    */
      for(Stru=0; Stru<NumS; Stru++) { TStru = MaxT*Stru;
        for(Time=Ti; Time<=Tf; Time++) {
          InfAg = 0.0;
          for(Grnd=0; Grnd<NumG; Grnd++) { TGrnd = MaxT*Grnd;
                                           SGrnd = MaxS*Grnd;
              InfAg = InfAg + Inf[Stru+SGrnd]*Ag[Time+TGrnd];
          }
          Ar[Time+TStru] = At[Time+TStru] - InfAg;
        }
      }
/*    +----------------------------------------------------------+    */
/*    |  INITIALIZE  Am(T,NumM) = 0                              |    */
/*    +----------------------------------------------------------+    */
      for(Mode=0; Mode<NumM; Mode++) { TMode = MaxT*Mode;
        for(Time=Ti; Time<=Tf; Time++) {
          Am[Time+TMode] = 0.0;
        }
      }
/*    +----------------------------------------------------------+    */
/*    |  COMPUTE  Norm(NumS) = 1/Int(Ar(T,NumS)^2)               |    */
/*    +----------------------------------------------------------+    */
      for(Stru=0; Stru<NumS; Stru++) { TStru = MaxT*Stru;
          Norm[Stru] = 1/Intg ( &Ar[Ti+TStru],&Ar[Ti+TStru],Ti,Tf,dT );
      }
      for(Stru=0; Stru<NumS; Stru++) {
          if(Stru%8 == 0) printf("\n Norm =");
          printf("%10.6lf",Norm[Stru]);
      }
      return(0);
}
/**********************************************************************/
      double  Intg ( double X[],double Y[],int Ti,int Tf,double dT ) {
/*    double  Intg ( X, Y, Ti, Tf, dT )
      double  X[],Y[],dT; int Ti,Tf; {                                */
/*  ================================================================  */
/*    COMPUTE  the numerical integration  Intg  of function  X*Y      */
/*        over the time interval  Ti~Tf  with the fixed step size     */
/*        dT  by using the Simpson's rule ( 1 4 2 4 2 ... 4 1 )       */
/*    !!  (Tf-Ti) must be an even number which is multiple of 2       */
/*  ================================================================  */
      int     Time;
      double  A,B,fun;
/*    double  X[Ti:Tf],Y[Ti:Tf];                                      */
/*  ================================================================  */
      A = 0.0;
      B = 0.0;
      for(Time=1; Time<Tf-Ti; Time+=2) {
          A = A + X[Time]*Y[Time];
          B = B + X[Time+1]*Y[Time+1];
      }
      fun = (X[0]*Y[0]-X[Tf-Ti]*Y[Tf-Ti]+4.0*A+2.0*B) / 3.0 * dT;
      return(fun);
}
/**********************************************************************/
      int  Optimize() {
/*  ================================================================  */
/*    For each Swep of Mode = 1,NumM  :  ComputeArM  MinimizeErr      */
/*    ------------------------------------------------------------    */
/*    Err  = Error function to be minimized                           */
/*    OmgO = Old Omg before MinimizeErr                               */
/*    KsiO = Old Ksi before MinimizeErr                               */
/*    Diff = relative Difference between OmgO,KsiO and Omg,Ksi        */
/*  ================================================================  */
      int     Mode,Swep,Mf;
/*    double  Omg[MaxM],Ksi[MaxM];                                    */
      double  Err,OmgO,KsiO,Diff,tmp1,tmp2;
/*    COMMON  NumM,Mi,Omg,Ksi;                                        */
/*  ================================================================  */
      for(Mf=Mi; Mf<NumM; Mf++) {
        for(Swep=0; Swep<10; Swep++) {
          Diff = 0.0;
          for(Mode=0; Mode<=Mf; Mode++) {
/*    +----------------------------------------------------------+    */
/*    |  STORE  OmgO,KsiO                                        |    */
/*    +----------------------------------------------------------+    */
            OmgO = Omg[Mode];
            KsiO = Ksi[Mode];
/*    +----------------------------------------------------------+    */
/*    |  OPTIMIZE  Omg,Ksi    by    MINIMIZE  Err                |    */
/*    +----------------------------------------------------------+    */
            ComputeArM ( Mode );
            MinimizeErr ( Mode,&Omg[Mode],&Ksi[Mode],&Err );
/*    +----------------------------------------------------------+    */
/*    |  COMPUTE  Diff                                           |    */
/*    +----------------------------------------------------------+    */
            if((tmp1=fabs(OmgO/Omg[Mode]-1.0)) > Diff) Diff = tmp1;
            if((tmp2=fabs(KsiO/Ksi[Mode]-1.0)) > Diff) Diff = tmp2;
/*    ------------------------------------------------------------    */
            printf("\n Omg Ksi Err Diff OmgD KsiD =");
            printf("%10.5lf%10.5lf%10.5lf%10.5lf%10.5lf%10.5lf"
                    ,Omg[Mode],Ksi[Mode],Err,Diff,tmp1,tmp2);
          }
          if (Diff < 0.001)  goto L10;
        }
  L10:;}
      return(0);
}
/**********************************************************************/
      int  ComputeArM ( int M ) {
/*    int  ComputeArM ( M ) int M; {                                  */
/*  ================================================================  */
/*    COMPUTE  ArM                                                    */
/*    ------------------------------------------------------------    */
/*    ArM(T,NumS) = recorded Relative Modal Accel = Ar-Sum(Am*Phi)    */
/*  ================================================================  */
      int     Mode,Stru,Time,TM,SM,TMode,SMode,TStru;
/*    double  Ar[MaxT,MaxS],Am[MaxT,MaxM],Phi[MaxS,MaxM];             */
/*    double  ArM[MaxT,MaxS];                                         */
/*    COMMON  NumM,NumS,Ti,Tf,Ar,Am,ArM,Phi;                          */
/*  ================================================================  */
/*    +----------------------------------------------------------+    */
/*    |  COMPUTE  ArM(T,NumS) = Ar(T,NumS)                       |    */
/*    |                       - Sum(Am(T,NumM)*Phi(NumS,NumM))   |    */
/*    +----------------------------------------------------------+    */
      TM = MaxT*M; SM = MaxS*M;
      for(Stru=0; Stru<NumS; Stru++) { TStru = MaxT*Stru;
        for(Time=Ti; Time<=Tf; Time++) {
          ArM[Time+TStru] = Ar[Time+TStru] + Am[Time+TM]*Phi[Stru+SM];
          for(Mode=0; Mode<NumM; Mode++) { TMode = MaxT*Mode;
                                           SMode = MaxS*Mode;
            ArM[Time+TStru] = ArM[Time+TStru]
                            - Am[Time+TMode]*Phi[Stru+SMode];
          }
        }
      }
      return(0);
}
/**********************************************************************/
      int  MinimizeErr ( int M,double *OmgM,double *KsiM,double *Err ) {
/*    int  MinimizeErr ( M, OmgM, KsiM, Err )
      double *OmgM, *KsiM, *Err;  int M; {                            */
/*  ================================================================  */
/*    MINIMIZE  Err  to find Optimal OmgM,KsiM by using T.W.Lin's     */
/*        nonlinear minimization procedures and  Xmin                 */
/*    ............................................................    */
/*    ComputeErr : COMPUTE  Err  for given values of OmgM and KsiM    */
/*    Xmin       : FIND  Optimum values of OmgM or KsiM               */
/*    ------------------------------------------------------------    */
/*    Err       = Error function to be minimized                      */
/*    OmgM,KsiM = variables of error function to be optimized         */
/*    ------------------------------------------------------------    */
/*    X(1:2) = initial and optimum values of variables OmgM,KsiM      */
/*    F*     = Function values of X                          = Err    */
/*    TypX   = Typical values of X                                    */
/*    TolX   = Tolerance value for X/TypX                             */
/*    TolF   = Tolerance value for dF(X)/dX                           */
/*    dX     = new step size of X              = Max(|Xm-Xi|/TypX)    */
/*           = maximum relative total moving distance                 */
/*    Count  = number of Count over minimizing OmgM and KsiM <= 12    */
/*    Step   = number of Step in minimizing OmgM or KsiM     <= 16    */
/*  ================================================================  */
      int     I,Count,Step,Valley;
      double  X[2],TypX[2],TolX,TolF,dX,dXOld; double xmin();
      double  Xo,Fo,Xx,Fx,Xm,Fm,Xn,Xi,Rate,Buf[14];
/*  ================================================================  */
/*    +----------------------------------------------------------+    */
/*    |  INITIALIZE  X(2),TypX(2),TolX,dX                        |    */
/*    +----------------------------------------------------------+    */
      X[0]    = *OmgM;
      X[1]    = *KsiM;
      TypX[0] = 5.0;
      TypX[1] = 0.05;
      TolX    = 0.0001;
      dX      = 0.1;
/*    +----------------------------------------------------------+    */
/*    |  ITERATE  minimizing OmgM and KsiM up to Count=12 times  |    */
/*    +----------------------------------------------------------+    */
      Count = 0;
 L95: Count = Count + 1;
      dXOld = dX;
      dX    = 0.0;
      for(I=0; I<2; I++) {
/*    +----------------------------------------------------------+    */
/*    |  INITIALIZE  current Xx,Fx , optimum Xm,Fm , next Xn     |    */
/*    |  SAVE        initial Xi                                  |    */
/*    +----------------------------------------------------------+    */
          Xx = 1.e30;
          Fx = 1.e30;
          Xm = Xx;
          Fm = Fx;
          Xn = X[I];
          Xi = X[I];
/*    +----------------------------------------------------------+    */
/*    |  ITERATE  minimizing OmgM or KsiM up to Step=16 times    |    */
/*    +----------------------------------------------------------+    */
          Step = 0;
 L85:     Step = Step + 1;
/*    +----------------------------------------------------------+    */
/*    |  SAVE  previous Xo,Fo                                    |    */
/*    +----------------------------------------------------------+    */
          Xo = Xx;
          Fo = Fx;
/*    +----------------------------------------------------------+    */
/*    |  USE  next Xn from Xmin as current Xx,X for ComputeErr   |    */
/*    +----------------------------------------------------------+    */
          Xx   = Xn;
          X[I] = Xx;
          ComputeErr ( M,X[0],X[1],&Fx );
/*    +----------------------------------------------------------+    */
/*    |  SAVE  optimum Xm,Fm during iteration Steps              |    */
/*    +----------------------------------------------------------+    */
          if (Fx < Fm)  {
              Xm = Xx;
              Fm = Fx;
          }
/*    +----------------------------------------------------------+    */
/*    |  OBTAIN  next Xn from Xmin for next iteration Step       |    */
/*    +----------------------------------------------------------+    */
          Xn = xmin ( Xx,Fx,dXOld*TypX[I],Step,&Valley,Buf );
/*    +----------------------------------------------------------+    */
/*    |  CHECK  convergence of X,F                               |    */
/*    +----------------------------------------------------------+    */
          if (Step == 2)  TolF = TolX * fabs((Fx-Fo)/(Xx-Xo));
          if (Step < 5  &&  Valley == 0)  goto L85;
          if (Step < 16  &&
              fabs(Xx-Xo) > TolX*TypX[I]  &&
              fabs(Fx-Fo) > TolF*fabs(Xx-Xo))  goto L85;
/*    +----------------------------------------------------------+    */
/*    |  UPDATE  X , dX = Max(|Xm-Xi|/TypX)                      |    */
/*    +----------------------------------------------------------+    */
          X[I] = Xm;
          Rate = 0.5 * fabs(Xm-Xi)/TypX[I];
          if (Rate > dX)  dX = Rate;
          printf("\n %4d%7.4lf",Step,Rate);
      }
/*    +----------------------------------------------------------+    */
/*    |  CHECK  convergence of OmgM or KsiM                      |    */
/*    +----------------------------------------------------------+    */
      if (dX > dXOld)  dX = dXOld;
      if (dX > TolX && Count < 12)  goto L95;
/*    +----------------------------------------------------------+    */
/*    |  return  the minimum Err and the optimum OmgM,KsiM       |    */
/*    +----------------------------------------------------------+    */
      *OmgM = X[0];
      *KsiM = X[1];
      ComputeErr ( M,*OmgM,*KsiM,Err );
      return(0);
}
/**********************************************************************/
      int  ComputeErr ( int M,double OmgM,double KsiM,double *pErr ) {
/*    int  ComputeErr ( M, OmgM, KsiM, pErr )
      double OmgM, KsiM, *pErr; int M; {                              */
/*  ================================================================  */
/*    COMPUTE  AmM,PhiM,Err                                           */
/*    ------------------------------------------------------------    */
/*    AmM(T)     = computed Modal Acceleration of the Mth mode        */
/*    PhiM(NumS) = computed mode shapes of the Mth mode               */
/*    Err        = Error function = Sum(Norm*Int((ArM-AmM*PhiM)^2))   */
/*    PF         = Penalty Function to Err for computed OmgM,KsiM     */
/*  ================================================================  */
      int     Stru,Time,TM,SM,TStru;
/*    double  ArM[MaxT,MaxS],Am[MaxT,MaxM],Phi[MaxS,MaxM],Norm[MaxS]; */
      double  Err,Diff[MaxT]; double Intg(),pf();
/*    COMMON  NumS,Ti,Tf,ArM,Am,Phi,Norm,dT;                          */
/*  ================================================================  */
/*    +----------------------------------------------------------+    */
/*    |  COMPUTE  AmM(T),PhiM(NumS)                              |    */
/*    +----------------------------------------------------------+    */
       TM = MaxT*M; SM = MaxS*M;
       ComputeAmM ( OmgM,KsiM,&Phi[SM],&Am[TM] );
/*    +----------------------------------------------------------+    */
/*    |  COMPUTE  Err = Sum ( Norm(NumS) * Int ( (ArM(T,NumS)    |    */
/*    |               - AmM(T)*PhiM(NumS))^2 ) )                 |    */
/*    +----------------------------------------------------------+    */
      Err = 0.0;
      for(Stru=0; Stru<NumS; Stru++) { TStru = MaxT*Stru;
          for(Time=Ti; Time<=Tf; Time++) {
              Diff[Time] = ArM[Time+TStru] - Am[Time+TM] * Phi[Stru+SM];
          }
          Err = Err + Norm[Stru] * Intg (&Diff[Ti],&Diff[Ti],Ti,Tf,dT);
      }
/*    +----------------------------------------------------------+    */
/*    |  ADD  PF to Err for OmgM<0 and KsiM<0 and KsiM>0.5       |    */
/*    +----------------------------------------------------------+    */
      Err = Err + pf(    OmgM,100.0,0.001,1.0,1.0,1);
      Err = Err + pf(    KsiM,100.0,0.001,1.0,1.0,1);
      Err = Err + pf(0.5-KsiM,100.0,0.001,1.0,1.0,1);
      printf("\n OmgM,KsiM,Err =%10.5lf%10.5lf%7.3lf",OmgM,KsiM,Err);
      *pErr = Err;
      return(0);
}
/**********************************************************************/
      int  ComputeAmM ( double OmgM,double KsiM
                                   ,double PhiM[],double AmM[] ) {
/*    int  ComputeAmM ( OmgM, KsiM, PhiM, AmM )
      double OmgM, KsiM, PhiM[], AmM[]; {                             */
/*  ================================================================  */
/*    COMPUTE  AmM,PhiM  by iteration of linear modal minimization    */
/*        for linear parameter PhiM and participation factor B        */
/*    ------------------------------------------------------------    */
/*    COMPUTE S,Sjk,SjA << COMPUTE RHS , SOLVE B , COMPUTE BSB,BSA    */
/*        , COMPUTE PhiM , CHECK convergence >> NORMALIZE PhiM,B ,    */
/*        COMPUTE AmM                                                 */
/*    ------------------------------------------------------------    */
/*    ArM(T,NumS) = recorded Relative Modal Acceleration              */
/*    AmM(T)      = computed Modal Acceleration        = Sum(B*S)     */
/*    OmgM,KsiM   = Modal frequency & Modal damping ratio             */
/*    PhiM(NumS)  = Mode shapes = Int(SumBS*ArM)/Int(SumBS*SumBS)     */
/*    Norm(NumS)  = Normalization constant of Ar                      */
/*    ------------------------------------------------------------    */
/*    B(NumV)        = participation factor = soln of Sjk*B = RHS     */
/*    S(T,NumV)      = unit accel response of SDOF linear oscil.      */
/*    Sjk(NumV,NumV) = Int(Sj*Sk)                                     */
/*    SjA(NumV,NumS) = Int(Sj*ArM)                                    */
/*    RHS(NumV)      = Right Hand Side of linear equations            */
/*                   = Sum(Norm*PhiM*SjA)/Sum(Norm*PhiM^2)            */
/*    BSB            = Bj*Sjk*Bk = Int(Sum(B*S)*Sum(B*S))             */
/*    BSA            = Bj*SjA = Int(Sum(B*S)*ArM)                     */
/*    NumV           = Number of Variables of system = NumG*NumF+2    */
/*  ================================================================  */
      int     Stru,Time,J,K,Count,TJ,VJ,TK,VK,TStru,VStru;
/*    double  Ag[MaxT,MaxG,MaxF],ArM[MaxT,MaxS],AmM[MaxT];            */
/*    double  OmgM,KsiM,PhiM[MaxS],Norm[MaxS];                        */
      double  B[MaxV],S[MaxT*MaxV],Sjk[MaxV*MaxV],SjA[MaxV*MaxS];
      double  RHS[MaxV],BSB,BSA,Sum,PhiMax,PhiOld; double Intg();
/*    COMMON  NumS,Ti,Tf,NumV,Ag,ArM,Norm,dT;                         */
/*  ================================================================  */
/*    +----------------------------------------------------------+    */
/*    |  COMPUTE  S(T,NumV)                                      |    */
/*    +----------------------------------------------------------+    */
       ComputeS ( OmgM,KsiM,Ag,S );
/*    +----------------------------------------------------------+    */
/*    |  COMPUTE  Sjk = Skj = Int(Sj*Sk) , SjA = Int(Sj*ArM)     |    */
/*    +----------------------------------------------------------+    */
      for(J=0; J<NumV; J++) { TJ = MaxT*J; VJ = MaxV*J;
        for(K=0; K<=J; K++) { TK = MaxT*K; VK = MaxV*K;
          Sjk[J+VK] = Intg ( &S[Ti+TJ],&S[Ti+TK],Ti,Tf,dT );
          Sjk[K+VJ] = Sjk[J+VK];
        }
      }
      for(J=0; J<NumV; J++) { TJ = MaxT*J;
        for(Stru=0; Stru<NumS; Stru++) { TStru = MaxT*Stru;
                                         VStru = MaxV*Stru;
          SjA[J+VStru] = Intg ( &S[Ti+TJ],&ArM[Ti+TStru],Ti,Tf,dT );
        }
      }
/*    ------------------------------------------------------------    */
      Count = 0;
 L95 :Count = Count + 1;
/*    +----------------------------------------------------------+    */
/*    |  COMPUTE  RHS = Sum(Norm*PhiM*SjA) / Sum(Norm*PhiM^2)    |    */
/*    +----------------------------------------------------------+    */
      Sum = 0.0;
      for(Stru=0; Stru<NumS; Stru++) {
          Sum = Sum + Norm[Stru] * PhiM[Stru]*PhiM[Stru];
      }
      for(J=0; J<NumV; J++) {
          RHS[J] = 0.0;
          for(Stru=0; Stru<NumS; Stru++) { VStru = MaxV*Stru;
              RHS[J] = RHS[J] + Norm[Stru] * PhiM[Stru] * SjA[J+VStru];
          }
          RHS[J] = RHS[J] / Sum;
      }
/*    +----------------------------------------------------------+    */
/*    |  SOLVE  B from linear equations  Sjk * B = RHS           |    */
/*    +----------------------------------------------------------+    */
      SolveSystem ( Sjk,RHS,B,NumV );
/*    +----------------------------------------------------------+    */
/*    |  COMPUTE  BSB = Bj*Sjk*Bk = Int(Sum(B*S)*Sum(B*S))       |    */
/*    +----------------------------------------------------------+    */
      BSB = 0.0;
      for(J=0; J<NumV; J++) {
        for(K=0; K<NumV; K++) { VK = MaxV*K;
          BSB = BSB + B[J] * Sjk[J+VK] * B[K];
        }
      }
/*    ------------------------------------------------------------    */
      PhiMax = 0.0;
      Sum = 0.0;
      for(Stru=0; Stru<NumS; Stru++) { VStru = MaxV*Stru;
          PhiOld = PhiM[Stru];
/*    +----------------------------------------------------------+    */
/*    |  COMPUTE  BSA = Bj*SjA = Int(Sum(B*S)*ArM)               |    */
/*    +----------------------------------------------------------+    */
          BSA = 0.0;
          for(J=0; J<NumV; J++) {
              BSA = BSA + B[J] * SjA[J+VStru];
          }
/*    +----------------------------------------------------------+    */
/*    |  COMPUTE  PhiM = BSA / BSB                               |    */
/*    +----------------------------------------------------------+    */
          PhiM[Stru] = BSA / BSB;
          if (fabs(PhiM[Stru]) > PhiMax)  PhiMax = fabs(PhiM[Stru]);
          Sum = Sum + fabs(PhiM[Stru]-PhiOld);
      }
/*    +----------------------------------------------------------+    */
/*    |  CHECK  convergence of PhiM                              |    */
/*    +----------------------------------------------------------+    */
      if (Sum > PhiMax*NumS*0.001 && Count < 10)  goto L95;
/*    +----------------------------------------------------------+    */
/*    |  NORMALIZE  PhiM,B                                       |    */
/*    +----------------------------------------------------------+    */
      for(Stru=0; Stru<NumS; Stru++) {
          PhiM[Stru] = PhiM[Stru] / PhiMax;
      }
      for(J=0; J<NumV; J++) {
          B[J] = B[J] * PhiMax;
      }
/*     WRITE(*,'('' PhiM='',10F8.4)')  ( PhiM(Stru) , Stru=1,NumS )   */
/*     WRITE(*,'('' B='',10F8.4)')  ( B(J) , J=1,NumV )               */
/*    +----------------------------------------------------------+    */
/*    |  COMPUTE  AmM = Sum(B*S)                                 |    */
/*    +----------------------------------------------------------+    */
      for(Time=Ti; Time<=Tf; Time++) {
          AmM[Time] = 0.0;
          for(J=0; J<NumV; J++) { TJ = MaxT*J;
              AmM[Time] = AmM[Time] + B[J] * S[Time+TJ];
          }
      }
      return(0);
}
/**********************************************************************/
      int ComputeS ( double Omg,double Ksi,double G[],double S[] ) {
/*    int ComputeS ( Omg, Ksi, G, S ) double Omg, Ksi, G[], S[]; {    */
/*  ================================================================  */
/*    COMPUTE  unit acceleration response S(T,NumV) of SDOF linear    */
/*        oscillator  S + 2*Ksi*Omg*V + Omg^2*X = - G  by using       */
/*        Duhamel's integral over the time interval Ti~Tf ( Beck,     */
/*        J.L. & Dowling,M.J. , " Quick Algorithms for Computing      */
/*        Either Displacement, Velocity or Acceleration of an         */
/*        Oscillator " , Earthquake Engineering and Structural        */
/*        Dynamics, Vol.16, P.245-253, 1988 )                         */
/*    ------------------------------------------------------------    */
/*    S(T,1:NumV-2) = response of 'forced vibration' due to ground    */
/*                    accel,veloc,displ with X(To) = V(To) = 0        */
/*    S(T,NumV-1)   = response of 'free vibration' with X(To) = 1     */
/*    S(T,NumV)     = response of 'free vibration' with V(To) = 1     */
/*    ............................................................    */
/*    Omg,Ksi        = natural frequency & damping ratio of oscil.    */
/*    G(T,NumG,NumF) = recorded Ground support Accel,Veloc,Displ      */
/*                   = Ag(T,NumG), Vg(T,NumG), Xg(T,NumG)             */
/*    NumV           = Number of response Variables  = NumG*NumF+2    */
/*  ================================================================  */
      int     F,I,T,V,VV,VV1,VV2,FF,II_FF;
/*    double  G[MaxT,MaxG,MaxF],S[MaxT,MaxV];                         */
      double  A1,A2,A3,A4,B1,B2,B3,OmgT,KsiT,SKsiT,e,eSin,eCos;
/*    COMMON  NumF,NumG,Ti,Tf,NumV,dT;                                */
/*  ================================================================  */
/*    +----------------------------------------------------------+    */
/*    |  COMPUTE  the required repeat constants                  |    */
/*    +----------------------------------------------------------+    */
      OmgT = Omg * dT;
      KsiT = Ksi * OmgT;
      SKsiT = sqrt(1-Ksi*Ksi) * OmgT;
      e = exp(-KsiT);
      eSin = e * sin(SKsiT) / SKsiT;
      eCos = e * cos(SKsiT);
/*    +----------------------------------------------------------+    */
/*    |  COMPUTE  the time independent coefficients As & Bs      |    */
/*    +----------------------------------------------------------+    */
      A1 = 2.0 * eCos;
      A2 = - e*e;
      A3 = Omg*Omg * ( KsiT*eSin - eCos );
      A4 = - Omg * ( (1.0-2.0*Ksi*Ksi)*OmgT*eSin + 2.0*Ksi*eCos );
      B1 = eSin;
      B2 = eCos - (1.0+KsiT)*eSin;
      B3 = 1.0;
/*    +----------------------------------------------------------+    */
/*    |  COMPUTE  responses of forced vibration  S(T,1:NumV-2)   |    */
/*    +----------------------------------------------------------+    */
      V = 0;
      for(F=0; F<NumF; F++) { FF = MaxT*MaxG*F;
        for(I=0; I<NumG; I++) { II_FF = MaxT*I+FF; VV = MaxT*V;
          V = V + 1;
          S[Ti  +VV] = - B3*G[Ti  +II_FF];
          S[Ti+1+VV] = - B1*G[Ti+1+II_FF] - B2*G[Ti+II_FF];
          for(T=Ti+2; T<=Tf; T++) {
            S[T+VV] =                 A1*S[T-1+VV] + A2*S[T-2+VV]
                    - B1*(G[T+II_FF] - 2*G[T-1+II_FF] + G[T-2+II_FF]);
          }
        }
/*    +----------------------------------------------------------+    */
/*    |  UPDATE  Bs  from G=Ag to G=2*Ksi*Omg*Vg and G=Omg^2*Xg  |    */
/*    +----------------------------------------------------------+    */
        B1 = B1 * Omg;
        B2 = B2 * Omg;
        B3 = B3 * Omg;
      }
/*    +----------------------------------------------------------+    */
/*    |  COMPUTE  responses of free vibration  S(T,NumV-1:NumV)  |    */
/*    +----------------------------------------------------------+    */
      VV1 = VV+MaxT; VV2 = VV1+MaxT;
      S[Ti  +VV1] = - Omg*Omg;
      S[Ti  +VV2] = - 2.0*Ksi*Omg;
      S[Ti+1+VV1] = A3;
      S[Ti+1+VV2] = A4;
      for(T=Ti+2; T<=Tf; T++) {
          S[T+VV1] = A1*S[T-1+VV1] + A2*S[T-2+VV1];
          S[T+VV2] = A1*S[T-1+VV2] + A2*S[T-2+VV2];
      }
      NumV = V + 2;
      return(0);
}
/**********************************************************************/
      int  SolveSystem ( double A[],double B[],double X[],int N ) {
/*    int  SolveSystem ( A, B, X, N ) double A[], B[], X[]; int N; {  */
/*  ================================================================  */
/*    SOLVE  system of N linear equations  A(N,N)*X(N) = B(N)  of X   */
/*    ............................................................    */
/*    LDLTDec : Lower-Diagonal-Lower-Transpose Decomposition for A    */
/*              A = LDLT    AX = B    ->    LDLTX = B                 */
/*    FDBSub  : Forward-Diagonal-Backward Substitution to solve X     */
/*              LTX = Y    DY = Z    LZ = B                           */
/*    ------------------------------------------------------------    */
/*    A    : must be a symmetric positive definite square matrix      */
/*    MaxV = Maximum dimension of matrix A                    >= N    */
/*  ================================================================  */
/*    double  A[MaxV,MaxV],B[MaxV],X[MaxV];                           */
      double  L[MaxV*MaxV],D[MaxV];
      int     I,J,VJ; int cbdecp(),cbsolx();
/*  ================================================================  */
      for(J=0; J<N; J++) { VJ = MaxV*J;
        X[J] = B[J];
        for(I=0; I<=J; I++) {
          L[I+VJ] = A[I+VJ];
        }
      }
      cbdecp (L,N,N,MaxV);
      cbsolx (L,X,N,N,MaxV);
      return(0);
}
/**********************************************************************/
      int  OutputResult() {
/*  ================================================================  */
/*    WRITE  Ar,Ac    WRITE  Omg,Ksi,Phi                              */
/*    ------------------------------------------------------------    */
/*    Ar(T,NumS)     = Recorded relative Acceleration                 */
/*    Ac(T,NumS)     = Computed relative Acceleration = Sum(Am*Phi)   */
/*    Omg(NumM)      = computed optimal modal frequencies             */
/*    Ksi(NumM)      = computed optimal modal damping coefficients    */
/*    Phi(NumS,NumM) = computed optimal mode shapes                   */
/*  ================================================================  */
      FILE    *fopen(),*fp_w;
      int     Mode,Stru,Time,TMode,SMode,TStru;
      double  Ac;
/*    double  Ar[MaxT,MaxS],Am[MaxT,MaxM];                            */
/*    double  Omg[MaxM],Ksi[MaxM],Phi[MaxS,MaxM];                     */
/*    COMMON  NumM,NumS,Ti,Tf,Ar,Am,Omg,Ksi,Phi;                      */
/*  ================================================================  */
/*    +----------------------------------------------------------+    */
/*    |  WRITE  Ar(T,NumS)                                       |    */
/*    |         Ac(T,NumS) = Sum(Am(T,NumM)*Phi(NumS,NumM))      |    */
/*    +----------------------------------------------------------+    */
      if((fp_w=fopen("BI3D1.Res","w"))==NULL) exit(-1);
      for(Stru=0; Stru<NumS; Stru++) { TStru = MaxT*Stru;
        for(Time=Ti; Time<=Tf; Time++) {
          Ac = 0.0;
          for(Mode=0; Mode<NumM; Mode++) { TMode = MaxT*Mode;
                                           SMode = MaxS*Mode;
              Ac = Ac + Am[Time+TMode]*Phi[Stru+SMode];
          }
          fprintf(fp_w,"\n%5d%5d%12.3le%12.3le"
                      ,Stru,Time,Ar[Time+TStru],Ac);
        }
      }
      fclose(fp_w);
/*    +----------------------------------------------------------+    */
/*    |  WRITE  Omg(NumM),Ksi(NumM),Phi(NumS,NumM)               |    */
/*    +----------------------------------------------------------+    */
      if((fp_w=fopen("BI3D1.Out","w"))==NULL) exit(-1);
      for(Mode=0; Mode<NumM; Mode++) { SMode = MaxS*Mode;
        fprintf(fp_w,"\n Omg[%d] =%12.6lf   ",Mode,Omg[Mode]);
        fprintf(fp_w,"   Ksi[%d] =%12.6lf   ",Mode,Ksi[Mode]);
        for(Stru=0; Stru<NumS; Stru++) {
            fprintf(fp_w,"\n Phi[%2d,%d] =%12.6lf"
                   ,Stru,Mode,Phi[Stru+SMode]);
        }
      }
      fclose(fp_w);
      return(0);
}
/**********************************************************************/
